#include "MaterialLib.h"

#include <fstream>

ostream &operator<<( ostream &os, const Color &c) {
  os << c.r << ", " << c.g << ", " << c.b << ", " << c.a << endl;
  return os;
}

ostream &operator<<( ostream &os, const Material &m) {
  os << "Material: " << m.name << endl;
  os << " Ka " << m.ka << endl;
  os << " Kd " << m.kd << endl;
  os << " Ks " << m.ks << endl;
  os << " Shininess " << m.shininess << endl;
  return os;
}



MaterialLib::MaterialLib() {
  // afegeix un material per defecte a la posicio 0
  dict[ "material per defecte"] = 0;
  materials.push_back( Material( "material per defecte"));
  // afegeix un segon material ( vermell) a la posicio 1 
  dict[ "material vermell"] = 1;
  materials.push_back( Material( "material vermell", Color( 0.4f, 0.0, 0.0),Color( 0.8f, 0.0, 0.0))); 
  
}

void MaterialLib::readMtl( const char *filename) {
  ifstream f( filename); 	
  
  if ( f.fail()) {
#ifdef DEBUG
    cerr << "No puc obrir el fitxer de materials <" << filename << ">" << endl;
#endif
  } else {
    Material *mat = NULL; // apuntador al material que estem llegint
    string cad;
    f >> cad;
    while ( !f.eof()) {
      if ( cad == "newmtl") {
        // crear nou material
        string name;
        f >> name;
        dict[ name] = ( int)materials.size();
        materials.push_back( Material( name));
        mat = &materials[ materials.size()-1];
      } else if ( cad == "Ka") {
        f >> mat->ka.r >> mat->ka.g >> mat->ka.b; 
      } else if ( cad == "Kd") {
        f >> mat->kd.r >> mat->kd.g >> mat->kd.b; 
      }	else if ( cad == "Ks") {
        f >> mat->ks.r >> mat->ks.g >> mat->ks.b; 
      } else if ( cad == "d") {
        float alfa;
        f >> alfa;
        mat->ka.a = mat->kd.a = mat->ks.a = alfa;
      }	else if ( cad == "Ns") {
        f >> mat->shininess;
      }
      
      f >> cad;
    }
  }
}

const Material &MaterialLib::material( int index) const {
  if ( index >= 0 && index < int( materials.size()))
    return materials[ index];
  else return materials[ 0];
}

ostream &operator<<( ostream &os, const MaterialLib &m) {
  os << "Material Lib: " << endl;
  for ( unsigned int i = 0; i<m.materials.size(); i++)  os << m.materials[ i] << endl;
  return os;
}

