#include "Object.h"
#include <cmath>
#include <string>
#include <iostream>
#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

Object::Object( std::string n): name( n), 
                                _numVertices( 0), _numFaces( 0),
                                _lstVertices( NULL), _lstNormales( NULL),
                                _lstIndicesTriangs( NULL), _lstIndicesQuads( NULL),
                                _numTriangs( 0), _numQuads( 0),
				_err_str( NULL)
{}

Object::~Object(){
  deleteVALists(); // borramos las posibles listas
  set_err_str( NULL);
}

void Object::set_err_str( const char *str) {
  if ( _err_str) free( _err_str);
  _err_str = NULL;
  if ( str && *str) _err_str = strdup( str);
}

Box Object::boundingBox() const {
  return _boundingBox;
}

void Object::updateBoundingBox() {
  if ( vertices.size()) {
    Point p = vertices[ 0].coord;
    _boundingBox=Box( p, p);
    for ( unsigned int i=1; i<vertices.size(); i++)
      _boundingBox.update( vertices[ i].coord);
  }
}

void Object::updateNormals() {
  for( unsigned int i=0; i<faces.size(); ++i)
    faces[ i].updateNormal( vertices);
  
  // ahora, calculamos las normales suavizadas:
  for ( unsigned int i = 0; i < vertices.size(); i++) {
    vertices[ i].normal.x = 0.0;
    vertices[ i].normal.y = 0.0;
    vertices[ i].normal.z = 0.0;
  }
  for ( unsigned int i = 0; i < faces.size(); i++) {
    for ( unsigned int j = 0; j < faces[ i].vertices.size(); j++) {
      int idx = faces[ i].vertices[ j];
      vertices[ idx].normal.x += faces[ i].normal.x;
      vertices[ idx].normal.y += faces[ i].normal.y;
      vertices[ idx].normal.z += faces[ i].normal.z;
    }
  }
  for ( unsigned int i = 0; i < vertices.size(); i++) {
    vertices[ i].normal.normalize();
  }
}


/*
Lectura d'un fitxer OBJ

Cont fragments de codi de obj_2_ply.c ( C) Greg Turk

-----------------------------------------------------------------------
Copyright ( c) 1998 Georgia Institute of Technology.  All rights reserved.   

Permission to use, copy, modify and distribute this software and its   
documentation for any purpose is hereby granted without fee, provided   
that the above copyright notice and this permission notice appear in   
all copies of this software and that you do not sell the software.   
*/


/* for file reading */
static char **words;
static int max_words = 0;
static int num_words = 0;
#define BIG_STRING 4096
static char str[ BIG_STRING];
static char str_orig[ BIG_STRING];

void get_indices( char *word, int *vindex, int *tindex, int *nindex ) {
  /******************************************************************************
   * GET_INDICES breaks up a word that may have slash-separated numbers into one or more
   * numbers.
   * 
   * Entry:
   * word - word to break up
   * 
   * Exit:
   * vindex - first number ( vertex index)
   * tindex - second number ( texture index)
   * nindex - third number ( normal vector index)
   ******************************************************************************/
  char *null = ( char *)" ";
  char *ptr;
  char *tp;
  char *np;
  
  /* by default, the texture and normal pointers are set to the null string */

  tp = null;
  np = null;

  /* replace slashes with null characters and cause tp and np to point */
  /* to character immediately following the first or second slash */

  for ( ptr = word; *ptr != '\0'; ptr++) {
    if ( *ptr == '/') {
      if ( tp == null)
        tp = ptr + 1;
      else
        np = ptr + 1;

      *ptr = '\0';
    }
  }

  *vindex = atoi( word);
  *tindex = atoi( tp);
  *nindex = atoi( np);
}

void Object::make_face( char **words, int nwords, int currentMaterial ) {
  FaceObject face;
  for ( int i = 0; i < nwords; i++) {
    int vindex;
    int nindex;
    int tindex;

    if ( ( words[ i][ 0]>='0')&&( words[ i][ 0]<='9')) {
      get_indices( words[ i], &vindex, &tindex, &nindex);

#if 0
      printf( "vtn: %d %d %d\n", vindex, tindex, nindex);
#endif

      /* store the vertex index */

      if ( vindex > 0)       /* indices are from one, not zero */
        face.vertices.push_back( vindex - 1);
      else if ( vindex < 0)  /* negative indices mean count backwards */
        face.vertices.push_back( ( int)vertices.size() + vindex);
      else {
#ifdef DEBUG
        fprintf( stderr, "Zero indices not allowed: '%s'\n", str_orig);
        // exit( -1);
#endif
	return;
      }

      /*
        if ( ( tindex != 0 || nindex != 0) && warning == 0) {
        fprintf ( stderr, "\n");
        fprintf ( stderr, "Warning: textures and normals currently ignored.\n");
        fprintf ( stderr, "\n");
        warning = 1;
        }
      */
    }
  }
  face.material = currentMaterial;

  faces.push_back( face);
}

char *fetch_line( FILE *fp ) {
  //int i,j;
  char *ptr;
  char *ptr2;
  char *result;
  //char *comment_ptr;

  /* read in a line */
  result = fgets( str, BIG_STRING, fp);

  /* return NULL if we're at the end-of-file */
  if ( result == NULL)
    return ( ( char *) -1);

  /* convert line-feed and tabs into spaces */
  /* ( this guarentees that there will be a space before the */
  /*  null character at the end of the string) */

  str[ BIG_STRING-2] = ' ';
  str[ BIG_STRING-1] = '\0';

  for ( ptr = str; *ptr != '\0'; ptr++) {
    if ( *ptr == '\t') {
      *ptr = ' ';
    }
    else if ( *ptr == '\n') {
      *ptr = ' ';
      break;
    }
  }

  /* copy the line */
  for ( ptr = str, ptr2 = str_orig; *ptr != '\0'; ptr++, ptr2++)
    *ptr2 = *ptr;
  *ptr2 = '\0';

  /* look to see if this is a comment line ( first non-space is '#') */

  for ( ptr = str; *ptr != '\0'; ptr++) {
    if ( *ptr == '#') {
      ptr++;
      while ( *ptr == ' ')
        ptr++;
      return ( ptr);
    }
    else if ( *ptr != ' ') {
      break;
    }
  }

  /* if we get here, we've got a non-comment line */

  /* strip off trailing comments */

  while ( *ptr != '\0') {
    if ( *ptr == '#') {
      *ptr++ = ' ';
      *ptr = '\0';
      break;
    }
    ptr++;
  }

  return ( NULL);
}

int fetch_words( void ) {
  char *ptr;

  /* allocate room for words if necessary */
  if ( max_words == 0) {
    max_words = 20;
    words = ( char **) malloc ( sizeof( char *) * max_words);
  }

  /* find the words in the line */

  ptr = str;
  num_words = 0;

  while ( *ptr != '\0') {
    /* jump over leading spaces */
    while ( *ptr == ' ')
      ptr++;

    /* break if we reach the end */
    if ( *ptr == '\0')
      break;

    /* allocate more room for words if necessary */
    if ( num_words >= max_words) {
      max_words += 10;
      words = ( char **) realloc( words, sizeof( char *) * max_words);
    }

    /* save pointer to beginning of word */
    words[ num_words++] = ptr;

    /* jump over non-spaces */
    while ( *ptr != ' ')
      ptr++;

    /* place a null character here to mark the end of the word */
    *ptr++ = '\0';
  }

  /* return the number of words */
  return ( num_words);
}

string getPath( const string &filename) {
  int i=( int)filename.length()-1;
  bool found = false;
  while ( i>=0 && !found) {
    if ( filename[ i]=='/' || filename[ i]=='\\') found=true;
    else --i;
  }
  return filename.substr( 0, i+1);
}

// llegeix tots els fitxers .mtl i els afegeix a matlib
void read_mtllib( char **words, int nwords, MaterialLib &matlib, 
                  const string &filename ) {
  string path = getPath( filename);
  for ( int i = 0; i < nwords; i++) {
    int size = ( int)strlen( words[ i]) - 1;
    while ( size && ( words[ i][ size]=='\n' || words[ i][ size]=='\r') ) words[ i][ size--]=0;
    matlib.readMtl( ( path+words[ i]).c_str());
  }
}

// Llegeix un fitxer .obj
//  Si el fitxer referencia fitxers de materials ( .mtl), tambe es llegeixen. 
//  Tots els elements del fitxer es llegeixen com a un unic objecte.
//  

void Object::readObj( const char *filename, MaterialLib &matlib) {
  set_err_str( NULL);
  int currentMaterial = -1;  
  char buf[ 8192];

  FILE *fp = fopen( filename,"rb");
  if ( !fp) {
#ifdef DEBUG
    cout << "No puc obrir el fitxer " << filename << endl;
#endif
    sprintf( buf, "Could not open file '%s'", filename);
    set_err_str( buf);
  } else {
    clear();
    while ( true) {
      char *comment_ptr = fetch_line( fp);

      if ( comment_ptr == ( char *) -1)  /* end-of-file */
        break;

      /* did we get a comment? */
      if ( comment_ptr) {
        //make_comment ( comment_ptr);
        continue;
      }

      /* if we get here, the line was not a comment */
      int nwords = fetch_words();

      /* skip empty lines */
      if ( nwords == 0)
        continue;

      char *first_word = words[ 0];

      if ( !strcmp( first_word, "v")) {
        if ( nwords < 4) {
	  sprintf( buf, "Too few coordinates: '%s'", str_orig);
#ifdef DEBUG
          fprintf( stderr, buf);
          // exit( -1);
#endif
	  set_err_str( buf);
	  fclose( fp);
	  return;
        }
        float x = ( float)atof( words[ 1]);
        float y = ( float)atof( words[ 2]);
        float z = ( float)atof( words[ 3]);

        if ( nwords == 5) {
          float w = ( float)atof( words[ 4]);
          if ( w != 0.0f) {
            float inv_w = 1.0f / w;
            x *= inv_w;
            y *= inv_w;
            z *= inv_w;
          }
        }

        //addVertex( Vertex( Point( x,y,z)));
        vertices.push_back( Vertex( Point( x,y,z)));
      } else if ( !strcmp( first_word, "vn")) {
      } else if ( !strcmp( first_word, "vt")) {
      } else if ( !strcmp( first_word, "f")) {
        make_face( &words[ 1], nwords-1, currentMaterial);
      } else if ( !strcmp( first_word, "mtllib")) {
        // added
        read_mtllib( &words[ 1], nwords-1, matlib, filename);
      } else if ( !strcmp( first_word, "usemtl")) {
        size_t size = strlen( words[ 1])-1;
        while ( size && ( words[ 1][ size]=='\n' || words[ 1][ size]=='\r') ) words[ 1][ size--]=0;
        currentMaterial = matlib.index( words[ 1]);
      } else {
        // fadded
        // fprintf( stderr, "Do not recognize: '%s'\n", str_orig);
      }  
    }
    fclose( fp);
  }
  updateNormals();
  updateBoundingBox();
  int numFaces[ 16] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
  for ( unsigned int i = 0; i < faces.size(); i++) {
    int nSizes = ( int)faces[ i].vertices.size();
    if ( nSizes < 16) 
      numFaces[ nSizes]++;
  }
  _numTriangs = numFaces[ 3];
  _numQuads = numFaces[ 4];
  // char buf[ 2048];
  sprintf( buf, "Vertices read = %d, Faces read = %d ( ", ( int)vertices.size(), ( int)faces.size());
  int coma = 0;
  for ( int i = 1; i < 16; i++) {
    if ( numFaces[ i]) {
      char tipo[ 80];
      switch( i) {
        case  1: strcpy( tipo, "points"); break;
        case  2: strcpy( tipo, "lines"); break;
        case  3: strcpy( tipo, "triangles"); break;
        case  4: strcpy( tipo, "quadrilaterals"); break;
        case  5: strcpy( tipo, "pentagons"); break;
        case  6: strcpy( tipo, "hexagons"); break;
        case  7: strcpy( tipo, "heptagons"); break;
        case  8: strcpy( tipo, "octogons"); break;
        case  9: strcpy( tipo, "eneagons"); break;
        case 10: strcpy( tipo, "decagons"); break;
        default: sprintf( tipo, "%d-gons", i); break;
      }
      char tmp[ 80];
      sprintf( tmp, "%s%d %s", coma++ ? ", " : "", numFaces[ i], tipo);
      strcat( buf, tmp);
    }
  }
#ifdef DEBUG
  printf( "%s)\n", buf);
#endif
}

// Netela les dades de l'objecte per a poder carregar un nou model
void Object::clear() {
  vertices.clear();
  faces.clear();
  _boundingBox.init( Point());
}

void Object::translate( float x, float y, float z) {
  for ( unsigned int i = 0; i < vertices.size(); i++) {
    vertices[ i].coord.x += x;
    vertices[ i].coord.y += y;
    vertices[ i].coord.z += z;
  }
  updateBoundingBox();
}

void Object::deleteVALists() {
  if ( _lstVertices) delete[ ] _lstVertices;
  _lstVertices = NULL;
  if ( _lstNormales) delete[ ] _lstNormales;
  _lstNormales = NULL;
  if ( _lstIndicesTriangs) delete[ ] _lstIndicesTriangs;
  _lstIndicesTriangs = NULL;
  if ( _lstIndicesQuads) delete[ ] _lstIndicesQuads;
  _lstIndicesQuads = NULL;
}

void Object::createVALists() {
  // printf( "Creando VA's\n");
  deleteVALists(); // borramos las posibles listas
  if ( vertices.size()) {
    _numVertices = vertices.size();
    _lstVertices = new float[ vertices.size() * 3];
    _lstNormales = new float[ vertices.size() * 3];
    for ( unsigned int i = 0, iv = 0; i < vertices.size(); i++, iv += 3) {
      _lstVertices[ iv + 0] = vertices[ i].coord.x;
      _lstVertices[ iv + 1] = vertices[ i].coord.y;
      _lstVertices[ iv + 2] = vertices[ i].coord.z;
      _lstNormales[ iv + 0] = vertices[ i].normal.x;
      _lstNormales[ iv + 1] = vertices[ i].normal.y;
      _lstNormales[ iv + 2] = vertices[ i].normal.z;
    }
  }
  if ( faces.size()) {
    _numFaces = faces.size();
    if ( _numTriangs) 
      _lstIndicesTriangs = new unsigned int[ _numTriangs * 3];
    if ( _numQuads)
      _lstIndicesQuads = new unsigned int[ _numQuads * 4];
    int idxTriang = 0, idxQuad = 0;
    for ( unsigned int i = 0; i < faces.size(); i++) {
      if ( faces[ i].vertices.size() < 3) {
#ifdef DEBUG
	printf( "Face with less than 3 vertices: %d\n", ( int)faces[ i].vertices.size());
#endif
	continue;
      } else if ( faces[ i].vertices.size() == 3) {
	_lstIndicesTriangs[ idxTriang++] = faces[ i].vertices[ 0];
	_lstIndicesTriangs[ idxTriang++] = faces[ i].vertices[ 1];
	_lstIndicesTriangs[ idxTriang++] = faces[ i].vertices[ 2];
      } else if ( faces[ i].vertices.size() == 4) {
	_lstIndicesQuads[ idxQuad++] = faces[ i].vertices[ 0];
	_lstIndicesQuads[ idxQuad++] = faces[ i].vertices[ 1];
	_lstIndicesQuads[ idxQuad++] = faces[ i].vertices[ 2];
	_lstIndicesQuads[ idxQuad++] = faces[ i].vertices[ 3];
      } else {
#ifdef DEBUG
	printf( "Face with more than 4 vertices: %d\n", ( int)faces[ i].vertices.size());
#endif
      } 
    }
  }
}

void Object::renderVALists() {

  // glEnableClientState( GL_VERTEX_ARRAY);
  // glEnableClientState( GL_NORMAL_ARRAY);
  // 
  // glVertexPointer( 3, GL_FLOAT, 0, _lstVertices);
  // 
  // glNormalPointer( GL_FLOAT, 0, _lstNormales);
  // 
  // if ( _numTriangs) 
  //   glDrawElements( GL_TRIANGLES, _numTriangs * 3, GL_UNSIGNED_INT, _lstIndicesTriangs);
  // 
  // if ( _numQuads) 
  //   glDrawElements( GL_QUADS, _numQuads * 4, GL_UNSIGNED_INT, _lstIndicesQuads);
  // 
  // glDisableClientState( GL_NORMAL_ARRAY);
  // glDisableClientState( GL_VERTEX_ARRAY);
}

int Object::write( const char *fileName) {
  FILE *fo = fopen( fileName, "w");
  if ( !fo) return 0;
  // formato GiD
  fprintf( fo, "MESH \"%s\" dimension 3 ElemType Triangle Nnode 3\n", name.data());
  fprintf( fo, "Coordinates\n");
  for ( unsigned int i = 0; i < vertices.size(); i++) {
    fprintf( fo, "%d %g %g %g\n", i + 1, vertices[ i].coord.x, vertices[ i].coord.y, vertices[ i].coord.z);
  }
  fprintf( fo, "End coordinates\n");
  fprintf( fo, "Elements\n");
  // los tratamos todos como si fuesen triangulos
  bool hayCuadrados = 0;
  for ( unsigned int i = 0; i < faces.size(); i++) {
    if ( faces[ i].vertices.size() != 3) {
      // printf( "Face with more than 3 vertices: %d\n", ( int)faces[ i].vertices.size());
      if ( faces[ i].vertices.size() == 4)
	hayCuadrados = 1;
    } else {
      fprintf( fo, "%d %d %d %d\n", i + 1, 
	       faces[ i].vertices[ 0] + 1, faces[ i].vertices[ 1] + 1, faces[ i].vertices[ 2] + 1);
    }
  }
  fprintf( fo, "End elements\n");

  if ( hayCuadrados) {
    fprintf( fo, "MESH \"%s-4\" dimension 3 ElemType Quadrilateral Nnode 4\n", name.data());
    fprintf( fo, "Coordinates\n");
    fprintf( fo, "End coordinates\n");
    fprintf( fo, "Elements\n");
    // los tratamos todos como si fuesen triangulos
    bool hayOtros = 0;
    for ( unsigned int i = 0; i < faces.size(); i++) {
      if ( faces[ i].vertices.size() == 3) continue; // ya los hemos escrito antes
      if ( faces[ i].vertices.size() != 4) {
	printf( "Face with more than 4 vertices: %d\n", ( int)faces[ i].vertices.size());
	hayOtros = 1;
      } else {
	fprintf( fo, "%d %d %d %d %d\n", i + 1, 
		 faces[ i].vertices[ 0] + 1, faces[ i].vertices[ 1] + 1, 
		 faces[ i].vertices[ 2] + 1, faces[ i].vertices[ 3] + 1);
      }
    }
    fprintf( fo, "End elements\n");
  }

  fclose( fo);

#ifdef DEBUG
  printf( "Object %s written in file %s\n", name.data(), fileName);
#endif
  return 1;
}
