/* -*- c++ -*- Created: Miguel Pasenau, Modified: miguel ene-2008 */

#ifndef _GID_ARRAYS_H
#define _GID_ARRAYS_H

//#define DEBUG_MAPA_MEMORIA
#ifdef VERIFPROT_LIB
//undefine for verifprot.lib because MapaMemoria has more dependencies
#undef DEBUG_MAPA_MEMORIA
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

#ifdef DEBUG_MAPA_MEMORIA
#include "MemMap.h"
#endif

#ifndef TD_Verify
#ifndef NDEBUG
#define TD_Verify( ptr, size)  { \
  if ( !( ptr)) {   \
    fprintf( stderr,   \
    "%s: not enough memory for %d bytes.\n", #ptr, size);  \
  }  \
  assert( ptr && "Not enough memory.");  \
}
#else // NDEBUG
#define TD_Verify( ptr, size)   ((void) 0)
#endif // NDEBUG
#endif // TD_Verify

//#define GID_ARRAYS_DEBUG

#define TAM_CACHO_BITS              12
#define TAM_CACHO_SIMPLE            4096
#define TAM_CACHO_MASK              ( TAM_CACHO_SIMPLE - 1)
#define TAM_CACHO_ID                ~TAM_CACHO_MASK

#ifndef FALSE
#define FALSE 0
#endif

#ifndef TRUE
#define TRUE 1
#endif

#ifdef GID_ARRAYS_DEBUG
extern int GidDynamicArray_instancias;
extern FILE *GidDynamicArray_f_ini, *GidDynamicArray_f_fin;
#endif

#define DEBUG_PLEX_SIMPLE

#ifndef TD_Error
#define TD_Error( A)          { FPRINTF_UTF( stderr, A); exit(2); }
#define TD_Error2( A, B)      { FPRINTF_UTF( stderr, A, B); exit(2); }
#define TD_Error3( A, B, C)   { FPRINTF_UTF( stderr, A, B, C); exit(2); }
#endif

template < class T>
class GidDynamicArray {
private:
  int _tam, _tam_cacho, _last;
  T *_d;
  char *_name;
  unsigned char _pon_0, _usa_alloc;
  
  void initialize_d( void) {
    if ( _usa_alloc)
      _d = ( T *)malloc( _tam_cacho * sizeof( T));
    else
      _d = new T[ _tam_cacho];
    assert( _d && "not enough memory in GidDynamicArray::inicializa");
    _tam = _tam_cacho;// * sizeof( T);
    TD_Verify( _d, _tam_cacho);
    if ( _pon_0 && _usa_alloc) 
      memset( ( void *)_d, 0, _tam_cacho * sizeof( T));
    _last = -1;
    _tam = _tam_cacho;
#ifdef GID_ARRAYS_DEBUG
    FPRINTF_UTF( GidDynamicArray_f_ini, "INICIALIZA: %s.\n", _name ? _name : "N/A");
    fflush( GidDynamicArray_f_ini);
#else
    //_name = NULL;
#endif
  }

  void free_d( void) {
    if ( _d) {
      if ( _usa_alloc)
	free( _d);
      else
	delete[] _d;
    }
    _d = NULL;
    _last = -1;
  }

  void swapIdx( const int idx_e1, const int idx_e2) {
    // swapBytes( &( ( *this)[ idx_e1]), &( ( *this)[ idx_e2])); 
    if ( idx_e1 != idx_e2) {
      unsigned char *ptr_e1 = ( unsigned char *)getItemPtr( idx_e1);//&( ( *this)[ idx_e1]);
      unsigned char *ptr_e2 = ( unsigned char *)getItemPtr( idx_e2);//&( ( *this)[ idx_e2]);
      unsigned int tam = sizeof( T);
      while ( tam) {
   	tam--;
   	unsigned char t = *ptr_e1;
   	*ptr_e1++ = *ptr_e2;
   	*ptr_e2++ = t;
      }
    }
  }

  void Bubble( const int lo_org, const int hi_org,
		int ( *FuncComp)( const T &e1, const T &e2)) {
    int lo = lo_org;
    int hi = hi_org;
    while ( lo < hi) {
      int idx_max = lo;
      for ( int idx = lo + 1; idx <= hi; idx++) {
	int idx_p = idx;
	// if ( FuncComp( ( *this)[ idx_p], ( *this)[ idx_max]) > 0) {
	if ( FuncComp( getItem( idx_p), getItem( idx_max)) > 0) {
	  idx_max = idx_p;
	}
      }
      swapIdx( idx_max,  hi);
      hi--;
    }
  }

public:
  GidDynamicArray( const unsigned char init = FALSE, const int tam_pref = TAM_CACHO_SIMPLE, 
		 const char *s = NULL, const unsigned char usa_alloc = TRUE) : 
    _tam_cacho( tam_pref), _pon_0( init), _usa_alloc( usa_alloc) {
    // #ifdef DEBUG_PLEX_SIMPLE
    //     if ( tam_pref <= 0) 
    // 	 TD_Error2( "GidDynamicArray: Tamanno no valido: %d.\n", tam_pref);
    // #endif
    assert( ( tam_pref > 0) && "GidDynamicArray: Tamanno no valido.\n");
    if ( _tam_cacho <= 0) _tam_cacho = TAM_CACHO_SIMPLE; // por si acaso

    _d = NULL;
    _tam = 0;// tam_pref;// * sizeof( T);
    _last = -1;
#ifdef GID_ARRAYS_DEBUG
    if ( !GidDynamicArray_f_ini) {
#ifndef WIN32
      GidDynamicArray_f_ini = FOPEN_UTF( "/tmp/td_ini.out", "w");
      GidDynamicArray_f_fin = FOPEN_UTF( "/tmp/td_fin.out", "w");
#else
      GidDynamicArray_f_ini = FOPEN_UTF( "c:\\temp\\td_ini.out", "w");
      GidDynamicArray_f_fin = FOPEN_UTF( "c:\\temp\\td_fin.out", "w");
#endif
    }
    GidDynamicArray_instancias++;
    if ( s) {
      _name = strdup( s);
    } else {
      char buf[ N_PEQ];
#ifndef WIN32
      sprintf( buf, "%3d bytes no. %6d in '%s'", sizeof( T), GidDynamicArray_instancias, __PRETTY_FUNCTION__);
#else
      sprintf( buf, "%3d bytes no. %6d in '%s' %s", sizeof( T), GidDynamicArray_instancias, __FUNCDNAME__, __FUNCSIG__);
#endif
      _name = strdup( buf);
    }
    FPRINTF_UTF( GidDynamicArray_f_ini, "%s.\n", _name);
    fflush( GidDynamicArray_f_ini);
#else // GID_ARRAYS_DEBUG
    if ( s) _name = strdup( s);
    else _name = NULL;
#endif // GID_ARRAYS_DEBUG
  }

  void init( const int do_init) {
    if ( !_d) initialize_d();
    if ( _last == -1) {
      _pon_0 = do_init;
      if ( _pon_0 && _usa_alloc) 
	memset( ( void *)_d, 0, _tam * sizeof( T));
    }
  }
  
  void reset( void) {
    free_d();

#ifdef GID_ARRAYS_DEBUG
    // FPRINTF_UTF( GidDynamicArray_f_ini, "RESET: %s.\n", _name);
    // fflush( GidDynamicArray_f_ini);
#else
    if ( _name)
      free( _name);
    _name = NULL;
#endif
  }
  
  void hard_reset( const int init = FALSE, const int tam_pref = TAM_CACHO_SIMPLE, const char *s = NULL) {
    _pon_0 = init;
    // #ifdef DEBUG_PLEX_SIMPLE
    //     if ( tam_pref <= 0) 
    // 	 TD_Error2( "GidDynamicArray: Tamanno no valido: %d.\n", tam_pref);
    // #endif
    assert( ( tam_pref > 0) && "GidDynamicArray: Tamanno no valido.\n");

    _tam_cacho = tam_pref;
    if ( _tam_cacho <= 0) _tam_cacho = TAM_CACHO_SIMPLE; // por si acaso

    initialize_d();

#ifdef GID_ARRAYS_DEBUG
    FPRINTF_UTF( GidDynamicArray_f_ini, "HARD_RESET: %s.\n", _name);
    fflush( GidDynamicArray_f_ini);
#else
    if ( s) _name = strdup( s);
    else _name = NULL;
#endif
  }
  
  ~GidDynamicArray() { 
    //reset();
    free_d();

    if ( _name) {
#ifdef GID_ARRAYS_DEBUG
      FPRINTF_UTF( GidDynamicArray_f_fin, "%s.\n", _name);
      fflush( GidDynamicArray_f_fin);
#endif
      free( _name);
    }
    _name = NULL;
  }
  
#if 0
  T operator[] ( int index) const {
    if ( index < _tam /*<= _last*/) 
      return _d[ index];
    else return ( T)NULL;
  }
#endif

  T &operator[] ( int index) {
    if ( !_d) initialize_d();
    if ( index <= _last) 
      return _d[ index];
    else if ( index < _tam) {
      _last = index;
      return _d[ index];
    } else {
      int old_tam = _tam;
      do {
        _tam += _tam_cacho;
	// mas rapido y menos reallocs, pero ms desperdicio de memoria en windows
        // _tam += _tam;
      } while ( index >= _tam);
      if ( _usa_alloc) {
	_d = ( T *)realloc( _d, _tam * sizeof( T));
	assert( _d && "not enough memory in GidDynamicArray::[] realloc");
      } else {
	T *tmp = new T[ _tam];
	assert( _d && "not enough memory in GidDynamicArray::[] new/realloc _tmp");
	if ( _d && tmp) {
	  for ( int it = 0; it < old_tam; it++)
	    tmp[ it] = _d[ it];
	  delete[] _d;
	}
	_d = tmp;
	assert( _d && "not enough memory in GidDynamicArray::[] new/delete-realloc");
      }
      TD_Verify( _d, _tam);
      if ( _pon_0 && _usa_alloc) 
	memset( ( void *)&_d[ old_tam], 0, ( _tam - old_tam) * sizeof( T));
      _last = index;
      return _d[ index];
    }
  }

  const T &getItem( const int index) const {
    assert( index <= _last);
    return _d[ index];
  }

  const T *getItemPtr( const int index) const {
    assert( index <= _last);
    return &_d[ index];
  }

#ifdef DEBUG_MAPA_MEMORIA
  size_t MapaMemoria( MemMap *MapaMem, const char *pref) const {
    size_t tam = sizeof( *this);
    MapaMem->append( this, tam, pref, "TD_base");
    if ( _name) {
      MapaMem->append( _name, strlen( _name), pref, "TD_nom");
      tam += strlen( _name);
    }
    MapaMem->append( _d, _tam * sizeof( T), pref, "TD_tbl");
    tam += _tam * sizeof( T);
    return tam;
  }
#endif

  T &pos( int index) {
    return ( *this)[ index];
  }

  int append( const T &item) {
    int idx = _last;
    idx++;
    ( *this)[ idx] = item;
    return idx;
  }

  int bucketSize( void) const { return TAM_CACHO_SIMPLE; };
  int getReservedSize( void) const { return _tam;/*_last*/ };
  int size( void) const { return _last + 1; };
  
  char *&name( void) { return _name; }
  char *name( void) const { return _name; }
  
  void setName( const char *nuevo) { 
    if ( _name) free( _name);
    _name = NULL;
    if ( nuevo) {
      _name = strdup( nuevo);
    }
  };

  T *datos( void) const { return _d; }
  operator T *() const { return _d; }
  // operator const T *() const { return _d; }

  int last() const { return _last; }

  void delete_last() {
    _last--;
    if ( _last == -1) reset();
  }

  void sort( int ( *func_cmp)( const void *, const void *)) {
    if ( _d) qsort( ( void *)_d, _last + 1, sizeof( T), func_cmp);
  }

  T *search( const T *clave, int ( *func_cmp)( const void *, const void *)) {
    T * ret = NULL;
    if ( _d)
      ret = ( T *)bsearch( ( const void *)clave, ( const void *)_d, _last + 1, sizeof( T), func_cmp);
    return ret;
  }

  T *searchLineal( const T *clave, int ( *func_cmp)( const void *, const void *)) {
    T * ret = NULL;
    if ( _d) {
      for ( int idx = 0; idx < _last + 1; idx++) {
	int resultado = func_cmp( clave, &pos( idx));
	if ( !resultado) {
	  ret = &pos( idx);
	  break;
	}
      }
    }
    return ret;
  }
#define TD2_QSORT_MIN          8    // test muestran que este es un buen valor
  void QSort( int ( *FuncComp)( const T &e1, const T &e2))  {
    const int num = size();
    if ( num < 2) return;
    
    int lostk_idx[ 30], histk_idx[30];
    int stkptr = 0;                             // pila para guardar subtablas a procesar
    int lo_idx = 0;
    int hi_idx = num - 1;
    
  recurse:
    
    unsigned int size = ( hi_idx - lo_idx) + 1; // elementos a sortr
    if ( size <= TD2_QSORT_MIN) {
      Bubble( lo_idx, hi_idx, FuncComp);
    } else {
      int mid_idx = lo_idx + size / 2; // seleccinamos el elemento del medio
      swapIdx( mid_idx, lo_idx);                  // lo metemos al principio de la tabla
      
      int loguy_idx = lo_idx;
      int higuy_idx = hi_idx + 1;
      for( ; ; ) {
	do {
	  loguy_idx++;
	} while ( ( loguy_idx <= hi_idx) && 
		  // ( FuncComp( ( *this)[ loguy_idx], ( *this)[ lo_idx]) <= 0));
		  ( FuncComp( getItem( loguy_idx), getItem( lo_idx)) <= 0));
	do {
	  higuy_idx--;
	} while ( ( higuy_idx > lo_idx) && 
		  // ( FuncComp( ( *this)[ higuy_idx], ( *this)[ lo_idx]) >= 0));
		  ( FuncComp( getItem( higuy_idx), getItem( lo_idx)) >= 0));
	if ( higuy_idx < loguy_idx)
	  break;
	
	swapIdx( loguy_idx, higuy_idx);
      }
      
      swapIdx( lo_idx, higuy_idx);       // devolvemos el elemento particionador a su sitio
      
      if ( higuy_idx - 1 - lo_idx >= hi_idx - loguy_idx) {
	if ( lo_idx + 1 < higuy_idx) {
	  lostk_idx[ stkptr] = lo_idx;
	  histk_idx[ stkptr] = higuy_idx - 1;
	  stkptr++;
	}                              // guardamos la recursion mayor para luego
	
	if ( loguy_idx < hi_idx) {
	  lo_idx = loguy_idx;
	  goto recurse;                // hacemos la recursion pequea
	}
      } else {
	if (loguy_idx < hi_idx) {
	  lostk_idx[ stkptr] = loguy_idx;
	  histk_idx[ stkptr] = hi_idx;
	  stkptr++;           
	}                             // guardamos la recursion mayor para luego
	
	if ( lo_idx + 1 < higuy_idx) {
	  hi_idx = higuy_idx - 1;
	  goto recurse;         
	}                            // hacemos la recursion pequea
      }
    }
    
    stkptr--;
    if ( stkptr >= 0) {
      lo_idx = lostk_idx[ stkptr];
      hi_idx = histk_idx[ stkptr];
      goto recurse;                 // pop subtabla de la pila
    } else
      return;                       // todas las subtablas hechas
  }
#undef TD2_QSORT_MIN

};

#define TAM_CACHO_TABLA_PTR      256

template < class T>
class GidArrayList {
private:
  int _last;
  T **_d;
  unsigned char _pon_0, _usa_alloc;
  short int _tam_tabla_ptr;

  void alloca_bucket( int idx) {
    // ahora reservamos el primer trozo
    if ( _usa_alloc) {
      int tt = TAM_CACHO_SIMPLE * sizeof( T);
      _d[ idx] = ( T *)malloc( tt);
    } else {
      _d[ idx] = new T[ TAM_CACHO_SIMPLE];
    }
    TD_Verify( _d[ idx], TAM_CACHO_SIMPLE);
    if ( _pon_0 && _usa_alloc) 
      memset( ( void *)_d[ idx], 0, TAM_CACHO_SIMPLE * sizeof( T));
  }

  // void swapBytes( unsigned char *e1, unsigned char *e2) {
  //   if ( e1 != e2) {
  //     unsigned int tam = sizeof( T);
  //     while ( tam) {
  // 	tam--;
  // 	unsigned char t = *e1;
  // 	*e1++ = *e2;
  // 	*e2++ = t;
  //     }
  //   }
  // }

  void swapIdx( const int idx_e1, const int idx_e2) {
    // swapBytes( &( ( *this)[ idx_e1]), &( ( *this)[ idx_e2])); 
    if ( idx_e1 != idx_e2) {
      unsigned char *ptr_e1 = ( unsigned char *)getItemPtr( idx_e1);//&( ( *this)[ idx_e1]);
      unsigned char *ptr_e2 = ( unsigned char *)getItemPtr( idx_e2);//&( ( *this)[ idx_e2]);
      unsigned int tam = sizeof( T);
      while ( tam) {
   	tam--;
   	unsigned char t = *ptr_e1;
   	*ptr_e1++ = *ptr_e2;
   	*ptr_e2++ = t;
      }
    }
  }

  void Bubble( const int lo_org, const int hi_org,
		int ( *FuncComp)( const T &e1, const T &e2)) {
    int lo = lo_org;
    int hi = hi_org;
    while ( lo < hi) {
      int idx_max = lo;
      for ( int idx = lo + 1; idx <= hi; idx++) {
	int idx_p = idx;
	// if ( FuncComp( ( *this)[ idx_p], ( *this)[ idx_max]) > 0) {
	if ( FuncComp( getItem( idx_p), getItem( idx_max)) > 0) {
	  idx_max = idx_p;
	}
      }
      swapIdx( idx_max,  hi);
      hi--;
    }
  }

public:
  GidArrayList( const unsigned char init = FALSE, const unsigned char usa_alloc = TRUE) : 
    _pon_0( init), _usa_alloc( usa_alloc) {
    // la tabla de punteros primero
    _tam_tabla_ptr = TAM_CACHO_TABLA_PTR;
    _d = ( T **)malloc( _tam_tabla_ptr * sizeof( T *));
    TD_Verify( _d, _tam_tabla_ptr);
    memset( ( void *)_d, 0, _tam_tabla_ptr * sizeof( T *));
    // ahora reservamos el primer trozo
    alloca_bucket( 0);
    _last = -1;
  }

  void init( const int do_init) {
    if ( _last == -1) {
      _pon_0 = do_init;
      if ( _pon_0 && _usa_alloc) {
	for ( int it = 0; _d[ it] && ( it < _tam_tabla_ptr); it++)
	  memset( ( void *)_d[ it], 0, TAM_CACHO_SIMPLE * sizeof( T));
      }
    }
  }
  
  void reset( void) {
    for ( int it = 1; _d[ it] && ( it < _tam_tabla_ptr); it++) {
      if ( _usa_alloc) {
	free( _d[ it]);
      } else {
	delete[] _d[ it];
      }
      _d[ it] = NULL;
    }

    if ( _usa_alloc) {
      if ( !_d[ 0]) {
	_d[ 0] = ( T *)malloc( TAM_CACHO_SIMPLE * sizeof( T));
      }
    } else {
      if ( _d[ 0]) delete[] _d[ 0];
      _d[ 0] = new T[ TAM_CACHO_SIMPLE];
    }

    TD_Verify( _d[ 0], TAM_CACHO_SIMPLE);
    if ( _pon_0 && _usa_alloc) 
      memset( ( void *)_d[ 0], 0, TAM_CACHO_SIMPLE * sizeof( T));
    _last = -1;
  }
  
  void hard_reset( const int init = FALSE, const char *s = NULL) {
    _pon_0 = init;
    // la tabla de punteros primero
    _tam_tabla_ptr = TAM_CACHO_TABLA_PTR;
    _d = ( T **)malloc( _tam_tabla_ptr * sizeof( T *));
    TD_Verify( _d, _tam_tabla_ptr);
    memset( ( void *)_d, 0, _tam_tabla_ptr * sizeof( T *));
    // ahora reservamos el primer trozo
    alloca_bucket( 0);
    _last = -1;
  }
  
  ~GidArrayList() { 
    reset();
    if ( _d[ 0]) {
      if ( _usa_alloc)
	free( _d[ 0]);
      else
	delete[] _d[ 0];
    }
    free( _d);
    _d = NULL;
  }

#ifdef DEBUG_MAPA_MEMORIA
  size_t MapaMemoria( MemMap *MapaMem, const char *pref) const {
    size_t tam = sizeof( *this);
    MapaMem->append( this, tam, pref, "TD2_base");
    MapaMem->append( _d, _tam_tabla_ptr * sizeof( T *), pref, "TD2_tbl");
    tam += _tam_tabla_ptr * sizeof( T *);
    for ( int it = 1; _d[ it] && ( it < _tam_tabla_ptr); it++) {
      size_t tam_trozo = TAM_CACHO_SIMPLE * sizeof( T);
      MapaMem->append( _d[ it], tam_trozo, pref, "TD2_trz");
      tam += tam_trozo;
    }
    return tam;
  }
#endif

  T &operator[] ( const int index) {
    // int idx_id = index & TAM_CACHO_ID;
    // if ( !idx_id) {
    if ( index < TAM_CACHO_SIMPLE) {
      if ( index <= _last) 
	return _d[ 0][ index];
      else {
	_last = index;
	return _d[ 0][ index];
      }
    } else {
      // int idx_ptr = idx_id >> TAM_CACHO_BITS;
      int idx_ptr = index >> TAM_CACHO_BITS;
      if ( idx_ptr >= _tam_tabla_ptr) {
	int old_tam_tabla_ptr = _tam_tabla_ptr;
	do {
	  _tam_tabla_ptr += TAM_CACHO_TABLA_PTR;
	} while ( idx_ptr >= _tam_tabla_ptr);
	_d = ( T **)realloc( _d, _tam_tabla_ptr * sizeof( T *));
	memset( ( void *)&_d[ old_tam_tabla_ptr], 0, ( _tam_tabla_ptr - old_tam_tabla_ptr) * sizeof( T *));
      }
      if ( !_d[ idx_ptr]) {
	alloca_bucket( idx_ptr);
      }
      if ( index > _last) 
	_last = index;
      return _d[ idx_ptr][ index & TAM_CACHO_MASK];
    }
  }

  const T &operator[] ( const int index) const {
    assert( index <= _last);
    // if ( index > _last) return T();
    // int idx_id = index & TAM_CACHO_ID;
    // if ( !idx_id) {
    if ( index < TAM_CACHO_SIMPLE) {
      // if ( index <= _last) 
      return _d[ 0][ index];
    } else {
      // int idx_ptr = idx_id >> TAM_CACHO_BITS;
      int idx_ptr = index >> TAM_CACHO_BITS;
      // if ( idx_ptr < _tam_tabla_ptr)
      return _d[ idx_ptr][ index & TAM_CACHO_MASK];
    }
    // return T();
  }

  const T &getItem( const int index) const {
    assert( index <= _last);
    if ( index < TAM_CACHO_SIMPLE) {
      return _d[ 0][ index];
    } else {
      int idx_ptr = index >> TAM_CACHO_BITS;
      return _d[ idx_ptr][ index & TAM_CACHO_MASK];
    }
  }

  const T *getItemPtr( const int index) const {
    assert( index <= _last);
    if ( index < TAM_CACHO_SIMPLE) {
      return &( _d[ 0][ index]);
    } else {
      int idx_ptr = index >> TAM_CACHO_BITS;
      return &( _d[ idx_ptr][ index & TAM_CACHO_MASK]);
    }
  }

  int append( const T &item) {
    int idx = _last;
    idx++;
    ( *this)[ idx] = item;
    return idx;
  }

  // int bucketSize( void) const { return TAM_CACHO_SIMPLE; };
  // int HoyManyCaben( void) const { return _tam;/*_last*/ };
  int size( void) const { return _last + 1; };
  
  // T *datos( void) const { return _d; }
  // operator T *() const { return _d; }

  int last() const { return _last; }

  void delete_last() {
    _last--;
    if ( _last == -1) reset();
  }

#define TD2_QSORT_MIN          8    // test muestran que este es un buen valor
  void QSort( int ( *FuncComp)( const T &e1, const T &e2))  {
    const int num = size();
    if ( num < 2) return;
    
    int lostk_idx[ 30], histk_idx[30];
    int stkptr = 0;                             // pila para guardar subtablas a procesar
    int lo_idx = 0;
    int hi_idx = num - 1;
    
  recurse:
    
    unsigned int size = ( hi_idx - lo_idx) + 1; // elementos a sortr
    if ( size <= TD2_QSORT_MIN) {
      Bubble( lo_idx, hi_idx, FuncComp);
    } else {
      int mid_idx = lo_idx + size / 2; // seleccinamos el elemento del medio
      swapIdx( mid_idx, lo_idx);                  // lo metemos al principio de la tabla
      
      int loguy_idx = lo_idx;
      int higuy_idx = hi_idx + 1;
      for( ; ; ) {
	do {
	  loguy_idx++;
	} while ( ( loguy_idx <= hi_idx) && 
		  // ( FuncComp( ( *this)[ loguy_idx], ( *this)[ lo_idx]) <= 0));
		  ( FuncComp( getItem( loguy_idx), getItem( lo_idx)) <= 0));
	do {
	  higuy_idx--;
	} while ( ( higuy_idx > lo_idx) && 
		  // ( FuncComp( ( *this)[ higuy_idx], ( *this)[ lo_idx]) >= 0));
		  ( FuncComp( getItem( higuy_idx), getItem( lo_idx)) >= 0));
	if ( higuy_idx < loguy_idx)
	  break;
	
	swapIdx( loguy_idx, higuy_idx);
      }
      
      swapIdx( lo_idx, higuy_idx);       // devolvemos el elemento particionador a su sitio
      
      if ( higuy_idx - 1 - lo_idx >= hi_idx - loguy_idx) {
	if ( lo_idx + 1 < higuy_idx) {
	  lostk_idx[ stkptr] = lo_idx;
	  histk_idx[ stkptr] = higuy_idx - 1;
	  stkptr++;
	}                              // guardamos la recursion mayor para luego
	
	if ( loguy_idx < hi_idx) {
	  lo_idx = loguy_idx;
	  goto recurse;                // hacemos la recursion pequea
	}
      } else {
	if (loguy_idx < hi_idx) {
	  lostk_idx[ stkptr] = loguy_idx;
	  histk_idx[ stkptr] = hi_idx;
	  stkptr++;           
	}                             // guardamos la recursion mayor para luego
	
	if ( lo_idx + 1 < higuy_idx) {
	  hi_idx = higuy_idx - 1;
	  goto recurse;         
	}                            // hacemos la recursion pequea
      }
    }
    
    stkptr--;
    if ( stkptr >= 0) {
      lo_idx = lostk_idx[ stkptr];
      hi_idx = histk_idx[ stkptr];
      goto recurse;                 // pop subtabla de la pila
    } else
      return;                       // todas las subtablas hechas
  }
#undef TD2_QSORT_MIN

  // devuelve el indice del elemento de la tabla sortda igual a la aguja, si hay repes devuelve uno de ellos
  // si no, devuelve -1
  int BSearchIdx( const T &aguja, int ( *FuncComp)( const T &e1, const T &e2)) const {
    const int n = size();
    if ( !n) return -1;
    int lo = 0;
    int hi = n - 1;
    int mid = ( hi - lo + 1) >> 1; // la mitad
    while ( mid && ( hi > lo)) {
      mid += lo; // el medio entre lo y hi
      int val = FuncComp( ( *this)[ mid], aguja);
      if ( !val) {
	return mid;
      } 
      if ( val < 0) {
	lo = mid + 1; // ajustamos el lo
      } else { // val > 0
	hi = mid - 1; // ajustamos el hi
      }
      mid = ( hi - lo + 1) >> 1; // la mitad de lo que queda
    }
    if ( !mid && ( lo < n)) {
      if ( !FuncComp( ( *this)[ lo], aguja))  return lo;
      // if ( !FuncComp( getItem( lo), aguja))  return getItemPtr( lo);
    }
    return -1;

    // const T *ret = NULL;
    // int i = 0;
    // int tam = size();
    // while ( i < tam) {
    //   int val = FuncComp( ( *this)[ i], aguja);
    //   if ( val >= 0) {
    // 	if ( !val) ret = &( ( *this)[ i]);
    // 	break;
    //   }
    //   i++;
    // }
    // return ret;
  }

  // devuelve el puntero al elemento de la tabla sortda igual a la aguja, si hay repes devuelve uno de ellos
  // si no, devuelve NULL
  const T *BSearch( const T &aguja, int ( *FuncComp)( const T &e1, const T &e2)) const {
    int idx = this->BSearchIdx( aguja, FuncComp);
    return idx >= 0 ? &( ( *this)[ idx]) : NULL;
  }

};

template <class T>
class GidDynamicList: public GidArrayList< T> {
};

#define _TAM_PILA_DEF_      256

template <class T>
class GidStack: public GidArrayList< T> {
public:
  GidStack( const int tam = _TAM_PILA_DEF_): GidArrayList< T>( TRUE, FALSE), _tamanno( tam), _cima( 0) {
  }

  void push( const T e) {
    if ( ( _cima + 1) < _tamanno) {
      _cima++;
    }
    ( *this)[ _cima] = e;
  }

  const T pop() {
    if ( _cima >= 0)
      _cima--;
    return ( *this)[ _cima + 1];
  }

  const T top() {
    return ( *this)[ _cima];
  }

private:
  int _tamanno, _cima;
};
#undef _TAM_PILA_DEF_

#endif //_GID_ARRAYS_H
