#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifdef _WIN32
#define strdup _strdup
#endif

#define BUILD_GID_PLUGIN
#include "gid_plugin_import.h"
/* 
DECLARE_GID_PLUGIN_FUNCTIONS;
*/

#include <ctype.h>
#include "gid_arrays.h"

static char *_G_err_str = NULL;

void set_err_str( const char *str = NULL) {
  if ( _G_err_str) free( _G_err_str);
  _G_err_str = NULL;
  if ( str) {
    _G_err_str = strdup( str);
  }
}

int create_demo_triangs() {
  GiD_NewPostProcess();
  int mesh_id = GiD_NewMesh( GID_POST, GIDPOST_NEW_MESH, "Hello");
  GiD_SetColor( mesh_id, 0.5f, 0.3f, 0.8f, 0.0f);
  float lst_vertices[ 4][ 3] = { { 0.0, 0.0, 0.0}, { 1.0, 0.0, 0.0}, { 1.0, 1.0, 0.0}, { 0.0, 1.0, 0.0}};
  GiD_SetVertexPointer( mesh_id, GIDPOST_FLOAT, GIDPOST_VERTICES, 3, 4, 0, lst_vertices);
  int lst_elements[ 2][ 3] = { { 0, 1, 2}, { 2, 3, 0}};
  GiD_SetElementPointer( mesh_id, GIDPOST_FLOAT, GIDPOST_CONNECTIVITIES, GIDPOST_TRIANGLE, 2, 0, lst_elements, 0, NULL);
  
  float lst_scalar_res[ 4] = { 0.0, 1.0, 2.0, 3.0};
  int res_id = GiD_NewResult( "new analysis", 0.0, "test result", 0); // 0 --> for all meshes
  GiD_SetResultPointer( res_id, GIDPOST_FLOAT, GIDPOST_RESULTS, GIDPOST_SCALAR, GIDPOST_NODAL, 4, 0, lst_scalar_res);
  GiD_EndResult( res_id);
  GiD_EndMesh( mesh_id);
  set_err_str( "Everything fine.");
  return 0;
}

static GidDynamicArray< float> *lst_off_points = NULL;
static GidDynamicArray< float> *lst_off_pnt_colours = NULL;

static GidDynamicArray< int> *lst_off_lines = NULL;
static GidDynamicArray< float> *lst_off_line_colours = NULL;

static GidDynamicArray< int> *lst_off_triangles = NULL;
static GidDynamicArray< float> *lst_off_triangle_colours = NULL;

static GidDynamicArray< int> *lst_off_quads = NULL;
static GidDynamicArray< float> *lst_off_quad_colours = NULL;

static GidDynamicArray< int> *lst_off_pentas = NULL;
static GidDynamicArray< float> *lst_off_penta_colours = NULL;

static GidDynamicArray< int> *lst_off_hexas = NULL;
static GidDynamicArray< float> *lst_off_hexa_colours = NULL;

#define ALLOC_ARRAY( name, type)      {                       \
  if ( !name) {		  	                              \
    name = new GidDynamicArray< type>( FALSE, 1024 * 1024);   \
  }                                                           \
}

void alloc_arrays() {
  ALLOC_ARRAY( lst_off_points, float);
  ALLOC_ARRAY( lst_off_pnt_colours, float);

  ALLOC_ARRAY( lst_off_lines, int);
  ALLOC_ARRAY( lst_off_line_colours, float);

  ALLOC_ARRAY( lst_off_triangles, int);
  ALLOC_ARRAY( lst_off_triangle_colours, float);

  ALLOC_ARRAY( lst_off_quads, int);
  ALLOC_ARRAY( lst_off_quad_colours, float);

  ALLOC_ARRAY( lst_off_pentas, int);
  ALLOC_ARRAY( lst_off_penta_colours, float);

  ALLOC_ARRAY( lst_off_hexas, int);
  ALLOC_ARRAY( lst_off_hexa_colours, float);
}
#undef ALLOC_ARRAY

#define FREE_ARRAY( name)  { \
  if ( name) {		     \
    delete ( name);	     \
  }			     \
  name = NULL;               \
}

void free_arrays() {
  FREE_ARRAY( lst_off_points);
  FREE_ARRAY( lst_off_pnt_colours);

  FREE_ARRAY( lst_off_lines);
  FREE_ARRAY( lst_off_line_colours);

  FREE_ARRAY( lst_off_triangles);
  FREE_ARRAY( lst_off_triangle_colours);

  FREE_ARRAY( lst_off_quads);
  FREE_ARRAY( lst_off_quad_colours);

  FREE_ARRAY( lst_off_pentas);
  FREE_ARRAY( lst_off_penta_colours);

  FREE_ARRAY( lst_off_hexas);
  FREE_ARRAY( lst_off_hexa_colours);
}
#undef FREE_ARRAY

int end_off_file( char *buf, FILE *f) {
  fclose( f);
  free( buf);
  return 0;
}

static char *get_useful_line( char *buf, int max_len, FILE *f_in) {
  char *ret = fgets( buf, max_len, f_in);
  while ( ret) {
    char *s = ret;
    while ( *s && isspace( *s)) {
      s++;
    }
    if ( *s && *s != '#') {
      break;
    }
    ret = fgets( buf, max_len, f_in);
  }
  return ret;
}

int read_off_file( const char *filename) {
  char buf[ 8192];
  FILE *f_in = fopen( filename, "r");
  if ( !f_in) {
    sprintf( buf, "can not open '%s'.", filename);
    set_err_str( buf);
    return 0;
  }
  alloc_arrays();
  const int tam_line = 1024 * 1024;
  char *line = ( char *)malloc( tam_line * sizeof( char));

  /* first line, should be OFF */
  char *ret_str = get_useful_line( line, tam_line, f_in);
  if ( !ret_str) {
    set_err_str( "can not get useful line while reading the header.");
    return end_off_file( line, f_in);
  }
  if ( !strncmp( line, "OFF", strlen( line))) {
    sprintf( buf, "Unknown header '%s'.", line);
    set_err_str( buf);
    return end_off_file( line, f_in);
  }

  /* second line, should be #vertices #faces #edges */
  int num_vertices = 0, num_faces = 0, num_edges = 0;
  ret_str = get_useful_line( line, tam_line, f_in);
  if ( !ret_str) {
    set_err_str( "can not get useful line while reading mesh info.");
    return end_off_file( line, f_in);
  }
  int n = sscanf( line, "%d %d %d", &num_vertices, &num_faces, &num_edges);
  if ( ( n != 3) || !num_vertices || !num_faces) {
    return end_off_file( line, f_in);
  }

  // vertices
  for ( int ip = 0; ip < num_vertices; ip++) {
    ret_str = get_useful_line( line, tam_line, f_in);
    if ( !ret_str) {
      set_err_str( "can not get useful line while reading node coordinates.");
      return end_off_file( line, f_in);
    }
    float x = 0, y = 0, z = 0, r = 0, g = 0, b = 0, a = 0;
    n = sscanf( line, "%g %g %g %g %g %g %g", &x, &y, &z, &r, &g, &b, &a);
    if ( n < 3) {
      sprintf( buf, "can not get node coordinates from '%s'.", line);
      return end_off_file( line, f_in);
    }
    lst_off_points->append( x);
    lst_off_points->append( y);
    lst_off_points->append( z);
    if ( n > 3) {
      lst_off_pnt_colours->append( r);
      lst_off_pnt_colours->append( g);
      lst_off_pnt_colours->append( b);
      lst_off_pnt_colours->append( a);
    }
  }

  // faces
  for ( int ic = 0; ic < num_faces; ic++) {
    ret_str = get_useful_line( line, tam_line, f_in);
    if ( !ret_str) {
      set_err_str( "can not get useful line while reading element connectivity.");
      return end_off_file( line, f_in);
    }
    int num_nodes = 0;
    n = sscanf( line, "%d", &num_nodes);
    if ( n < 1) {
      sprintf( buf, "can not get element connectivity from '%s'.", line);
      return end_off_file( line, f_in);
    }
    float r = 0, g = 0, b = 0, a = 0;
    if ( num_nodes == 2) { // lines
      int n1 = 0, n2 = 0;
      n = sscanf( line, "%d %d %d %g %g %g %g", &num_nodes,
		  &n1, &n2, &r, &g, &b, &a);
      lst_off_lines->append( n1);
      lst_off_lines->append( n2);
      if ( n > 3) {
	lst_off_line_colours->append( r);
	lst_off_line_colours->append( g);
	lst_off_line_colours->append( b);
	lst_off_line_colours->append( a);
      }
    } else if ( num_nodes == 3) { // triangles
      int n1 = 0, n2 = 0, n3 = 0;
      n = sscanf( line, "%d %d %d %d %g %g %g %g", &num_nodes,
		  &n1, &n2, &n3, &r, &g, &b, &a);
      lst_off_triangles->append( n1);
      lst_off_triangles->append( n2);
      lst_off_triangles->append( n3);
      if ( n > 4) {
	lst_off_triangle_colours->append( r);
	lst_off_triangle_colours->append( g);
	lst_off_triangle_colours->append( b);
	lst_off_triangle_colours->append( a);
      }
    } else if ( num_nodes == 4) { // quadrilaterals
      int n1 = 0, n2 = 0, n3 = 0, n4 = 0;
      n = sscanf( line, "%d %d %d %d %d %g %g %g %g", &num_nodes,
		  &n1, &n2, &n3, &n4, &r, &g, &b, &a);
      lst_off_quads->append( n1);
      lst_off_quads->append( n2);
      lst_off_quads->append( n3);
      lst_off_quads->append( n4);
      if ( n > 5) {
	lst_off_quad_colours->append( r);
	lst_off_quad_colours->append( g);
	lst_off_quad_colours->append( b);
	lst_off_quad_colours->append( a);
      }
    } else if ( num_nodes == 5) { // pentas
      int n1 = 0, n2 = 0, n3 = 0, n4 = 0, n5 = 0;
      n = sscanf( line, "%d %d %d %d %d %d %g %g %g %g", &num_nodes,
		  &n1, &n2, &n3, &n4, &n5, &r, &g, &b, &a);
      lst_off_pentas->append( n1);
      lst_off_pentas->append( n2);
      lst_off_pentas->append( n3);
      lst_off_pentas->append( n4);
      lst_off_pentas->append( n5);
      if ( n > 6) {
	lst_off_penta_colours->append( r);
	lst_off_penta_colours->append( g);
	lst_off_penta_colours->append( b);
	lst_off_penta_colours->append( a);
      }
    } else if ( num_nodes == 6) { // hexa
      int n1 = 0, n2 = 0, n3 = 0, n4 = 0, n5 = 0, n6 = 0;
      n = sscanf( line, "%d %d %d %d %d %d %d %g %g %g %g", &num_nodes,
		  &n1, &n2, &n3, &n4, &n5, &n6, &r, &g, &b, &a);
      lst_off_hexas->append( n1);
      lst_off_hexas->append( n2);
      lst_off_hexas->append( n3);
      lst_off_hexas->append( n4);
      lst_off_hexas->append( n5);
      lst_off_hexas->append( n6);
      if ( n > 7) {
	lst_off_hexa_colours->append( r);
	lst_off_hexa_colours->append( g);
	lst_off_hexa_colours->append( b);
	lst_off_hexa_colours->append( a);
      }
    } else {
      /* can not handle more than 4 vertices... */
      sprintf( buf, "can not handle faces with %d vertices.", num_nodes);
      set_err_str( buf);
    }
  }
  
  end_off_file( line, f_in);
  return lst_off_points->size();
}


int off_2_gid( const char *filename) {
  const char *name = strrchr( filename, '/');
#ifdef _WIN32
  if ( !name) name = strrchr( filename, '\\');
#endif // _WIN32
  if ( !name) name = filename;
  else name++;

  int ret = read_off_file( filename);

  int err = 0;
  if ( ret && lst_off_points->size() && 
       ( lst_off_lines->size() || lst_off_triangles->size() || lst_off_quads->size() ||
         lst_off_pentas->size() || lst_off_hexas->size())) {
    err = 0;
    GiD_NewPostProcess();
    int mesh_id = GiD_NewMesh( GID_POST, GIDPOST_NEW_MESH, name);
    GiD_SetColor( mesh_id, 0.8f, 0.8f, 0.8f, 1.0f);
    GiD_SetVertexPointer( mesh_id, GIDPOST_FLOAT, GIDPOST_VERTICES, 3, 
			  lst_off_points->size() / 3, 0, ( void *)( float *)( *lst_off_points));
    // create lines:
    if ( lst_off_lines->size()) {
      GiD_SetElementPointer( mesh_id, GIDPOST_FLOAT, GIDPOST_CONNECTIVITIES, GIDPOST_LINE,
                             lst_off_lines->size() / 2, 0, ( void *)( int *)( *lst_off_lines), 
			     0, NULL);
    }
    // create triangles:
    if ( lst_off_triangles->size()) {
      GiD_SetElementPointer( mesh_id, GIDPOST_FLOAT, GIDPOST_CONNECTIVITIES, GIDPOST_TRIANGLE,
                             lst_off_triangles->size() / 3, 0, ( void *)( int *)( *lst_off_triangles), 
			     0, NULL);
    }
    // create quadrilaterals:
    if ( lst_off_quads->size()) {
      GiD_SetElementPointer( mesh_id, GIDPOST_FLOAT, GIDPOST_CONNECTIVITIES, GIDPOST_QUADRILATERAL,
                             lst_off_quads->size() / 4, 0, ( void *)( int *)( *lst_off_quads), 
                             0, NULL);
    }
    GiD_EndMesh( mesh_id);
    
    // create pentahedras:
    if ( lst_off_pentas->size()) {
      // triangulate
      int num_triangs = ( lst_off_pentas->size() / 5) * 3;
      int *lst_triangs = ( int *)malloc( num_triangs * 3 * sizeof( int));
      int it = 0;
      for ( int i = 0; i < lst_off_pentas->size(); i += 5) {
        int n1 = ( *lst_off_pentas)[ i];
        int n2 = ( *lst_off_pentas)[ i + 1];
        int n3 = ( *lst_off_pentas)[ i + 2];
        int n4 = ( *lst_off_pentas)[ i + 3];
        int n5 = ( *lst_off_pentas)[ i + 4];
        // triang 1: n1 n2 n3
        lst_triangs[ it] = n1;
        lst_triangs[ it + 1] = n2;
        lst_triangs[ it + 2] = n3;
        // triang 2: n1 n3 n4
        lst_triangs[ it + 3] = n1;
        lst_triangs[ it + 4] = n3;
        lst_triangs[ it + 5] = n4;
        // triang 3: n1 n4 n5
        lst_triangs[ it + 6] = n1;
        lst_triangs[ it + 7] = n4;
        lst_triangs[ it + 8] = n5;
        it += 9;
      }
      char buf[ 8192];
      sprintf( buf, "%s ( pentahedrons)", name);
      int mesh_id = GiD_NewMesh( GID_POST, GIDPOST_NEW_MESH, name);
      GiD_SetColor( mesh_id, 0.8f, 0.8f, 0.8f, 1.0f);
      GiD_SetVertexPointer( mesh_id, GIDPOST_FLOAT, GIDPOST_VERTICES, 3, 
                            lst_off_points->size() / 3, 0, ( void *)( float *)( *lst_off_points));
      GiD_SetElementPointer( mesh_id, GIDPOST_FLOAT, GIDPOST_CONNECTIVITIES, GIDPOST_TRIANGLE,
                             num_triangs, 0, ( void *)lst_triangs, 
                             0, NULL);
      free( lst_triangs);
      GiD_EndMesh( mesh_id);
    }
    // create hexahedras:
    if ( lst_off_hexas->size()) {
      // triangulate
      int num_triangs = ( lst_off_hexas->size() / 5) * 4;
      int *lst_triangs = ( int *)malloc( num_triangs * 3 * sizeof( int));
      int it = 0;
      for ( int i = 0; i < lst_off_hexas->size(); i += 6) {
        int n1 = ( *lst_off_hexas)[ i];
        int n2 = ( *lst_off_hexas)[ i + 1];
        int n3 = ( *lst_off_hexas)[ i + 2];
        int n4 = ( *lst_off_hexas)[ i + 3];
        int n5 = ( *lst_off_hexas)[ i + 4];
        int n6 = ( *lst_off_hexas)[ i + 5];
        // triang 1: n1 n2 n3
        lst_triangs[ it] = n1;
        lst_triangs[ it + 1] = n2;
        lst_triangs[ it + 2] = n3;
        // triang 2: n1 n3 n4
        lst_triangs[ it + 3] = n1;
        lst_triangs[ it + 4] = n3;
        lst_triangs[ it + 5] = n4;
        // triang 3: n1 n4 n6
        lst_triangs[ it + 6] = n1;
        lst_triangs[ it + 7] = n4;
        lst_triangs[ it + 8] = n6;
        // triang 4: n4 n5 n6
        lst_triangs[ it + 6] = n4;
        lst_triangs[ it + 7] = n5;
        lst_triangs[ it + 8] = n6;
        it += 12;
      }
      char buf[ 8192];
      sprintf( buf, "%s ( hexahedrons)", name);
      int mesh_id = GiD_NewMesh( GID_POST, GIDPOST_NEW_MESH, name);
      GiD_SetColor( mesh_id, 0.8f, 0.8f, 0.8f, 1.0f);
      GiD_SetVertexPointer( mesh_id, GIDPOST_FLOAT, GIDPOST_VERTICES, 3, 
                            lst_off_points->size() / 3, 0, ( void *)( float *)( *lst_off_points));
      GiD_SetElementPointer( mesh_id, GIDPOST_FLOAT, GIDPOST_CONNECTIVITIES, GIDPOST_TRIANGLE,
                             num_triangs, 0, ( void *)lst_triangs, 
                             0, NULL);
      free( lst_triangs);
      GiD_EndMesh( mesh_id);
    }
    
    // now the other properties:
    int num_properties = ( lst_off_pnt_colours->size() ? 1 : 0) + 
      ( lst_off_line_colours->size() ? 1 : 0) +
      ( lst_off_triangle_colours->size() ? 1 : 0) +
      ( lst_off_quad_colours->size() ? 1 : 0) +
      ( lst_off_penta_colours->size() ? 1 : 0) +
      ( lst_off_hexa_colours->size() ? 1 : 0);
    bool both_properties = num_properties > 1;
    int num_elements = lst_off_lines->size() + lst_off_triangles->size() + lst_off_quads->size();

    // colours on vertices
    if ( lst_off_pnt_colours->size()) {
      // rellenamos si faltan:
      for ( int ic = lst_off_pnt_colours->size(); ic < lst_off_points->size(); ic++) {
	lst_off_pnt_colours->append( 0.1f);
	lst_off_pnt_colours->append( 0.1f);
	lst_off_pnt_colours->append( 0.1f);
	lst_off_pnt_colours->append( 1.0f);
      }
      char tmp_name[ 1024];
      if( both_properties) {
	strcpy( tmp_name, "vertex//colour");
      } else {
	strcpy( tmp_name, "colour");
      }
      int res_id = GiD_NewResult( "Geometry", 0.0, tmp_name, 0);
      GiD_SetResultPointer( res_id, GIDPOST_FLOAT, GIDPOST_RESULTS, GIDPOST_VECTOR_4, GIDPOST_NODAL, 
			    lst_off_pnt_colours->size(), 0, ( void *)( float *)( *lst_off_pnt_colours));
      GiD_EndResult( res_id);
    }

    // colours on lines
    if ( lst_off_line_colours->size()) {
      // rellenamos si faltan:
      for ( int ic = lst_off_line_colours->size(); ic < num_elements; ic++) {
	lst_off_pnt_colours->append( 0.1f);
	lst_off_pnt_colours->append( 0.1f);
	lst_off_pnt_colours->append( 0.1f);
	lst_off_pnt_colours->append( 1.0f);
      }
      char tmp_name[ 1024];
      if( both_properties) {
	strcpy( tmp_name, "line//colour");
      } else {
	strcpy( tmp_name, "colour");
      }
      int res_id = GiD_NewResult( "Geometry", 0.0, tmp_name, 0);
      GiD_SetResultPointer( res_id, GIDPOST_FLOAT, GIDPOST_RESULTS, GIDPOST_VECTOR_4, GIDPOST_ELEMENTAL, 
			    lst_off_line_colours->size() / 4, 0, ( void *)( float *)( *lst_off_line_colours));
      GiD_EndResult( res_id);
    }

    // colours on lines
    if ( lst_off_line_colours->size()) {
      // rellenamos si faltan:
      for ( int ic = lst_off_line_colours->size(); ic < num_elements; ic++) {
	lst_off_line_colours->append( 0.1f);
	lst_off_line_colours->append( 0.1f);
	lst_off_line_colours->append( 0.1f);
	lst_off_line_colours->append( 1.0f);
      }
      char tmp_name[ 1024];
      if( both_properties) {
	strcpy( tmp_name, "line//colour");
      } else {
	strcpy( tmp_name, "colour");
      }
      int res_id = GiD_NewResult( "Geometry", 0.0, tmp_name, 0);
      GiD_SetResultPointer( res_id, GIDPOST_FLOAT, GIDPOST_RESULTS, GIDPOST_VECTOR_4, GIDPOST_ELEMENTAL, 
			    lst_off_line_colours->size() / 4, 0, ( void *)( float *)( *lst_off_line_colours));
      GiD_EndResult( res_id);
    }

    // colours on triangles
    if ( lst_off_triangle_colours->size()) {
      // rellenamos si faltan:
      for ( int ic = lst_off_triangle_colours->size(); ic < num_elements; ic++) {
	lst_off_triangle_colours->append( 0.1f);
	lst_off_triangle_colours->append( 0.1f);
	lst_off_triangle_colours->append( 0.1f);
	lst_off_triangle_colours->append( 1.0f);
      }
      char tmp_name[ 1024];
      if( both_properties) {
	strcpy( tmp_name, "triangle//colour");
      } else {
	strcpy( tmp_name, "colour");
      }
      int res_id = GiD_NewResult( "Geometry", 0.0, tmp_name, 0);
      GiD_SetResultPointer( res_id, GIDPOST_FLOAT, GIDPOST_RESULTS, GIDPOST_VECTOR_4, GIDPOST_ELEMENTAL, 
			    lst_off_triangle_colours->size() / 4, 0, ( void *)( float *)( *lst_off_triangle_colours));
      GiD_EndResult( res_id);
    }

    // colours on quads
    if ( lst_off_quad_colours->size()) {
      // rellenamos si faltan:
      for ( int ic = lst_off_quad_colours->size(); ic < num_elements; ic++) {
	lst_off_quad_colours->append( 0.1f);
	lst_off_quad_colours->append( 0.1f);
	lst_off_quad_colours->append( 0.1f);
	lst_off_quad_colours->append( 1.0f);
      }
      char tmp_name[ 1024];
      if( both_properties) {
	strcpy( tmp_name, "quad//colour");
      } else {
	strcpy( tmp_name, "colour");
      }
      int res_id = GiD_NewResult( "Geometry", 0.0, tmp_name, 0);
      GiD_SetResultPointer( res_id, GIDPOST_FLOAT, GIDPOST_RESULTS, GIDPOST_VECTOR_4, GIDPOST_ELEMENTAL, 
			    lst_off_quad_colours->size() / 4, 0, ( void *)( float *)( *lst_off_quad_colours));
      GiD_EndResult( res_id);
    }

    // colours on pentas
    // needs redimension as we've triangulated the pentahedrons
    // ...

    // colours on hexas
    // needs redimension as we've triangulated the pentahedrons
    // ...
    
    // liberamos memoria:
    free_arrays();
    
  } else {
    err = 1;
  }
  
  
  // create_demo_triangs();

  return err;
}

/*************************************************************************
 * Interface functions to GiD
 *************************************************************************/

// must be of type:
// typedef int ( *_t_fncGiD_PluginImport)( const char *filename);
// called by GiD
extern "C" GID_DLL_EXPORT int GiD_PostImportFile( const char *filename) {

  char *path2file = strdup( filename);
  char *s = strrchr( path2file, '/');
  if ( !s) s = strrchr( path2file, '\\');
  if ( s) *( ++s) = '\0';
  else strcpy( path2file, "./");
  
  bool res = true;
  int ret_val = 1; // 1 - error, 0 - success

  // extern "C" GID_DLL_EXPORT void hello() {
  // printf( "-->hello.\n");
  // printf( "-->hello: %d\n", hola);
  printf( "importing %s: %s\n", filename, res ? "Ok" : "can not be read");
  if ( res) {
    // ret_val = create_demo_triangs();
    ret_val = off_2_gid( filename);
  } else {
    char buf[ 8192];
    sprintf( buf, "Problems reading the Polygon '%s'.", filename);
    set_err_str( buf);
    ret_val = 1;
  }
  free( path2file);
  return ret_val;
}

extern "C" GID_DLL_EXPORT const char *GiD_PostImportGetLibName( void) {
  return "Object File Format Import"; 
}

extern "C" GID_DLL_EXPORT const char *GiD_PostImportGetFileExtensions( void) {
  return "{{OFF files} {.off}} {{All files} {*}}";
}

extern "C" GID_DLL_EXPORT const char *GiD_PostImportGetDescription( void) {
  return "OFF ( Object File Format) import plugin for GiD"; 
}

extern "C" GID_DLL_EXPORT const char *GiD_PostImportGetErrorStr( void) {
  return _G_err_str; 
}
