/* -*- c++ -*- */
#ifndef _GID_PLUGIN_IMPORT_H_
#define _GID_PLUGIN_IMPORT_H_

#if defined( __linux__) || defined( __MACH__)

#  define GID_DLL_IMPORT
#  define GID_DLL_EXPORT

#elif defined( _WIN32)
#  define GID_DLL_IMPORT __declspec( dllimport)
#  define GID_DLL_EXPORT __declspec( dllexport)

#else
#  error "GID_DLL_IMPORT and GID_DLL_EXPORT should be defined properly for this system"

#endif // __linux__

typedef enum { GIDPOST_FLOAT = 1, GIDPOST_DOUBLE = 2} _t_gidBasicType;
typedef enum { GIDPOST_VERTICES = 1, GIDPOST_IDX_VERTICES = 2} _t_gidVertexListType;
typedef enum { GIDPOST_CONNECTIVITIES = 1, GIDPOST_IDX_CONNECTIVITIES = 2} _t_gidElementListType;
typedef enum { GIDPOST_RESULTS = 1, GIDPOST_IDX_RESULTS = 2} _t_gidResultListType;
typedef enum {
  GIDPOST_TRIANGLE = 1,      // 3 nodes
  GIDPOST_QUADRILATERAL = 2, // 4 nodes
  GIDPOST_LINE = 3,          // 2 nodes
  GIDPOST_TETRAHEDRON = 4,   // 4 nodes
  GIDPOST_HEXAHEDRON = 5,    // 6 nodes
  GIDPOST_POINT = 6,         // 1 node
  GIDPOST_PRISM = 7,         // 6 nodes
  GIDPOST_PYRAMID = 8,       // 5 nodes
  GIDPOST_SPHERE = 9,        // 1 node, in float_pointer: 1 radius
  GIDPOST_CIRCLE = 10        // 1 node, in float_pointer: 1 radius, 1 normal ( x, y, z)
} _t_gidElementType;

typedef enum {
  GIDPOST_SCALAR = 1,
  GIDPOST_VECTOR_2 = 2, // x, y
  GIDPOST_VECTOR_3 = 3, // x, y, z
  GIDPOST_VECTOR_4 = 4, // x, y, z, signed modulus
  GIDPOST_MATRIX_3 = 5, // Sxx, Syy, Sxy
  GIDPOST_MATRIX_4 = 6, // Sxx, Syy, Sxy, Szz
  GIDPOST_MATRIX_6 = 7, // Sxx, Syy, Sxy, Szz, Syz, Sxz
  GIDPOST_EULER = 8     //  Euler Angle 1, Euler Angle 2, Euler Angle 3 ?
} _t_gidResultType;

typedef enum { GIDPOST_NODAL = 1,
               GIDPOST_ELEMENTAL = 2,
               GIDPOST_GAUSSIAN = 3} _t_gidResultLocation;

typedef enum {
  GID_PRE = 1,
  GID_POST = 2
} _t_gidMode;

typedef enum {
  GIDPOST_NEW_MESH = 1, 
  GIDPOST_MERGE_MESH = 2, // at the moment, only new_mesh and merge_mesh supported
  GIDPOST_MULTIPLE_MESH = 3
} _t_gidMeshType;

typedef int ( *_t_fnc_GiD_NewPostProcess)( void);

typedef int ( *_t_fnc_GiD_NewMesh)( _t_gidMode gid_mode, _t_gidMeshType mesh_type, const char *name);
typedef int ( *_t_fnc_GiD_SetColor)( int id, float red, float green, float blue, float alpha);
// offsetNextElement = offset between one element and the next element (in bytes).
//                   = 0 for consecutive elements
typedef int ( *_t_fnc_GiD_SetVertexPointer)( int id,
                                          _t_gidBasicType basic_type,
                                          _t_gidVertexListType list_type,
                                          int num_components,
                                          int num_vertices,
                                          unsigned int offset_next_element,
                                          const void *pointer);
typedef int ( *_t_fnc_GiD_SetElementPointer)( int id,
                                           _t_gidBasicType basic_type, // for circles and sphere info
                                           _t_gidElementListType list_type,
                                           _t_gidElementType element_type,
                                           int num_elements,
                                           unsigned int offset_next_element,
                                           const void *pointer,
                                           unsigned int offset_float_data,
                                           const void *float_pointer);
// if id == 0, result defined for all meshes, useful for nodal results
typedef int ( *_t_fnc_GiD_NewResult)( const char *analysis_name, double step_value,
                                      const char *result_name, int mesh_id);
typedef int ( *_t_fnc_GiD_SetResultPointer)( int result_id,
                                          _t_gidBasicType basic_type,
                                          _t_gidResultListType list_type,
                                          _t_gidResultType result_type,
                                          _t_gidResultLocation result_location,
                                          int num_results,
                                          unsigned int offset_next_result,
                                          const void *pointer);
typedef int ( *_t_fnc_GiD_EndResult)( int id);
typedef int ( *_t_fnc_GiD_EndMesh)( int id);

struct Tcl_Interp;
typedef Tcl_Interp *( *_t_fnc_GiD_GetTclInterpreter)();

// in order to register the library within GiD, GiD needs:
// the library name --> const char *GiD_PostImportGetLibName( void)
//          returns a const char * with the library name, without spaces
//          for instance:  "GiD_PostFile"
// the extensions supported --> const char *GiD_PostImportGetFileExtensions( void)
//          returns a const char * with the extensions to be filtered in
//          the file dialog window, in tcl-list format, for instance:
//          "{{%s} {.res .msh .bin}} {{%s} {.lst}} {{%s} {*}}"
// a description --> const char *GiD_PostImportGetDescription( void)
//          returns a const char * with a description of the supported
//          file format. May be NULL
typedef const char *( *_t_fnc_GiD_PostImportGetLibName)( void);
typedef const char *( *_t_fnc_GiD_PostImportGetFileExtensions)( void);
typedef const char *( *_t_fnc_GiD_PostImportGetDescription)( void);
// prototype: int GiD_PostImportFile( const char *filename)
// --> to be called by GiD: returns 1 on error, 0 on success
typedef int ( *_t_fnc_GiD_PostImportFile)( const char *filename);
// prototype: const char *GiD_PostImportGetErrorStr( const char *filename)
// --> returns error string, if the above function returns an error
typedef const char *( *_t_fnc_GiD_PostImportGetErrorStr)( void);

#ifndef GID_EXTERN
#ifdef BUILD_GID_PLUGIN
#define GID_EXTERN GID_DLL_EXPORT
#else
#define GID_EXTERN extern
#endif
#endif

#ifdef __cplusplus
extern "C" {
#endif
  GID_EXTERN _t_fnc_GiD_NewPostProcess GiD_NewPostProcess;
  GID_EXTERN _t_fnc_GiD_NewMesh GiD_NewMesh;
  GID_EXTERN _t_fnc_GiD_SetColor GiD_SetColor;
  GID_EXTERN _t_fnc_GiD_SetVertexPointer GiD_SetVertexPointer;
  GID_EXTERN _t_fnc_GiD_SetElementPointer GiD_SetElementPointer;
  
  GID_EXTERN _t_fnc_GiD_NewResult GiD_NewResult;
  GID_EXTERN _t_fnc_GiD_SetResultPointer GiD_SetResultPointer;  
  GID_EXTERN _t_fnc_GiD_EndResult GiD_EndResult;
  
  GID_EXTERN _t_fnc_GiD_EndMesh GiD_EndMesh;
  GID_EXTERN _t_fnc_GiD_GetTclInterpreter GiD_GetTclInterpreter;
#ifdef __cplusplus
}
#endif

#endif // _GID_PLUGIN_IMPORT_H_
