#include "gid_plugin_import.h"
#include <tcl.h>

// in ply_gid_import.cpp
void set_err_str( const char *str);

static char *_G_ply_script =
"                                                                                                  ;  \
proc ::PlyImportProgressBarDestroy { w} {                                                          ;  \
    if { [ winfo exists $w]} {                                                                     ;  \
	destroy $w                                                                                 ;  \
    }                                                                                              ;  \
}                                                                                                  ;  \
                                                                                                   ;  \
set ::UpdatingPlyImportProgressBar 0                                                               ;  \
                                                                                                   ;  \
proc ::PlyImportProgressBar { numerator divisor { title \"\"} { parent .gid}} {                    ;  \
    if { $::UpdatingPlyImportProgressBar} { return}                                                ;  \
    set ::UpdatingPlyImportProgressBar 1                                                           ;  \
                                                                                                   ;  \
    set w $parent.___ply_import_progress_bar___                                                    ;  \
    if { \"$parent\" == \".\"} {                                                                   ;  \
        set w .___ply_import_progress_bar___                                                       ;  \
    }                                                                                              ;  \
                                                                                                   ;  \
    if { ![ winfo exists $w]} {                                                                    ;  \
	toplevel $w                                                                                ;  \
	if { \"$title\" != \"\"} { wm title $w $title}                                             ;  \
	set mode determinate                                                                       ;  \
	if { ( $numerator < 0) || ( $divisor < 0)} {                                               ;  \
	    set mode indeterminate                                                                 ;  \
	}                                                                                          ;  \
	set pb_width 360                                                                           ;  \
	set pb_height 48                                                                           ;  \
        ttk::label $w.lbl -text \"$title\" -width 30                                               ;  \
	ttk::progressbar $w.progressBar -mode $mode -length $pb_width                              ;  \
	grid $w.lbl -sticky new -padx 4 -pady 2                                                    ;  \
	grid $w.progressBar -sticky news -padx 4 -pady 2                                           ;  \
	grid rowconfigure $w 1 -weight 1                                                           ;  \
	grid columnconfigure $w 0 -weight 1                                                        ;  \
        set main_width [ winfo width $parent]                                                      ;  \
	set main_height [ winfo height $parent]                                                    ;  \
	set main_x [ winfo x $parent]                                                              ;  \
	set main_y [ winfo y $parent]                                                              ;  \
	set pb_x [ expr int( 0.5 + $main_x + 0.5 * ( $main_width - $pb_width))]                    ;  \
	if { $pb_x < 0} { set pb_x 100}                                                            ;  \
	set pb_y [ expr int( 0.5 + $main_y + 0.5 * ( $main_height - $pb_height))]                  ;  \
	if { $pb_y < 0} { set pb_y 100}                                                            ;  \
	wm geometry $w +${pb_x}+${pb_y}                                                            ;  \
	update                                                                                     ;  \
	update idletasks                                                                           ;  \
    }                                                                                              ;  \
    if { \"$title\" != \"\"} {                                                                     ;  \
        $w.lbl configure -text $title                                                              ;  \
        wm title $w $title                                                                         ;  \
    }                                                                                              ;  \
    $w.progressBar configure -maximum [ expr abs( $divisor)] -value [ expr abs( $numerator)]       ;  \
    raise $w                                                                                       ;  \
                                                                                                   ;  \
    set destroy_win 0                                                                              ;  \
    if { $divisor == 0 } {                                                                         ;  \
	set destroy_win 1                                                                          ;  \
    }                                                                                              ;  \
    if { ( $numerator >= 0) && ( $divisor >= 0) && ( $numerator >= $divisor)} {                    ;  \
	set destroy_win 1                                                                          ;  \
    }                                                                                              ;  \
    if { $destroy_win} {                                                                           ;  \
	after 1000 [ list PlyImportProgressBarDestroy $w]                                          ;  \
    }                                                                                              ;  \
    set ::UpdatingPlyImportProgressBar 0                                                           ;  \
}                                                                                                  ;  \
";

static bool _G_tcl_init = false;
static Tcl_Interp *_G_gid_interp = NULL;

void get_tcl_error( Tcl_Interp *interp) {
  const char *tcl_err = Tcl_GetStringResult( interp);
  printf( "Error: %s\n", tcl_err);
  set_err_str( tcl_err);
}

static int DefineTclScript( Tcl_Interp *interp) {
  int ret = Tcl_Eval( interp, _G_ply_script);
  if ( ret != TCL_OK) {
    get_tcl_error( interp);
  }
  return ret;
}

void init() {
  if ( !_G_tcl_init) {
    _G_gid_interp = GiD_GetTclInterpreter();
    DefineTclScript( _G_gid_interp);
    _G_tcl_init = true;
  }
}

void PlyProgressBar( int numerator, int divisor, const char *title = NULL) {
  if ( !_G_tcl_init) {
    init();
  }
  char buf[ 8192];
  if ( title && *title) {
    sprintf( buf, "::PlyImportProgressBar %d %d \"%s\"; update; update idletasks",
             numerator, divisor, title);
  } else {
    sprintf( buf, "::PlyImportProgressBar %d %d; update; update idletasks", numerator, divisor);
  }
  int ret = Tcl_Eval( _G_gid_interp, buf);
  if ( ret != TCL_OK) {
    get_tcl_error( _G_gid_interp);
  }
}
