namespace eval UNV {
}

proc UNV::ReadW { } {
    set filename [Browser-ramR file read .gid [= "Read UNV file"] \
	    {} {{{Ideas} {.unv }} {{Ideas} {.uff}} {{All files} {.*}}} ]
    if {$filename == ""} {
	return
    }
    UNV::Read $filename
}

proc UNV::OpenResultFile { filename } {
    variable fp_res
    if { [info exists fp_res] } {
	UNV::CloseResultFile
    }
    set fp_res [open $filename w]
    if { $fp_res != "" } {
	puts $fp_res "GiD Post Results File 1.0"
    } else {
	unset fp_res
    }
}

proc UNV::CloseResultFile { } {
    variable fp_res
    if { [info exists fp_res] } {
	close $fp_res
	unset fp_res
    }
}

proc UNV::PutResult { name analysis step info } {
    variable fp_res
    if { ![info exists fp_res] } {
	set modelname [GiD_Info Project ModelName]
	if { $modelname != "UNNAMED" } {
	    set filename [file join $modelname.gid [file tail $modelname].post.res]
	} else {
	    set filename [file join $::env(TMP) kk.post.res]            
	}
	OpenResultFile $filename
    }
     
    if { [info exists fp_res] } {
	puts $fp_res "Result \"$name\" \"$analysis\" $step Vector OnNodes"
	puts $fp_res "Values"
	foreach {nodenum values} $info {
	    puts $fp_res "$nodenum $values"
	}
	puts $fp_res "End Values"
    }
}

#create could be GEOMETRY or MESH
proc UNV::Read { filename {create MESH} } {
    variable node_label definition_coordinate_sytem displacement_coordinate_syteme node_color x y z
    variable line_num number_of_nodes line_color
    variable model_type analysis_type data_characteristic specific_data_type data_type ndv
    variable load_case_number mode_number
    variable re_eigenvalue im_eigenvalue re_modal_a im_modal_a re_modal_b im_modal_b
    if { ![file exists $filename] } { 
	WarnWin [= "file '%s' not exists" $filename]
	return 1
    }   
    set fp [open $filename r]
    if { $fp == "" } {
	WarnWin [= "Cannot open file '%s'" $filename]
	return 1
    }
    #offset to be added to the number if a previous model exists
    if { $create == "MESH" } {
	set offset [GiD_Info Mesh MaxNumNodes]
    } else {
	set offset [GiD_Info Geometry MaxNumPoints]
    }
    set info ""
    set layer Layer0
    set type 0
    set fail 0
    set state 0
    set record 1
    ::GidUtils::WaitState .gid
    while { ![eof $fp] } {
	gets $fp line
	if { $state == 0 } {
	    if { [string range $line 0 5] == "    -1"} {
		set state 1
	    }
	} elseif { $state == 1 } {
	    set type [string trim $line]
	    if { ![string is integer $type] } {
		set fail 1
		break
	    }
	    set state 2
	} elseif { $state == 2 } {    
	    if { [string range $line 0 5] == "    -1"} {
		if { $type == 15 } {
		    foreach {num x y z} $info {
		        incr num $offset
		        if { $create == "MESH" } {
		            GiD_Mesh create node $num "$x $y $z"
		        } else {
		            GiD_Geometry create point $num $layer $x $y $z
		        }
		    }
		} elseif { $type == 55 } {
		    UNV::PutResult  "Structural" "Complex Eigenvalue $mode_number" 1 $info                    
		} elseif { $type == 82 } {
		    set n1 0
		    foreach n $info {
		        if { $n>0 } {
		            if { $n1>0 } {
		                set n2 $n
		                incr n2 $offset
		                if { $create == "MESH" } {
		                    GiD_Mesh create element append linear 2 "$n1 $n2"
		                } else {
		                    GiD_Geometry create line append stline $layer $n1 $n2   
		                }
		                set n1 $n2
		            } else {
		                set n1 $n
		                incr n1 $offset
		            }
		        } else {
		            set n1 0                                                      
		        }
		    }  
		}
		set info ""
		set state 0
		set record 1
		continue
	    }       
	    switch $type {
		15 {
		    if { [scan $line {%10i%10i%10i%10i%13e%13e%13e} node_label definition_coordinate_sytem displacement_coordinate_syteme node_color x y z] != 7 } {
		        set fail 1                   
		    }
		    if { $fail } {
		        break
		    }
		    lappend info $node_label $x $y $z
		}
		55 {
		    switch $record {
		        1 - 2 - 3 - 4 - 5 {
		            #headers, ignore them a this moment                            
		        }
		        6 {
		            if { [scan $line {%6i%6i%6i%6i%6i%6i} model_type analysis_type data_characteristic specific_data_type data_type ndv] != 6 } {
		                set fail 1
		            }
		        }
		        7 {
		            if { $analysis_type == 3} {
		                #Complex Eigenvalue
		                if { [scan $line {%10i%10i%10i%10i} dummy2 dummy6 load_case_number mode_number] != 4 } {
		                    set fail 1                                   
		                }
		            } else {
	  
		            }                                               
		        }
		        8 {
		            if { [scan $line {%13e%13e%13e%13e%13e%13e} re_eigenvalue im_eigenvalue re_modal_a im_modal_a re_modal_b im_modal_b] != 6 } {
		                set fail 1
		            }               
		        }
		        9 {
		            if { [scan $line {%10i} node_number] != 1 } {
		                set fail 1                       
		            }
		            lappend info $node_number
		        }
		        default {
		            if { [scan $line {%13e%13e%13e%13e%13e%13e} re_x im_x re_y im_y re_z im_z] != 6 } {
		                set fail 1
		            }
		            lappend info [list $re_x $re_y $re_z]
		            incr record -2 ;#to set next record as 9                                   
		        }                       
		    }
		    if { $fail } {
		        break
		    }
		    incr record

		}
		82 {
		    switch $record {
		        1 {
		            if { [scan $line {%10i%10i%10i} line_num number_of_nodes line_color] != 3 } {
		                set fail 1                              
		            }                                                   
		        }
		        2 {
		            set name $line                           
		        }
		        default {
		            set info [concat $info $line]
		        }
		    }
		    incr record
		    if { $fail } {
		        break
		    }
		}
		default {
		    
		}
	    }
	}
    }
    close $fp

    UNV::CloseResultFile

    ::GidUtils::EndWaitState .gid
    ::GidUtils::SetWarnLine [= "File read"]
    if { $create == "MESH" } {
	if { [GiD_Info project ViewMode] != "MESHUSE" } {
	    GiD_Process MEscape Meshing MeshView
	}
    } else {
	if { [GiD_Info project ViewMode] != "GEOMETRYUSE" } {
	    GiD_Process MEscape Geometry
	}
    }
    #GiD_Redraw
    GiD_Process Zoom Frame
    return 0
}

proc UNV::AddToMenu { } {
    if { [GidUtils::AreWindowsDisabled] } {
	return
    }
    if { [GiDMenu::GetOptionIndex Files [list Import [= "UNV"]...] PRE] != -1 } {
	return
    }

    #try to insert this menu after the word "Files->Import->XYZ nodes"
    set position [GiDMenu::GetOptionIndex Files [list Import "XYZ nodes"] PRE]
    if { $position == -1 } {
	set position end
    }
    GiDMenu::InsertOption Files [list Import [= "UNV"]...] $position PRE UNV::ReadW "" "" insertafter _
}
 

#invoke this menus changes
UNV::AddToMenu
GiDMenu::UpdateMenus

#register the proc to be automatically called when re-creating all menus (e.g. when doing files new)
GiD_RegisterPluginAddedMenuProc UNV::AddToMenu
 

#register the proc to be automatically called when dropping a file
GiD_RegisterExtensionProc ".unv .uff" PRE UNV::Read

