
proc InitGIDProject { dir } {
    global ProgramName VersionNumber cmas2d_CompassLIBPriv 
    set ProgramName cmas2d_CompassLIB
    set VersionNumber 1.0   
    global ProgramTag GIDDEFAULT
    
    package require tdom
    dom parse [tDOM::xmlReadFile [file join $dir cmas2d_CompassLIB.xml]] doc
    
    set ProgramName [$doc selectNodes string(Infoproblemtype/Program/Name)]
    set VersionNumber [$doc selectNodes string(Infoproblemtype/Program/Version)]  
    set CompassLIBid [$doc selectNodes string(Infoproblemtype/Program/CompassLIBid)]
    set SplashImage [$doc selectNodes string(Infoproblemtype/Program/SplashImage)]
    
    catch { unset cmas2d_CompassLIBPriv }
    set cmas2d_CompassLIBPriv(problemtypedir) $dir  

    # CHECKING COMPATIBILITY BETWEEN PROBLEM TYPE AND GiD 
    # (The use if GiD_info mesh must require version 9)    
    set VersionRequired "9.0.0"
    set comp -1
    catch { 
	set comp [::GidUtils::VersionCmp $VersionRequired]
    }
    if { $comp < 0 } {
	WarnWin [= "Error: %s Interface requires GiD %s or later." $ProgramName $VersionRequired]
    }

    GidChangeDataLabel "Data units" ""
    GidChangeDataLabel "Interval" ""
    GidChangeDataLabel "Conditions" ""
    GidChangeDataLabel "Materials" ""
    GidChangeDataLabel "Interval Data" ""
    GidChangeDataLabel "Problem Data" ""
    GidChangeDataLabel "Local axes" ""

    GidAddUserDataOptions "---" "" 3
    GidAddUserDataOptions [= "Groups"] "gid_groups_conds::open_groups .gid window" 5
    GidAddUserDataOptions [= "Data"] \
	"gid_groups_conds::open_conditions window" 6
    GidAddUserDataOptions [= "Data (internal)"] \
	"gid_groups_conds::open_conditions menu" 7

    GidAddUserDataOptions "---" "" 10
    GidAddUserDataOptionsMenu [= "Local axes"] \
	"gid_groups_conds::local_axes_menu %W" 11    
    
    GidAddUserDataOptions "---" "" 12
	
    GidAddUserDataOptions [= "Check properties entities"] \
	[list gid_groups_conds::check_properties_unicity .gid {/*/container[@n="Properties"]}] 13
    
    set xp {/*/container[@n="Properties"]/container[@n="materials"]}
    GidAddUserDataOptions [= "Import/export materials"] \
	[list gid_groups_conds::import_export_materials .gid $xp] 1
	  
    ::UpdateMenus

    if { [lsearch -exact $::auto_path [file join $dir scripts]] == -1 } {
	lappend ::auto_path [file join $dir scripts] [file join $dir exec] \
	    [file join $GIDDEFAULT problemtypes common_lib]
    }
    
    cmas2d_CompassLIB::load_gid_groups_conds      
 
    gid_groups_conds::SetProgramName $ProgramName
    gid_groups_conds::SetCompassLIBid $CompassLIBid
    gid_groups_conds::SetLibDir [file join $dir exec] 

    gid_groups_conds::begin_problemtype [file join $dir cmas2d_CompassLIB_default.spd] \
	[cmas2d_CompassLIB::Givecmas2d_CompassLIBDefaultsFile]
    
    #We create the initial window (random surface creation)    
    CreateWindow $dir
}

proc EndGIDProject {} {
    gid_groups_conds::end_problemtype [cmas2d_CompassLIB::Givecmas2d_CompassLIBDefaultsFile]
}

proc InitGIDPostProcess {} {
    global cmas2d_CompassLIBPriv
    
    gid_groups_conds::close_all_windows
    gid_groups_conds::open_post check_default
    set dir $cmas2d_CompassLIBPriv(problemtypedir)  
}

proc EndGIDPostProcess {} {
    gid_groups_conds::close_all_windows
    gid_groups_conds::open_conditions check_default
}
 
# Load GiD project files (initialise XML Tdom structure)
proc LoadGIDProject { filespd } {
    global cmas2d_CompassLIBPriv
    if { ![file exists $filespd] } { return }
    set versionPT [gid_groups_conds::give_data_version]
    gid_groups_conds::open_spd_file $filespd
    set versionData [gid_groups_conds::give_data_version]
    if { [package vcompare $versionPT $versionData] == 1 } {
	after idle cmas2d_CompassLIB::upgrade_problemtype
    }
}
# Save GiD project files (save XML Tdom structure to spd file)
proc SaveGIDProject { filespd } {
    global cmas2d_CompassLIBPriv
    gid_groups_conds::save_spd_file $filespd
}

proc AfterTransformProblemType { filename oldproblemtype newproblemtype } {

    set spd_file [file join $filename.gid [file tail $filename].spd]
    return [gid_groups_conds::transform_problemtype $spd_file]
}

proc AfterWriteCalcFileGIDProject { filename errorflag } {
    global cmas2d_CompassLIBPriv       

    if { ![info exists gid_groups_conds::doc] } {
	WarnWin [= "Error: data not OK"]
	return
    }

    if { [info exists cmas2d_CompassLIBPriv(before_calc_handler_list)] } {
	foreach i $cmas2d_CompassLIBPriv(before_calc_handler_list) {
	    uplevel #0 $i [list $gid_groups_conds::doc $filename]
	}
    }
    set err [catch { b_write_calc_file::writeCalcFile $gid_groups_conds::doc \
		$filename } ret]
    if { $err } {       
	snit_messageBox -parent .gid -message \
	    [= "Error when preparing data for analysis (%s)" $ret]
	return "-cancel-"
    }
    return $ret
}

namespace eval cmas2d_CompassLIB {}

proc cmas2d_CompassLIB::load_gid_groups_conds {} { 
    package require compass_utils 1.4  
    package require gid_groups_conds
}

proc cmas2d_CompassLIB::Givecmas2d_CompassLIBDefaultsFile {} {
    global ProgramName VersionNumber
    set filename [GiveGidDefaultsFile]
    set dirname [file dirname $filename]
    set extname [file extension $filename]
    set cmas2d_CompassLIBname $ProgramName$VersionNumber
    if { $::tcl_platform(platform) == "windows" } {
	return [file join $dirname $cmas2d_CompassLIBname.ini]
    } else {
	return [file join $dirname .$cmas2d_CompassLIBname.ini]
    }
}

proc cmas2d_CompassLIB::upgrade_problemtype {} {

    set w [dialogwin_snit .gid._ask -title [= "Action"] -entrytext \
	    [= "The model needs to be upgraded. Do you want to upgrade to new version?"]]
    set action [$w createwindow]
    destroy $w
    if { $action < 1 } { return }
    set project [lindex [GiD_Info Project] 0]
    
    GiD_Process escape escape escape escape Data Defaults TransfProblem \
	$project
}

proc cmas2d_CompassLIB::LocalAxesMenu { menu } {
    
    #if { [$menu index end] ne "none" } { return }

    $menu delete 0 end

    set local_axes [GiD_Info localaxes]

    foreach i [list Point Line Surface] name [list [= Points] [= Lines] [= Surfaces]] {
	$menu add cascade -label $name -menu $menu.m$i
	destroy $menu.m$i
	set m [menu $menu.m$i -tearoff 0]
	if { [lsearch "Line Surface" $i] != -1 } {
	    $m add command -label [= "Assign Automatic"] -command [list GiD_Process \
		    escape escape escape escape Data Conditions AssignCond ${i}_Local_axes \
		    change -Automatic-]
	    $m add command -label [= "Assign Automatic alt"] -command [list GiD_Process \
		    escape escape escape escape Data Conditions AssignCond ${i}_Local_axes \
		    change -Automatic_alt-]
	    $m add separator
	}
	set idx 0
	foreach j $local_axes {
	    $m add command -label [= "Assign '%s'" $j] -command [list GiD_Process \
		    escape escape escape escape Data Conditions AssignCond ${i}_Local_axes \
		    change $j]
	    incr idx
	}
	if { $idx } { $m add separator }
	$m add command -label [= "Unassign"] -command [list GiD_Process \
		escape escape escape escape Data Conditions AssignCond ${i}_Local_axes \
		Unassign]
	
	$m add separator
	$m add command -label [= "Draw"] -command [list GiD_Process \
		escape escape escape escape Data Conditions DrawCond -LocalAxes- \
		${i}_Local_axes -draw-]
    }
    set ns [list [= "Define#C#menu"] --- [= "Draw#C#menu"] [= "Draw all#C#menu"] \
	    --- [= "Delete#C#menu"] [= "Delete all#C#menu"]]
    set cs {
	"Data LocalAxes DefineLocAxes"
	{} "Data LocalAxes DrawLocAxes"
	"Data LocalAxes DrawLocAxes -All-"
	{} "Data LocalAxes DeleteLA"
	"Data LocalAxes DeleteAllLA"
    }
    $menu add separator
    foreach n $ns c $cs {
	if { $n eq "---" } {
	    $menu add separator
	} else {
	    $menu add command -label $n -command [concat "GiD_Process escape escape \
		    escape escape" $c]
	}
    }
}


#######################################################
#    Add items to control-1 contextual menu 
#######################################################

proc cmas2d_CompassLIB::AddDataItemsToMenu { menu whatuse type entity } {

    set DisableGraphics [.central.s disable graphics]
    set DisableWarnLine [.central.s disable warnline]
    
    .central.s disable graphics 1
    .central.s disable warnline 1

    switch $type {
	Points { set cnds point_groups }
	Lines { set cnds line_groups }
	Surfaces { set cnds surface_groups }
	Volumes { set cnds volume_groups }
	Nodes { set cnds point_groups }
	Elements { set cnds "line_groups surface_groups volume_groups" }
    }
    switch $type Points - Lines - Surfaces - Volumes { set where geometry } Nodes - \
	Elements { set where mesh }

    set doc $gid_groups_conds::doc

    set submenu ""
    set needsseparator 0
    foreach cnd $cnds {
	set ret [GiD_Info Conditions $cnd $where $entity]
	foreach i $ret {
	    foreach "num face - group" $i break
	    if { $num == 0 } { continue }
	    if { $num eq "E" } { set num $face }
	    if { $submenu == "" } {
		set j 1
		while { [winfo exists $menu.m$j] } { incr j }
		set submenu [menu $menu.m$j -tearoff 0]
		set text [= "Groups"]
		$menu add cascade -label $text -menu $submenu
		set needsseparator 1
	    }
	    set xp [format_xpath {//group[@n=%s and ancestor::condition]} \
		    $group]
	    set domNodes [$doc selectNodes $xp]

	    set txt [= "Group: %s" $group]
	    if { ![llength $domNodes] } {
		ContextualEntity::AddToMenu $submenu $txt \
		    [list cmas2d_CompassLIB::_AddDataItemsToMenuItem $group]
	    } else {
		set j 1
		while { [winfo exists $submenu.m$j] } { incr j }
		set subsubmenu [menu $submenu.m$j -tearoff 0]
		$submenu add cascade -label $txt -menu $subsubmenu
		ContextualEntity::AddToMenu $subsubmenu $txt \
		    [list cmas2d_CompassLIB::_AddDataItemsToMenuItem $group]
		foreach domNode $domNodes {
		    set cndNode [$domNode selectNodes ancestor::condition]
		    set blockNode [$domNode selectNodes {ancestor::blockdata[@sequence='1']}]
		    if { $blockNode ne "" } {
		        set txt [= "Condition: %s - %s" [$blockNode @name] [$cndNode @pn]]
		    } else {
		        set txt [= "Condition: %s" [$cndNode @pn]]
		    }
		    ContextualEntity::AddToMenu $subsubmenu $txt \
		        [list cmas2d_CompassLIB::_AddDataItemsToMenuItemD $group \
		            $domNode]
		}
	    }
	}
    }
    if { $needsseparator } { $menu add separator }
    
    if { !$DisableGraphics } { .central.s disable graphics 0 }
    if { !$DisableWarnLine } { .central.s disable warnline 0 }

}

proc cmas2d_CompassLIB::_AddDataItemsToMenuItem { group } {
    gid_groups_conds::open_groups .gid window_force window
    set wg $gid_groups_conds::gid_group_var
    set tree [$wg givetreectrl]
    
    $tree selection clear
    foreach item [$tree item children root] {
	if { [$tree item text $item 0] eq $group } {
	    $tree selection add $item
	    break
	}
    }
}

proc cmas2d_CompassLIB::_AddDataItemsToMenuItemD { group domNode } {

    set what [gid_groups_conds::open_conditions window_type]
    if { $what eq "none" } { set what window }

    set xpath [gid_groups_conds::nice_xpath $domNode]
    gid_groups_conds::open_conditions $what -select_xpath $xpath
}

############################################################################
#    Additional procs for this problemtype (kept from the old version)
############################################################################

proc CreateWindow {dir} { 
    if { [GidUtils::AreWindowsDisabled] } {
	return
    }
 
    set w .gid.win_example
    if { [ winfo exist $w]} {
	destroy $w        
    }   
    
    toplevel $w
    wm withdraw $w

    set w $w
    
    set x [expr [winfo rootx .gid]+[winfo width .gid]/2-[winfo width $w]/2]
    set y [expr [winfo rooty .gid]+[winfo height .gid]/2-[winfo height $w]/2]
    
    wm geom $w +$x+$y
    wm transient $w .gid    

    InitWindow $w [= "CMAS2D_CompassLIB.TCL - Example tcl file"] ExampleCMAS "" \
	"" 1

    frame $w.top
    label $w.top.title_text -text [= " TCL window example for CMAS2D_CompassLIB problem type "]
   
    frame $w.information  -relief ridge -bd 2 
    label $w.information.path        -text [= " Problem Type path: $dir "]
    label $w.information.help   -text [= " This problemtype allows the user to compute the center of mass of a given 2D geometry. \
      \n The geometry can be generated by the user but he can also take a random surface suggested by GiD (next button) "]

    frame $w.bottom
    button $w.bottom.start -text [= "CONTINUE"] \
	   -height 1 -width 14 -command "destroy $w"
    button $w.bottom.random -text [= "RANDOM SURFACE"] \
	    -height 1 -width 20 -command "CreateRandomSurface $w"

    pack $w.top.title_text -pady 10
    pack $w.information.path $w.information.help -side top -anchor w
    
    pack $w.bottom.start $w.bottom.random -side left -anchor center
    pack $w.top 
    pack $w.information -expand yes -fill both
    pack $w.bottom -side top -padx 6 -pady 10 -ipady 2
}



proc CreateRandomSurface {w} {
    set ret [tk_dialogRAM $w.dialog [= "Warning!!"] \
		 [= "Warning: this will create a nurbs surface in your \
	current project"] "" 1 [= "Ok"] [= "Cancel"]]
    
    if {$ret ==0} {
	Create_surface
	destroy $w
    }
}

proc Create_surface {} {
    set a_x [expr rand()*10]
    set a_y [expr rand()*10]
    
    set b_x [expr $a_x + rand()*10]
    set b_y [expr $a_y + rand()*10]
    
    set c_x [expr $b_x + rand()*10]
    set c_y [expr $b_y - rand()*10]
    
    if {$a_y < $c_y} {
	set d_y [expr $a_y - rand()*10]
	set d_x [expr $a_x + rand()*10]
    } else {
	set d_y [expr $c_y - rand()*10] 
	set d_x [expr $c_x - rand()*10]
    }
    
    GiD_Process escape escape escape escape geometry create line \
	$a_x,$a_y,0.000000 \
	$b_x,$b_y,0.000000 \
	$c_x,$c_y,0.000000 \
	$d_x,$d_y,0.000000 \
	close
    GiD_Process escape escape escape escape geometry create NurbsSurface Automatic \
	4 escape
    GiD_Process zoom frame escape escape escape escape 
 }

