#old deprecated commands

#compatibility with old versions without namespace

proc GiDVersionNumber { VersionString } {
    return [GidUtils::VersionNumber $VersionString]
}

proc GiDVersionSubchar { VersionString } {
    return [GidUtils::VersionSubchar $VersionString]
}

proc GiDVersionBetaNumber { VersionString } {
    return [GidUtils::VersionBetaNumber $VersionString]
}

proc GiDVersionCmp { Version } {
    return [GidUtils::VersionCmp $Version]
}

proc CorrectExp { realnum  } {
    return [MathUtils::CorrectExp $realnum]
}


#NOTE:It exist from GiD7.4.7b a new GiD-tcl command, more powerful than CorrectExp
#     this command is:
#     GiD_FormatReal <format> <realnum> ?forcewidth|forcewidthnastran?
#     <format> is a 'C' style format, and if set "forcewidth" the minimum size of this format is considered
#     also as the final size (this is not the 'C' default).
#     An special option is "forcewidthnastran", to allow to remove the 'e' character,
#     sample: format="%8.3e" realnum=-2.345e-13
#             without width option    return -2.345e-13 (10 digits, not 8)
#             with forcewidth         return -2.3e-13 (8 digits, but only 2 significant digits)
#             with forcewidthnastran  return -2.34-13 (8 digits, without 'e', but with 3 significant digits)
#
# CorrectExp return two digits exponent: e.g 1.5e+045 return 1.5e+45
namespace eval MathUtils {
}

proc MathUtils::CorrectExp { realnum  } {
    set prev ""
    set expsgn ""
    set a [regexp "(.*)?(\[eE\])(\[\+\-\])?(\[0-9\]+)" $realnum match prev expchar expsgn expnum ]
    if { $a } {
        if { [string length $expnum] ==3 && [string index $expnum 0] == 0 } {
            set expnum [string range $expnum 1 2]
        }
        return "$prev$expchar$expsgn$expnum"
    } else {
        return $realnum
    }
}

#old manage menus commands

# Updates changes
proc UpdateMenus {} {
    return [GiDMenu::UpdateMenus]
}

# to try to get the current translated menu name of an old translated menu name
# because menus are renamed, and old problemtypes can use the old name
proc ConvertDeprecatedTranslatedMenuNameToCurrent { menu_name_translated {prepost PREPOST} } {
    if { $prepost=="PRE" || $prepost=="PREPOST" } {
        #back compatibility: Meshing menu renamed to Mesh in 7.5.9b
        if { $menu_name_translated == [_ "Meshing"] } {
            return [_ "Mesh#C#menu"]
        }
        #back compatibility: menus renamed from oldname to oldname#C#menu
        set oldmenunames [list [_ "Files"] [_ "View#C#Verb"] [_ "Geometry"] [_ "Utilities"] [_ "Data"] [_ "Mesh"] \
                [_ "Calculate"] [_ "Help"] ]
        set i [lsearch $oldmenunames $menu_name_translated]
        if { $i != -1 } {
            set newmenunames [list [_ "Files#C#menu"] [_ "View#C#menu,Verb"] [_ "Geometry#C#menu"] \
                    [_ "Utilities#C#menu"] [_ "Data#C#menu"] [_ "Mesh#C#menu"] \
                    [_ "Calculate#C#menu"] [_ "Help#C#menu"] ]
            return [lindex $newmenunames $i]
        }
    }
    if { $prepost=="POST" || $prepost=="PREPOST" } {
        #back compatibility: menus renamed from oldname to oldname#C#menu
        set oldmenunames [list [_ "Files"] [_ "View#C#Verb"] [_ "Utilities"] \
                [_ "Do cuts"] [_ "View results"] [_ "Options"] \
                [_ "Window"] [_ "Help"]]
        set i [lsearch $oldmenunames $menu_name_translated]
        if { $i != -1 } {
            set newmenunames [list [_ "Files#C#menu"] [_ "View#C#menu,Verb"] [_ "Utilities#C#menu"] \
                    [_ "Do cuts#C#menu"] [_ "View results#C#menu"] [_ "Options#C#menu"] \
                    [_ "Window#C#menu"] [_ "Help#C#menu"]]
            return [lindex $newmenunames $i]
        }
    }
    return $menu_name_translated
}

# Creates a new menu. New menus are inserted between the 'Calculate' and 'Help' menu
# Arg 'prepost' can have these values:
# "PRE" to create the menu only in the preprocess
# "POST" to create the menu only in the postprocess
# or "PREPOST" to create the menu in the pre and postprocess
proc CreateMenu {new_menu_name prepost} {

  if {$prepost!="PRE" && $prepost!="POST" && $prepost!="PREPOST"} {
    return -code error [_ "Wrong arg. Must be PRE, POST or PREPOST."]}


  if {$prepost eq "PRE"} {
    set where [list ""]
  } elseif {$prepost eq "POST"} {
    set where [list "P"]
  } else {
    set where [list "" "P"]
  }

  foreach c $where {
    upvar \#0 MenuNames$c MenuNames
    upvar \#0 MenuEntries$c MenuEntries
    upvar \#0 MenuCommands$c MenuCommands
    upvar \#0 MenuAcceler$c MenuAcceler
    upvar \#0 MenuIcons$c MenuIcons

    set length   [llength $MenuNames]
    set pos   [expr [llength $MenuNames]-1]

    set MenuNames [linsert $MenuNames $pos $new_menu_name]
    set MenuEntries($length)  $MenuEntries($pos)
    set MenuCommands($length) $MenuCommands($pos)
    set MenuAcceler($length)  $MenuAcceler($pos)
    set MenuIcons($length)  $MenuIcons($pos)

    set MenuEntries($pos)  [list ""]
    set MenuCommands($pos) [list ""]
    set MenuAcceler($pos)  [list ""]
    set MenuIcons($pos)  [list ""]
  }
}


# -----------------------------------------------------------------------------
# DeleteMenu --
#
#  Delete a toplevel menu
#  menu_name must be [_ "gid_englishmenuname" ]  or [= "problemtype_englishmenuname"]
#  prepost must be  PRE , POST or PREPOST
#  for example:
#    DeleteMenu [_ "Mesh"] PRE
#
#  Remember call UpdateMenus.
# -----------------------------------------------------------------------------
proc DeleteMenu {menu_name prepost} {
  global MenuNames MenuEntries MenuCommands MenuAcceler
  global MenuNamesP MenuEntriesP MenuCommandsP MenuAccelerP

  if {$prepost!="PRE" && $prepost!="POST" && $prepost!="PREPOST"} {
    return -code error [_ "Wrong arg. Must be PRE, POST or PREPOST."]
  }
  #back compatibility: menus were renamed
  set menu_name [ConvertDeprecatedTranslatedMenuNameToCurrent $menu_name $prepost]

  if {$prepost eq "PRE"} {
    set where [list "" "PRE"]
  } elseif {$prepost eq "POST"} {
    set where [list "P" "POST"]
  } else {
    set where [list "" "PRE" "P" "POST"]
  }
  foreach {c desc} $where {
    upvar \#0 MenuNames$c MenuNames
    upvar \#0 MenuEntries$c MenuEntries
    upvar \#0 MenuCommands$c MenuCommands
    upvar \#0 MenuAcceler$c MenuAcceler
    upvar \#0 MenuIcons$c MenuIcons

    set i [lsearch $MenuNames $menu_name]
    if { $i == "-1" } {
        #try to search the translated word, for old problemtypes not updated to translation
        #set i [GiD_FindMenuIndex $menu_name $desc _]
	set i [ GiDMenu::_FindIndex $menu_name $desc _]
        if { $i == "-1" } {
            WarnWin [_ "ERROR: Menu %s doesn't exist in $desc." $menu_name]
            return
        } else {
            #::GidUtils::SetWarnLine [_ "Warning: This problemtype must be updated for translation"]
        }
    }
    set MenuNames [lreplace $MenuNames $i $i]
    array unset MenuEntries $i,*
    array unset MenuCommands $i,*
    array unset MenuAcceler $i,*
    array unset MenuIcons $i,*

    set l [llength $MenuNames]
    for {incr i} {$i <= $l} {incr i} {
      foreach item [array names MenuEntries ${i}*] {
        set new_item [regsub ${i}* $item [expr $i-1]]
        set MenuEntries($new_item) $MenuEntries($item)
        unset MenuEntries($item)
      }
      foreach item [array names MenuCommands ${i}*] {
        set new_item [regsub ${i}* $item [expr $i-1]]
        set MenuCommands($new_item) $MenuCommands($item)
        unset MenuCommands($item)
      }
      foreach item [array names MenuAcceler ${i}*] {
        set new_item [regsub ${i}* $item [expr $i-1]]
        set MenuAcceler($new_item) $MenuAcceler($item)
        unset MenuAcceler($item)
      }
      foreach item [array names MenuIcons ${i}*] {
          set new_item [regsub ${i}* $item [expr $i-1]]
          set MenuIcons($new_item) $MenuIcons($item)
          unset MenuIcons($item)
      }
    }
    array unset MenuEntries $l,*
    array unset MenuCommands $l,*
    array unset MenuAcceler $l,*
    array unset MenuIcons $l,*
    array unset MenuEntries $l
    array unset MenuCommands $l
    array unset MenuAcceler $l
    array unset MenuIcons $l
  }
}

# Creates a new option for a given menu in a given
# position. (positions start at 0), "end" can be  used
# By default, if it already exists an option in the given position,
# that option is substitued by the new one.
# if Arg ins_repl is "insert" the new menu option is inserted
# Arg 'command' is the command called when the menu option
# is selected.
# Arg 'prepost' can have these values:
# "PRE" to insert the option in the preprocess menus
# "POST" to insert the option in the postprocess menus
# or "PREPOST" to insert the option in the pre and postprocess menus
# Arg acceler is an optional key accelerator, like "Control-s"
# Arg icon is the name of an small icon (16x16 pixels)

proc InsertMenuOption {menu_name option_name position command prepost {ins_repl "replace"} \
        {acceler ""} {icon ""} } {

    if {$prepost!="PRE" && $prepost!="POST" && $prepost!="PREPOST"} {
        return -code error [_ "Wrong arg. Must be PRE, POST or PREPOST."]
    }

    set $ins_repl [string tolower $ins_repl]
    if {$ins_repl != "replace" && $ins_repl != "insert"} {
        return -code error [_ "Wrong arg. Must be replace or insert."]
    }
    #back compatibility: menus were renamed
    set menu_name [ConvertDeprecatedTranslatedMenuNameToCurrent $menu_name $prepost]

    if {$prepost eq "PRE"} {
        set where [list "" "PRE"]
    } elseif {$prepost eq "POST"} {
        set where [list "P" "POST"]
    } else {
        set where [list "" "PRE" "P" "POST"]
    }
    foreach {c desc} $where {
        upvar \#0 MenuNames$c MenuNames
        upvar \#0 MenuEntries$c MenuEntries
        upvar \#0 MenuCommands$c MenuCommands
        upvar \#0 MenuAcceler$c MenuAcceler
        upvar \#0 MenuIcons$c MenuIcons

        set i [lsearch $MenuNames $menu_name]
        if { $i == "-1" } {
            #try to search the translated word, for old problemtypes not updated to translation
            #set i [GiD_FindMenuIndex $menu_name $desc _]
	    set i [ GiDMenu::_FindIndex $menu_name $desc _]
            if { $i == "-1" } {
                WarnWin [_ "ERROR: Menu %s doesn't exist in $desc." $menu_name]
                return
            } else {
                #::GidUtils::SetWarnLine [_ "Warning: This problemtype must be updated for translation"]
            }
        }

        set tail_opt_name [find_option i $option_name $desc 1]
        if {$tail_opt_name== ""} {return}
        if { $position == "end" } {
            set position [expr {[llength $MenuEntries($i)]-1}]
        }
        if {[llength $MenuEntries($i)]<= $position} {
            lappend MenuEntries($i) $tail_opt_name
            lappend MenuCommands($i) "-np- $command"
            if {[info exists MenuAcceler($i)]} {
                lappend MenuAcceler($i) $acceler
            }
            if {[info exists MenuIcons($i)]} {
                lappend MenuIcons($i) $icon
            }
        } else {
            if {$ins_repl=="replace"} {
                del_submenus $i $position $desc
                set MenuEntries($i) [lreplace $MenuEntries($i) $position $position $tail_opt_name]
                set MenuCommands($i) [lreplace $MenuCommands($i) $position $position "-np- $command"]
                if {[info exists MenuAcceler($i)] && [llength $MenuAcceler($i)] > $position} {
                    set MenuAcceler($i) [lreplace $MenuAcceler($i) $position $position $acceler]
                }
                if {[info exists MenuIcons($i)] && [llength $MenuIcons($i)] > $position } {
                    set MenuIcons($i) [lreplace $MenuIcons($i) $position $position $icon]
                }
            } elseif {$ins_repl=="insert"} {
                set MenuEntries($i) [linsert $MenuEntries($i) $position  $tail_opt_name]
                set MenuCommands($i) [linsert $MenuCommands($i) $position  "-np- $command"]
                if {[info exists MenuAcceler($i)]} {
                    set MenuAcceler($i) [linsert $MenuAcceler($i) $position $acceler]
                }
                if {[info exists MenuIcons($i)]} {
                    set MenuIcons($i) [linsert $MenuIcons($i) $position $icon]
                }
                move_submenus $i $position $desc "down"
            }
        }
    }
}

# Removes an option of a given menu.
# Arg 'prepost' can have these values:
# "PRE" to remove an option of the preprocess menus
# "POST" to remove an option of the postprocess menus
# or "PREPOST" to remove an option of the pre and the postprocess menus

proc RemoveMenuOption {menu_name option_name prepost} {
  if {$prepost!="PRE" && $prepost!="POST" && $prepost!="PREPOST"} {
    return -code error [_ "Wrong arg. Must be PRE, POST or PREPOST."]
  }
  #back compatibility: menus were renamed
  set menu_name [ConvertDeprecatedTranslatedMenuNameToCurrent $menu_name $prepost]

  if {$prepost eq "PRE"} {
    set where [list "" "PRE"]
  } elseif {$prepost eq "POST"} {
    set where [list "P" "POST"]
  } else {
    set where [list "" "PRE" "P" "POST"]
  }
  foreach {c desc} $where {
    upvar \#0 MenuNames$c MenuNames
    upvar \#0 MenuEntries$c MenuEntries
    upvar \#0 MenuCommands$c MenuCommands
    upvar \#0 MenuAcceler$c MenuAcceler
    upvar \#0 MenuIcons$c MenuIcons

    set i [lsearch $MenuNames $menu_name]
    if { $i == "-1" } {
      #try to search the translated word, for old problemtypes not updated to translation
      #set i [GiD_FindMenuIndex $menu_name $desc _]
	set i [ GiDMenu::_FindIndex $menu_name $desc _]
      if { $i == "-1" } {
        WarnWin [_ "ERROR: Menu %s doesn't exist in $desc." $menu_name]
        return
      } else {
        #::GidUtils::SetWarnLine [_ "Warning: This problemtype must be updated for translation"]
      }
    }
    set position [find_option i $option_name $desc]
    if {$position == ""} {return}

    del_submenus $i $position $desc
    move_submenus $i $position $desc "up"
    set MenuEntries($i)  [lreplace $MenuEntries($i) $position $position ]
    set MenuCommands($i) [lreplace $MenuCommands($i) $position $position]
    if {[info exists MenuAcceler($i)] && [llength $MenuAcceler($i)] > $position} {
      set MenuAcceler($i) [lreplace $MenuAcceler($i) $position $position ]
    }
    if {[info exists MenuIcons($i)] && [llength $MenuIcons($i)] > $position} {
        set MenuIcons($i) [lreplace $MenuIcons($i) $position $position]
    }
  }
}

# Modify a previous menu identified by menu_name option_name and prepost
# Arg 'new_command' is the command called when the menu option
# is selected.
# Arg 'prepost' can have these values:
# "PRE" to insert the option in the preprocess menus
# "POST" to insert the option in the postprocess menus
# or "PREPOST" to insert the option in the pre and postprocess menus
# Arg new_option_name can be the same as option_name or a new one
# Arg new_command is an optional new command (-keep- to keep the current)
# Arg new_acceler is an optional key accelerator, like "Control-s" (-maintain- to keep the current)
# Arg new_icon is the name of an small icon (16x16 pixels) (-default- to keep the current)
proc ModifyMenuOption {menu_name option_name prepost new_option_name {new_command -default-} \
        {new_acceler -default-} {new_icon -default-} } {

    if {$prepost!="PRE" && $prepost!="POST" && $prepost!="PREPOST"} {
        return -code error [_ "Wrong arg. Must be PRE, POST or PREPOST."]
    }

    #back compatibility: menus were renamed
    set menu_name [ConvertDeprecatedTranslatedMenuNameToCurrent $menu_name $prepost]

    if {$prepost eq "PRE"} {
        set where [list "" "PRE"]
    } elseif {$prepost eq "POST"} {
        set where [list "P" "POST"]
    } else {
        set where [list "" "PRE" "P" "POST"]
    }
    foreach {c desc} $where {
        upvar \#0 MenuNames$c MenuNames
        upvar \#0 MenuEntries$c MenuEntries
        upvar \#0 MenuCommands$c MenuCommands
        upvar \#0 MenuAcceler$c MenuAcceler
        upvar \#0 MenuIcons$c MenuIcons

        set i [lsearch $MenuNames $menu_name]
        if { $i == "-1" } {
            #try to search the translated word, for old problemtypes not updated to translation
            #set i [GiD_FindMenuIndex $menu_name $desc _]
	    set i [ GiDMenu::_FindIndex $menu_name $desc _]
            if { $i == "-1" } {
                ::GidUtils::SetWarnLine [_ "ERROR: Menu %s doesn't exist in $desc." $menu_name]
                return 1
            } else {
                #::GidUtils::SetWarnLine [_ "Warning: This problemtype must be updated for translation"]
            }
        }

        set tail_opt_name [find_option i $option_name $desc 1]
        if {$tail_opt_name== ""} { return 1 }

        set position [lsearch $MenuEntries($i) $option_name]
        if {$position == -1 } { return 1 }

        if { $new_option_name != "-default-" } {
            set MenuEntries($i) [lreplace $MenuEntries($i) $position $position $new_option_name]
        }
        if { $new_command != "-default-" } {
            set MenuCommands($i) [lreplace $MenuCommands($i) $position $position "-np- $new_command"]
        }
        if {[info exists MenuAcceler($i)] && [llength $MenuAcceler($i)] > $position} {
            if { $new_acceler != "-default-" } {
                set MenuAcceler($i) [lreplace $MenuAcceler($i) $position $position $new_acceler]
            }
        }
        if {[info exists MenuIcons($i)] && [llength $MenuIcons($i)] > $position } {
            if { $new_icon != "-default-" } {
                set MenuIcons($i) [lreplace $MenuIcons($i) $position $position $new_icon]
            }
        }
    }
    return 0
}

proc find_option {i option_name prepost {insert 0}} {
    global MenuNames MenuEntries MenuCommands MenuAcceler MenuIcons
    global MenuNamesP MenuEntriesP MenuCommandsP MenuAccelerP MenuIconsP

    upvar $i path_menu
    set p 0

    set prepost [string toupper $prepost]

    while {$p<[string length $option_name]} {
	set option ""
	set char [string index $option_name $p]
	if {$char == ">"} {
	    set p [expr $p +1]
	    set char [string index $option_name $p]
	}
	while {$char != ">" && $p < [string length $option_name]} {
	    append option $char
	    set p [expr $p +1]
	    set char [string index $option_name $p]
	}
	
	set position 0
	set end 0
	if {$prepost == "PRE"} {
	    if {![info exists MenuEntries($path_menu)]} {
		set MenuEntries($path_menu)  ""
		set MenuCommands($path_menu) ""
		set MenuAcceler($path_menu)  ""
		set MenuIcons($path_menu)  ""
		
		regexp {(.*),([0-9]+)} $path_menu b previous_menu previous_position
		set MenuCommands($previous_menu) [lreplace $MenuCommands($previous_menu) $previous_position $previous_position ""]
		if {[info exists MenuAcceler($previous_menu)] && [llength $MenuAcceler($previous_menu)] > $previous_position} {
		    set MenuAcceler($previous_menu)  [lreplace $MenuAcceler($previous_menu) $previous_position $previous_position ""]
		}
		if {[info exists MenuIcons($previous_menu)] && [llength $MenuIcons($previous_menu)] > $previous_position} {
		    set MenuIcons($previous_menu)  [lreplace $MenuIcons($previous_menu) $previous_position $previous_position ""]
		}
	    }
	
	    while { [cmpoption [lindex $MenuEntries($path_menu) $position] $option] != 0 && $end==0} {
		set position [expr $position +1]
		if {$position >= [llength $MenuEntries($path_menu)]} {
		    if {$insert==0 || $p < [string length $option_name]} {
		        return ""
		    }
		    set end 1
		}
	    }
	}
	
	set end 0
	if {$prepost == "POST"} {
	    if {![info exists MenuEntriesP($path_menu)]} {
		set MenuEntriesP($path_menu)  ""
		set MenuCommandsP($path_menu) ""
		set MenuAccelerP($path_menu)  ""
		set MenuIconsP($path_menu)  ""
		
		regexp {(.*),([0-9]+)} $path_menu b previous_menu previous_position
		set MenuCommandsP($previous_menu) [lreplace $MenuCommandsP($previous_menu) $previous_position $previous_position ""]
		if {[info exists MenuAccelerP($previous_menu)] && [llength $MenuAccelerP($previous_menu)] > $previous_position} {
		    set MenuAccelerP($previous_menu)  [lreplace $MenuAccelerP($previous_menu) $previous_position $previous_position ""]
		}
		if {[info exists MenuIconsP($previous_menu)] && [llength $MenuIconsP($previous_menu)] > $previous_position} {
		    set MenuIconsP($previous_menu)  [lreplace $MenuIconsP($previous_menu) $previous_position $previous_position ""]
		}
	    }
	    while {[lindex $MenuEntriesP($path_menu) $position] != $option && $end==0} {
		set position [expr $position +1]
		if {$position >= [llength $MenuEntriesP($path_menu)]} {
		    if {$insert==0 || $p < [string length $option_name]} {
		        return ""
		    }
		    set end 1
		}
	    }
	}
	if {$p<[string length $option_name]} {set path_menu "$path_menu,$position"}
    }

    if {$insert==1} {
	return $option
    } else {
	return $position
    }
}

#compare two words removing the last "..."
proc cmpoption { a b } {
    return [GiDMenu::cmpoption $a $b]
}

proc del_submenus {i position pre} {
    return [GiDMenu::del_submenus $i $position $pre]
}

proc rec_del_menu {param ent com acc icons} {
    return [GiDMenu::rec_del_menu $param $ent $com $acc $icons]
}

proc move_submenus {i position pre inc} {
    return [GiDMenu::move_submenus $i $position $pre $inc]
}

proc dec_inc_menu {param last_menu inc} {
    return [GiDMenu::dec_inc_menu $param $last_menu $inc]
}

proc rec_move_menu {param last_menu ent com acc icons inc} {
    return [GiDMenu::rec_move_menu $param $last_menu $ent $com $acc $icons $inc]
}


# this command SelectButton (and its dependencies EnableOrDisableWindow and EndSelectButton)
# were inapropiatedly used by stampack55, but it will disappear.

# what can be normal or disabled
proc EnableOrDisableWindow { w what } {
    catch { $w conf -state $what }
    foreach i [winfo children $w] {
	EnableOrDisableWindow $i $what
    }
}

proc SelectButton { w wplace text disable comm } {
    if { $disable == "disableall" } {
	EnableOrDisableWindow $w disabled
    }
    frame $w.finf -relief ridge -bd 4
    button $w.finf.f -text Finish -command "EndSelectButton $w $comm"
    label $w.finf.l -text $text
    pack $w.finf.f $w.finf.l -side left
    place $w.finf -in $w -x 0 -rely 1 -anchor sw
    delayedop changefunc "$w.finf.f invoke"
}

# only called internally
proc EndSelectButton { w comm } {
    delayedop -cancel changefunc "$w.finf.f invoke"
    destroy $w.finf
    GiD_Process Mescape
    EnableOrDisableWindow $w normal
    eval $comm
}

#to avoid problems if an old problemtype using tile invoke the old style command
interp alias "" style "" ttk::style

