
if { $::LAYERS_TREE } {
    package require treectrl
    package require base64

    # Get default colors
    set w [listbox .listbox]
    set SystemHighlight [$w cget -selectbackground]
    set SystemHighlightText [$w cget -selectforeground]
    destroy $w
    if { $::tcl_platform(platform) == "unix" } {
	# I hate that gray selection color
	set SystemHighlight #316ac5
	set SystemHighlightText White
    }
} else {
    package require tablelist_tile
    set LayersSelection ""
}

set LayersNoRedraw 0

proc EncodeName { x } {
    return [encrypter BASE64 encode [encoding convertto utf-8 $x]]
}

proc DecodeName { x } {
    return [encoding convertfrom utf-8 [encrypter BASE64 decode $x]]
}

proc SetSelection_Old { tbl } {
    global LayersSelection
    if { ![winfo exists $tbl] } { return }
    set LayersSelection ""
    set n [$tbl size]
    if { $n == 1 } {
	$tbl selection set 0
    }
    for { set i 0 } { $i < $n } { incr i } {
	if { [$tbl selection includes $i] } {
	    lappend LayersSelection [lindex [$tbl get $i] 1]
	}
    }
    return $LayersSelection
}

proc SetLayerAsSelection { T layer } {
    if { $::LAYERS_TREE } {
	return [SetLayerAsSelection_New $T $layer]
    } else {
	return [SetLayerAsSelection_Old $T $layer]
    }
}

proc SetLayerAsSelection_New { T layer } {
    if { ![winfo exists $T] } { return }
    $T selection clear
    set item [$T item id "tag [EncodeName $layer]"]
    if { $item != "" } {
	$T selection add $item
	$T see $item
    }
#     SetSelection $T
}

proc SetLayerAsSelection_Old { tbl layer } {
    if { ![winfo exists $tbl] } { return }
    $tbl selection clear
    for { set i 0 } { $i < [$tbl size] } { incr i } {
	if { [lindex [$tbl get $i] 1] == $layer } {
	    $tbl selection set $i
	    $tbl see $i
	    break
	}
    }
    SetSelection $tbl
}

proc SelectLayersByRegexpPressOk { T e } {
    if { $::LAYERS_TREE } {
	return [SelectLayersByRegexpPressOk_New $T $e]
    } else {
	return [SelectLayersByRegexpPressOk_Old $T $e]
    }
}

proc SelectLayersByRegexpPressOk_New { T e } {
    global wChangeLay LayerSelRegexp LayerSelAdd LayerSelCase
    if { $LayerSelAdd == "substitute" } {
	$T selection clear
    }
    foreach item [$T item range 1 end] {
	#set fullname [DecodeName [$T item tag names $item]]
	set i [$T item text $item 0]
	set change ""
	if { $LayerSelRegexp != "regex" } {
	    if { $LayerSelCase != "case" } {
		if { [string match [string tolower [$e get]] \
		        [string tolower $i]] } { set change yes }
	    } else {
		if { [string match [$e get] $i] } {
		    set change yes
		}
	    }
	} else {
	    if { $LayerSelCase != "case" } {
		if { [regexp -nocase -- [$e get] $i res] } {
		    if { $i == $res } { set change yes }
		}
	    } else {
		if { [regexp -- [$e get] $i res] } {
		    if { $i == $res } { set change yes }
		}
	    }
	}
	if { $change == "yes" } {
	    if { $LayerSelAdd != "substract" } {
		$T selection add $item
	    } else {
		$T selection clear $item
	    }
	}
    }
#     SetSelection $T
    destroy $wChangeLay.sel
    if { [winfo exists $wChangeLay.ent.e] } {
	focus $wChangeLay.ent.e
    }
}

proc SelectLayersByRegexpPressOk_Old { tbl e } {
    global wChangeLay LayerSelRegexp LayerSelAdd LayerSelCase
    if { $LayerSelAdd == "substitute" } {
	$tbl sel clear 0 end
    }
    set idx 0
    foreach i [$tbl get 0 end] {
	set i [lindex $i 1]
	set change ""
	if { $LayerSelRegexp != "regex" } {
	    if { $LayerSelCase != "case" } {
		if { [string match [string tolower [$e get]] \
		        [string tolower $i]] } { set change yes }
	    } else {
		if { [string match [$e get] $i] } {
		    set change yes
		}
	    }
	} else {
	    if { $LayerSelCase != "case" } {
		if { [regexp -nocase -- [$e get] $i res] } {
		    if { $i == $res } { set change yes }
		}
	    } else {
		if { [regexp -- [$e get] $i res] } {
		    if { $i == $res } { set change yes }
		}
	    }
	}
	if { $change == "yes" } {
	    if { $LayerSelAdd != "substract" } {
		$tbl sel set $idx
	    } else {
		$tbl sel clear $idx
	    }
	}
	incr idx
    }
    SetSelection $tbl
    destroy $wChangeLay.sel
    focus $wChangeLay.ent.e
}

proc SelectLayersByRegexp { tbl } {
    global wChangeLay LayerSelRegexp LayerSelAdd LayerSelCase

    if { [winfo exists $wChangeLay.sel] } {
	 destroy $wChangeLay.sel
    }
    ttk::frame $wChangeLay.sel -relief ridge -borderwidth 4
    ttk::frame $wChangeLay.sel.up
    set e [ttk::entry $wChangeLay.sel.up.e]
    ttk::button $wChangeLay.sel.up.ok -text [_ "OK"] \
	    -command [list SelectLayersByRegexpPressOk $tbl $e]
    grid $e $wChangeLay.sel.up.ok -sticky ew
    grid configure $wChangeLay.sel.up.ok -sticky w
    grid columnconfigure $wChangeLay.sel.up 0 -weight 1

    ttk::frame $wChangeLay.sel.down -borderwidth 2
    ttk::checkbutton $wChangeLay.sel.down.reg -text [_ "Regexp"] -onvalue regex \
	    -variable LayerSelRegexp
    ttk::checkbutton $wChangeLay.sel.down.case -text [_ "Case"] -onvalue case \
	    -variable LayerSelCase

    ttk::frame $wChangeLay.sel.rad -borderwidth 2
    ttk::radiobutton $wChangeLay.sel.rad.substitute -text [_ "Substitute"] -value substitute \
	    -variable LayerSelAdd
    ttk::radiobutton $wChangeLay.sel.rad.add -text [_ "Add"] -value add \
	    -variable LayerSelAdd
    ttk::radiobutton $wChangeLay.sel.rad.substract -text [_ "Substract"] -value substract \
	    -variable LayerSelAdd

    set LayerSelRegexp ""
    set LayerSelAdd substitute
    set LayerSelCase ""

    grid $wChangeLay.sel.down.reg $wChangeLay.sel.down.case -sticky ew

    grid $wChangeLay.sel.rad.substitute $wChangeLay.sel.rad.add \
	    $wChangeLay.sel.rad.substract -sticky ew

    grid $wChangeLay.sel.up -sticky ew
    grid $wChangeLay.sel.down -sticky ew
    grid $wChangeLay.sel.rad -sticky ew

    grid $wChangeLay.sel -row 1 -sticky new
    grid columnconfigure $wChangeLay.sel 0 -weight 1

    focus $e
    bind $e <Return> [list $wChangeLay.sel.up.ok invoke]
}

proc SendAndFill { layerList col } {
    global LayersNoRedraw

    if { $LayersNoRedraw != 2 } { GiD_Process Layers }
    foreach i $layerList {
	GiD_Process color $i $col
    }

    #FillInfoLayers ;#automatically called
    if { $LayersNoRedraw == 0 } { GiD_Process escape }
    if { $LayersNoRedraw == 1 } { set LayersNoRedraw 2 }
}

proc FillInfoLayers { } {
    global wChangeLay GidPriv
    if { [info exists wChangeLay] } {
	FillTableInfoLayers $GidPriv(Layers,table)
	if { [winfo exists $wChangeLay.touse.l] } {
	    $wChangeLay.touse.l conf -text [GiD_Info Project LayerToUse]
	}
    }
    if { [info exists GidPriv(ComboLayers,table)] } {
	FillTableInfoLayers $GidPriv(ComboLayers,table)
    }
    if { [info exists GidPriv(ComboLayers,entry)] && [winfo exists $GidPriv(ComboLayers,entry)] } {
	$GidPriv(ComboLayers,entry) delete 0 end
	$GidPriv(ComboLayers,entry) insert end [GiD_Info Project LayerToUse]
    }
}

proc FillTableInfoLayers_Old { tbl } {
    global LayersSelection GidPriv
    if { ![winfo exists $tbl] } { return }

    set LayersSelection ""
    set LayerNames [GiD_Info layers]
    set BackLayerNames [GiD_Info back_layers]

    if { $BackLayerNames != "" } {
	regsub -all {_\*back\*} $BackLayerNames {} pp
	set BackLayerNames $pp
    }

    set sellayers ""
    foreach i [$tbl curselection] {
	lappend sellayers [lindex [$tbl get $i] 1]
    }

    $tbl delete 0 end

    set LayerToUse [GiD_Info Project LayerToUse]

    if { [info exists GidPriv(LayersAlphabetic)] && $GidPriv(LayersAlphabetic) } {
	set LayerNames [lsort -dictionary $LayerNames]
    }
    foreach i $LayerNames {
	set LayersProp [lindex [GiD_Info layers $i] 0]
	$tbl insert end [list "" $i "" "" ""]
	
	set cur_col [ format "#%02x%02x%02x" [lindex $LayersProp 2] \
		[lindex $LayersProp 3] [lindex $LayersProp 4]]
	
	$tbl cellconfigure end,0 -background $cur_col -text {} -selectbackground $cur_col
	if { $i == $LayerToUse } {
	    $tbl cellconfigure end,1 -image [GetImage layer_touse.gif]
	} else {
	    $tbl cellconfigure end,1 -image [GetImage layer_nottouse.gif] ;#void image
	}
	
	if { [lindex $LayersProp 0] } {
	    $tbl cellconfigure end,2 -image [GetImage layer_on.gif]
	} else {
	    $tbl cellconfigure end,2 -image [GetImage layer_off.gif]
	}
	
	if { [lindex $LayersProp 1] } {
	    $tbl cellconfigure end,3 -image [GetImage layer_freeze.gif]
	} else {
	    $tbl cellconfigure end,3 -image [GetImage layer_unfreeze.gif]
	}
	if { [lindex $LayersProp 5] == 255 } {
	    $tbl cellconfigure end,4 -image [GetImage layer_transparent_off.gif]
	} else {
	    $tbl cellconfigure end,4 -image [GetImage layer_transparent_on.gif]
	}

	if { [ lsearch $BackLayerNames $i] != -1 } {
	    $tbl cellconfigure end,5 -image [GetImage layer_bringtofront.gif]
	} else {
	    $tbl cellconfigure end,5 -image [GetImage layer_sendtoback.gif] ;#void image
	}
	
	if { [lsearch -exact $sellayers $i] != -1 } {
	    $tbl selection set end
	}
    }

    $tbl activate [lsearch $LayerNames $LayerToUse]

    if { [llength [$tbl curselection]] == 1 } {
	$tbl see [$tbl curselection]
    }
}


proc SetLayersToDelete { tbl } {
    global LayersNoRedraw
    set parent [winfo parent $tbl]
    set LayersSelection [SetSelection $tbl]
    if { $LayersSelection == "" } {
	tk_dialogRAM  $parent.modal [_ "error"] \
		[_ "before applying operation delete it is necessary to select layers"] \
		error 0 [_ "OK"]
	return
    }

    if { $LayersNoRedraw != 2 } { GiD_Process Layers }
    set warnmessage 1
    foreach i $LayersSelection {
	GiD_Process delete $i
	
	if { $warnmessage && [catch { GiD_Info layers $i } ] == 0 } {
	    set ret [tk_dialogRAM  $parent.modal [_ "error"] \
		         [_ "Layer '%s' can't be erased.\n(see the message bar for details)" $i] \
		         error 0 [_ "OK"] [_ "OK to all"]]
	    if { $ret == 1 } { set warnmessage 0 }
	}
    }

    if { $LayersNoRedraw == 0 } { GiD_Process escape }
    if { $LayersNoRedraw == 1 } { set LayersNoRedraw 2 }

    #FillInfoLayers ;#automatically called
}

proc SetLayersToColor { tbl } {
    global LayersNoRedraw
    set parent [winfo parent $tbl]
    set LayersSelection [SetSelection $tbl]
    if { $LayersSelection == "" } {
	tk_dialogRAM  $parent.modal [_ "error"] \
	[_ "before applying operation color it is necessary to select layers"] \
	error 0 [_ "OK"]
	return
    }
    set sel $LayersSelection ;#trick because GiD_Process Layers can change LayersSelection

    if { $LayersNoRedraw != 2 } {
	GiD_Process Layers
	set LayersSelection $sel
    }

    set LayersProp [lindex [GiD_Info layers [lindex $LayersSelection 0]] 0]

    set cur_col [ format "#%02x%02x%02x%02x" [lindex $LayersProp 2] \
	    [lindex $LayersProp 3] [lindex $LayersProp 4] [lindex $LayersProp 5]]

    #set retcol [tk_chooseColor -parent $parent -title [_ "Select color"] -initialcolor $cur_col]
    set retcol [GIDChooseColor $parent.selectcolor -title [_ "Select color"] -color $cur_col]

    if { $retcol != "" } {
	set retcol [string range $retcol 1 end]
	if { [string length $retcol] == 8 } {
	    scan $retcol %2x%2x%2x%2x col1 col2 col3 alpha
	    set col [format %03i%03i%03i $col1 $col2 $col3]
	    foreach i $LayersSelection {
		GiD_Process color $i $col transparent $i $alpha
	    }
	} else {
	    scan $retcol %2x%2x%2x col1 col2 col3
	    set col [format %03i%03i%03i $col1 $col2 $col3]
	    foreach i $LayersSelection {
		GiD_Process color $i $col
	    }
	}
	#    RamColor .ramcolor "SendAndFill [list $LayersSelection]"
    }
    if { $LayersNoRedraw == 0 } { GiD_Process escape }
    if { $LayersNoRedraw == 1 } { set LayersNoRedraw 2 }

    #FillInfoLayers ;#automatically called

}

proc SetLayersToRename_Old { tbl } {
    global  wChangeLay LayersNoRedraw
    set parent [winfo parent $tbl]
    set LayersSelection [SetSelection $tbl]
    if { [llength $LayersSelection] !=1 } {
	tk_dialogRAM $parent.m [_ "error"] \
		[_ "Before applying rename it is necessary to select 1 layer"] \
		error 0 [_ "OK"]
	return
    }
    if { [winfo exists $wChangeLay.ent.e] } {
	set name [string trim [$wChangeLay.ent.e get]]
    } else {
	set name ""
    }
    if { $name == "" } {
	#tk_dialogRAM $parent.m [_ "error"] \
		#[_ "To rename one layer select it and write a new name before pressing the button"] \
		#error 0 [_ "OK"]
	#return
	set name [tk_dialogEntryRAM .gid.tempwinw [_ "Layers"] \
		[_ "Enter new layer name"]  gidquestionhead any "" ]
	if { $name == "--CANCEL--" ||  $name == "" } { return  }
    }


    if { ![IsValidLayername $name] } {
	WarnWin [_ "Bad layer name, start or end by '//' is not allowed"]
	return
    }

    foreach i [GiD_Info layers] {
	if { $i == $name } {
	    tk_dialogRAM $parent.m [_ "error"] [_ "Layer '%s' already exists" $i] error 0 [_ "OK"]
	    return
	}
    }

    if { [winfo exists $wChangeLay.ent.e] } {
	$wChangeLay.ent.e delete 0 end
    }
    SetLayersTo RENAME $tbl $name
}

proc SetLayersTo { type tbl {newname ""} } {
    global LayersNoRedraw
    set LayersSelection [SetSelection $tbl]
    if { $LayersSelection == "" } {
	WarnWin [_ "before applying operation %s it is necessary to select layers" $type]
	return
    }

    set VarToSend ""
    if { $LayersNoRedraw != 2 } {
	lappend VarToSend Layers
    }
    foreach i $LayersSelection {
	switch $type {
	    ON {
		lappend VarToSend On $i
	    }
	    OFF {
		lappend VarToSend Off $i
	    }
	    FREEZE {
		lappend VarToSend Freeze $i
	    }
	    UNFREEZE {
		lappend VarToSend UnFreeze $i
	    }
	    TRANSPARENT {
		lappend VarToSend Transparent $i 127
	    }
	    OPAQUE {
		lappend VarToSend Opaque $i
	    }
	    TOUSE {
		lappend VarToSend ToUse $i
	    }
	    RENAME {
		lappend VarToSend ChangeName $i $newname
	    }
	    BRINGTOFRONT {
		lappend VarToSend BringToFront $i
	    }
	}
    }
    if { $LayersNoRedraw == 0 } {
	lappend VarToSend escape
    }

    GiD_Process {*}$VarToSend
    if { $LayersNoRedraw == 1 } { set LayersNoRedraw 2 }

    #FillInfoLayers ;#automatically called
    if { $type == "RENAME" } {
	#Trick to have selected the new name after FillInfoLayers will be automatically called
	SetLayerAsSelection $tbl $newname
    }
}

proc SendLayerEntitiesAA { T what } {
    global wChangeLay GidPriv

    set LayersSelection [SetSelection $T]
    if { [llength $LayersSelection] !=1 } {
	WarnWin [_ "before applying 'Entities' it is necessary to select 1 layer"]
	return
    }
    set layer [lindex $LayersSelection 0]

    if { $GidPriv(LayersAlsoLower) == 0 } {
	GiD_Process Layers Entities $layer $what
    } else {
	GiD_Process Layers Entities $layer LowerEntities $what
    }
    set SmallWinSelecting [GiD_Set SmallWinSelecting]
    if { $::LAYERS_TREE } {
	FinishButton $wChangeLay $wChangeLay \
	    [_ "Press 'Finish' to end selection"] \
	    "" disableall  $SmallWinSelecting
    } else {
	FinishButton $wChangeLay $wChangeLay.buts \
	    [_ "Press 'Finish' to end selection"] \
	    "" disableall $SmallWinSelecting
    }
}


proc SendLayerEntities { m T } {
#     SetSelection $T
    set geomlistNames [GetGeometryLabels]
    lappend geomlistNames {*}[list [_ "Dimensions"] [_ "All"]]
    set geomlist [GetGeometryCommands]
    lappend geomlist {*}{Dimensions All}

    AddEntitiesToMenuAfter 0 $m [list SendLayerEntitiesAA $T] $geomlistNames $geomlist \
	    [list [_ "Nodes"] [_ "Elements"] [_ "Dimensions"] [_ "All"]] \
	    {Nodes Elements Dimensions All}
}

proc LayerToUseEntitiesAA { what } {
    GidUtils::SetWarnLine [_ "Pick one entity to set its layer as current layer to use"]
    GidUtils::DisableWarnLine
    set nnum [GidUtils::PickEntities $what single]
    GidUtils::EnableWarnLine
    if { $nnum != "" } {
	set retval [GiD_Info list_entities $what $nnum]
	set layername ""
	regexp {LAYER:([^\n\r]+)} $retval {} layername
	set layername [string trim $layername]
	if { $layername != "" } {
	    GiD_Process Layers ToUse $layername
	    FillInfoLayers
	    SetLayerAsSelection $::GidPriv(Layers,table) $layername
	} else {
	    WarnWin [_ "Selected entity does not belong to one layer"]
	}
    }
}

proc LayerToUseEntities { menu } {
    if { $::GidPriv(HideVolumeLevel) ==0} {
	set geomlistNames [list [_ "Point"] [_ "Line"] [_ "Surface"] [_ "Volume"]]
    } else {
	set geomlistNames [list [_ "Point"] [_ "Line"] [_ "Surface"]]
    }
    set geomlist [GetGeometryCommands]
    set meshlistNames [list [_ "Node"] [_ "Element"]]
    set meshlist {Nodes Elements}
    AddEntitiesToMenu $menu "LayerToUseEntitiesAA " $geomlistNames $geomlist $meshlistNames $meshlist
}

proc SendLayerEntitiesToBackAA { what } {
    global wChangeLay GidPriv

    if { $GidPriv(LayersBackOpposite) } {
	if { $GidPriv(LayersAlsoLower) == 0 } {
	    GiD_Process Layers SendToBackOpposite $what
	} else {
	    GiD_Process Layers SendToBackOpposite LowerEntities $what
	}
    } else {
	if { $GidPriv(LayersAlsoLower) == 0 } {
	    GiD_Process Layers SendToBack $what
	} else {
	    GiD_Process Layers SendToBack LowerEntities $what
	}
    }
    set SmallWinSelecting [GiD_Set SmallWinSelecting]
    if { $::LAYERS_TREE } {
	FinishButton $wChangeLay $wChangeLay \
	    [_ "Press 'Finish' to end selection"] \
	    "FillInfoLayers" disableall $SmallWinSelecting
    } else {
	FinishButton $wChangeLay $wChangeLay.buts \
	    [_ "Press 'Finish' to end selection"] \
	    "FillInfoLayers" disableall $SmallWinSelecting
    }
}


proc SendLayerEntitiesToBack { m tbl } {

    #SetSelection $tbl

    set geomlistNames [GetGeometryLabels]
    lappend geomlistNames {*}[list [_ "All"]]
    set geomlist [GetGeometryCommands]
    lappend geomlist {*}{All}

    AddEntitiesToMenuAfter 1 $m "SendLayerEntitiesToBackAA" $geomlistNames $geomlist \
	    [list [_ "Nodes"] [_ "Elements"] [_ "All"]] \
	    {Nodes Elements All}

    $m add separator
    if { [GiD_Info layers -hasbacklayers] } {
	$m add command -label [_ "Bring to front"] \
	    -command [list SetLayersTo BRINGTOFRONT $tbl] \
	    -image [GetImage layer_bringtofront.gif] -compound left
	$m add command -label [_ "Bring all to front"] \
	    -command {GiD_Process Layers BringToFrontAll escape ; FillInfoLayers} \
	    -image [GetImage layer_bringtofront.gif] -compound left
    } else {
	$m add command -label [_ "Bring to front"] -state disabled \
	    -image [GetImage layer_bringtofront.gif] -compound left
	$m add command -label [_ "Bring all to front"] -state disabled \
	    -image [GetImage layer_bringtofront.gif] -compound left
    }
}

proc GetAutomaticLayerFolderName { {prefix ""} } {
    set LayerNames [GiD_Info layers]
    set name ""
    for {set i 0} {$i<10000} {incr i} {
	set name ${prefix}Part${i}//Layer0
	if { [lsearch -exact $LayerNames $name] == -1 } { break }
    }
    return $name
}

proc CreateNewLayerFolder { {T ""} {name ""} } {
    set prefix ""
    if { $T != "" && [$T selection count] == 1 } {
	set item [$T selection get 0]
	if { [IsItemFolderOfLayers $T $item] } {
	    set prefix [DecodeName [$T item tag names $item]]
	}
    }
    if { $name == "" } {
	set name [GetAutomaticLayerFolderName $prefix]
    } else {
	if { ![IsValidLayername $name] } {
	    WarnWin [_ "Bad layer name, start or end by '//' is not allowed"]
	    return ""
	}
    }
    GiD_Process Layers New $name escape

    return $name
}

proc GetAutomaticLayerName { {prefix ""} } {
    set LayerNames [GiD_Info layers]
    set name ""
    for {set i 0} {$i<10000} {incr i} {
	set name ${prefix}Layer${i}
	if { [lsearch -exact $LayerNames $name] == -1 } { break }
    }
    return $name
}

proc CreateNewLayer { {T ""} {name ""} } {
    set prefix ""
    if { $::LAYERS_TREE } {
	if { $T != "" && [$T selection count] == 1 } {
	    set item [$T selection get 0]
	    if { [IsItemFolderOfLayers $T $item] } {
		set prefix [DecodeName [$T item tag names $item]]
	    }
	}
    }
    if { $name == "" } {
	set name [GetAutomaticLayerName $prefix]
    } else {
	if { ![IsValidLayername $name] } {
	    WarnWin [_ "Bad layer name, start or end by '//' is not allowed"]
	    return ""
	}
    }
    GiD_Process Layers New $name escape
    return $name
}


proc NewLayerFromEntry { T w } {
    set name [string trim [$w.ent.e get]]
    #regsub -all {[ ]+} $name {_} name

    if { $name != "" } {
	set LayerNames [GiD_Info layers]
	set ipos [lsearch -exact $LayerNames $name]
	if { $ipos != -1 } {
	    tk_dialogRAM  $w.modal [_ "error"] [_ "Layer %s already exists" $name] error 0 [_ "OK"]
	    return
	}
    }
    set name [CreateNewLayer $T $name]

    #FillInfoLayers ;#automatically called
    $w.ent.e delete 0 end
    if { $name != "" } {
	# seleccionamos la que acabamos de crear
	SetLayerAsSelection $T $name
    }
}

proc ConfigureListScrollbars { listbox sx sy } {
    foreach i "x y" {
	if { ![info exists s${i}] || ![winfo exists [set s${i}]] } { continue }
	foreach "${i}1 ${i}2" [$listbox ${i}view] break
	if { [set ${i}1] == 0 && [set ${i}2] == 1 } {
	    after idle grid remove [set s${i}]
	} else {
	    after idle grid [set s${i}]
	}
    }
}

proc MotionTableList { x y w tbl } {
    set tablelist::x [expr {$x + [winfo x $w]}]
    set tablelist::y [expr {$y + [winfo y $w]}]
    set cell [$tbl nearestcell $tablelist::x $tablelist::y]
    foreach {row col} [ split $cell ,] break
    if { $col != 1 } {
	return -code break
    }
}

proc ClickTableList { x y w T } {
    if { $::LAYERS_TREE } {
	return [ClickTableList_New $x $y $T]
    } else {
	return [ClickTableList_Old $x $y $w $T]
    }
}

proc ClickTableList_New { x y T } {
#     focus $T
	
    set info [$T identify $x $y]
    if { [lindex $info 0] == "item" && [llength $info] >= 4 } {
	set item [lindex $info 1]
	set col [lindex $info 3]
	if { [IsItemFolderOfLayers $T $item] } {
	    #folder of layers
	    return ""
	}
    } elseif { [lindex $info 0] == "header" && [lindex $info 1] == "0" } {
	if { [$T column cget C0 -arrow] == "up" } {
	    $T column configure C0 -arrow down
	    $T item sort 0 -dictionary -increasing
	} else {
	    $T column configure C0 -arrow up
	    $T item sort 0 -dictionary -decreasing
	}
	return ""
    } else {
	return ""
    }

    if { $col != 0 } {
	if { ![$T selection includes $item] } {
	    $T selection clear
	    $T selection add $item
	}
    }
    if { $col == 0 } {
	#SetLayersTo TOUSE $T
    } elseif { $col == 1 } {
	SetLayersTo TOUSE $T
    } elseif { $col == 2 } {
	SetLayersToColor $T
    } elseif { $col == 3 } {
	if { [$T item element cget $item C$col elemImgAny -image] == [GetImage layer_off.gif] } {
	    SetLayersTo ON $T
	} else {
	    SetLayersTo OFF $T
	}
    } elseif { $col == 4 } {
	if { [$T item element cget $item C$col elemImgAny -image] == [GetImage layer_freeze.gif] } {
	    SetLayersTo UNFREEZE $T
	} else {
	    SetLayersTo FREEZE $T
	}
    } elseif { $col == 5 } {
	if { [$T item element cget $item C$col elemImgAny -image] == [GetImage layer_transparent_off.gif] } {
	    SetLayersTo TRANSPARENT $T
	} else {
	    SetLayersTo OPAQUE $T
	}
    } elseif { $col == 6 } {
	if { [$T item element cget $item C$col elemImgAny -image] == [GetImage layer_bringtofront.gif] } {
	    SetLayersTo BRINGTOFRONT $T
	} else {
	}
    }

    if { $col != 0 } {
	return -code break
    }
    return ""
}

proc ClickTableList_Old { x y w tbl } {
    focus $w
    set cell [$tbl containingcell $x $y]
    foreach {row col} [ split $cell ,] break
    if { $row < 0 || $col < 0 } return ""
    #if click a unselected row then select only this row (but not if column == 1)
    if { $col != 1 } {
	if { ![$tbl selection includes $row] } {
	    $tbl selection clear
	    $tbl selection set $row
	}
    }

    if { $col == 0 } {
	SetLayersToColor $tbl
    } elseif { $col == 1 } {
	#SetLayersTo TOUSE $tbl
    } elseif { $col == 2 } {
	if { [lindex [$tbl cellconfigure $row,$col -image] 4] == [GetImage layer_off.gif] } {
	    SetLayersTo ON $tbl
	    #$tbl cellconfigure $row,$col -image [GetImage layer_on.gif]
	} else {
	    SetLayersTo OFF $tbl
	    #$tbl cellconfigure $row,$col -image [GetImage layer_off.gif]
	}
    } elseif { $col == 3 } {
	if { [lindex [$tbl cellconfigure $row,$col -image] 4] == [GetImage layer_freeze.gif] } {
	    SetLayersTo UNFREEZE $tbl
	    #$tbl cellconfigure $row,$col -image [GetImage layer_unfreeze.gif]
	} else {
	    SetLayersTo FREEZE $tbl
	    #$tbl cellconfigure $row,$col -image [GetImage layer_freeze.gif]
	}
    } elseif { $col == 4 } {
	if { [lindex [$tbl cellconfigure $row,$col -image] 4] == [GetImage layer_transparent_off.gif] } {
	    SetLayersTo TRANSPARENT $tbl
	    #$tbl cellconfigure $row,$col -image layer_transparent_on.gif
	} else {
	    SetLayersTo OPAQUE $tbl
	    #$tbl cellconfigure $row,$col -image layer_transparent_off.gif
	}
    } elseif { $col == 5 } {
	if { [lindex [$tbl cellconfigure $row,$col -image] 4] == [GetImage layer_bringtofront.gif] } {
	   SetLayersTo BRINGTOFRONT $tbl
	    #$tbl cellconfigure $row,$col -image layer_bringtofront.gif
	} else {
	    #$tbl cellconfigure $row,$col -image layer_sendtoback.gif
	}
    }

#     if { $col != 1 } {
#         return -code break
#     }
    return ""
}

proc InvertSelectionTableList { T } {
    if { $::LAYERS_TREE } {
	return [InvertSelectionTableList_New $T]
    } else {
	return [InvertSelectionTableList_Old $T]
    }
}

proc InvertSelectionTableList_New { T } {
    set newselection {}
    foreach item [$T item range 0 end] {
	if { ![$T selection includes $item] } {
	    lappend newselection $item
	}
    }
    $T selection clear
    foreach item $newselection {
	$T selection add $item
    }
}

proc InvertSelectionTableList_Old { tbl } {
    set newselection {}
    for { set i 0 } { $i < [$tbl size] } { incr i } {
	if { ![$tbl selection includes $i] } {
	    lappend newselection $i
	}
    }
    $tbl selection clear
    foreach i $newselection {
	$tbl selection set $i
    }
    #SetSelection $tbl
}


proc DoubleClickTableList { x y w T } {
    if { $::LAYERS_TREE } {
	return [DoubleClickTableList_New $x $y $T]
    } else {
	return [DoubleClickTableList_Old $x $y $w $T]
    }
}

proc DoubleClickTableList_New { x y T } {
    set info [$T identify $x $y]
    if { [lindex $info 0] == "item" && [llength $info] >= 4 } {
	set item [lindex $info 1]
	set col [lindex $info 3]
    } else {
	return
    }


    if { $col == 0 } {
	$T selection clear
	$T selection add $item
	if { [IsItemFolderOfLayers $T $item] } {
	
	} else {
	    SetLayersTo TOUSE $T
	}
    }
}

proc DoubleClickTableList_Old { x y w tbl } {
    set cell [$tbl containingcell $x $y]
    foreach {row col} [ split $cell ,] break
    if { $row < 0 || $col < 0 } return
    $tbl selection set $row
    if { $col == 1 } {
	SetLayersTo TOUSE $tbl
    }
}


# http://wiki.tcl.tk/950
proc sbset {sb first last} {
    if {$first <= 0 && $last >= 1} {
	grid remove $sb
    } else {
	grid $sb
    }
    $sb set $first $last
    return
}

proc TreePlusScrollbarsInAFrame {f h v} {

    frame $f -borderwidth 0

    treectrl $f.t -highlightthickness 0 -borderwidth 0
    $f.t configure -xscrollincrement 20 -yscrollincrement 20
#    $f.t configure -itemprefix item# -columnprefix column#
    if {$h} {
	::ttk::scrollbar $f.sh -orient horizontal -command "$f.t xview"
	#                $f.t configure -xscrollcommand "$f.sh set"
	$f.t notify bind $f.sh <Scroll-x> { sbset %W %l %u }
	bind $f.sh <ButtonPress-1> "focus $f.t"
    }
    if {$v} {
	::ttk::scrollbar $f.sv -orient vertical -command "$f.t yview"
	#                $f.t configure -yscrollcommand "$f.sv set"
	$f.t notify bind $f.sv <Scroll-y> { sbset %W %l %u }
	bind $f.sv <ButtonPress-1> "focus $f.t"
    }
    grid columnconfigure $f 0 -weight 1
    grid rowconfigure $f 0 -weight 1
    grid configure $f.t -row 0 -column 0 -sticky news
    if {$h} {
	grid configure $f.sh -row 1 -column 0 -sticky we
	after idle grid remove $f.sh
    }
    if {$v} {
	grid configure $f.sv -row 0 -column 1 -sticky ns
	after idle grid remove $f.sv
    }

#     bind $f.t <Control-Shift-ButtonPress-1> {
#         TreeCtrl::MarqueeBegin %W %x %y
#         break
#     }
#     bind $f.t <Control-Shift-Button1-Motion> {
#         TreeCtrl::MarqueeUpdate %W %x %y
#         break
#     }
#     bind $f.t <Control-Shift-ButtonRelease-1> {
#         TreeCtrl::MarqueeEnd %W %x %y
#         break
#     }

#     MakeListPopup $f.t
#     MakeHeaderPopup $f.t
#     bind $f.t <ButtonPress-3> {
#         ShowPopup %W %x %y %X %Y
#     }
#     if {[tk windowingsystem] eq "aqua"} {
#         bind $f.t <Control-ButtonPress-1> {
#             ShowPopup %W %x %y %X %Y
#         }
#     }
    return
}

#separator can be a multicharacter, like //
proc split2 { x separator } {
    set splitchar \uFFFF ;#forbidden unicode character, x must never contain it
    return [split [string map "$separator $splitchar" $x] $splitchar]
}

#hierarchy of layers by its name (separated by //)
#e.g. parent//chidren//subchildren
#return the list of chidren layers
proc GetChildLayerNames { {parent ""} } {
    global GidPriv
    set LayerNames0 [GiD_Info layers]
    set len [string length $parent]
    set LayerNames {}
    foreach layer $LayerNames0 {
	if { [string equal -length $len $parent $layer] } {
	    set last [string first // $layer $len]
	    if { $last == -1 } {
		set last end
	    } else {
		incr last
	    }
	    lappend LayerNames [string range $layer $len $last]
	}
    }
    if { [info exists GidPriv(LayersAlphabetic)] && $GidPriv(LayersAlphabetic) } {
	set LayerNames1 [lsort -unique -dictionary $LayerNames]
    } else {
	#must remove repetitions
	set LayerNames1 {}
	foreach layer $LayerNames {
	    if { [info exists name($layer)] } continue
	    lappend LayerNames1 $layer
	    set name($layer) 1
	}
    }
    return $LayerNames1
}

proc IsFolderOfLayers { layername } {
    if { [string range $layername end-1 end] == "//" } {
	return 1
    } else {
	return 0
    }
}

proc  IsItemFolderOfLayers { T item } {
#     return [IsFolderOfLayers [DecodeName [$T item tag names $item]]]
    return [$T item numchildren $item]
}

proc IsValidLayername { layername } {
    if { [string range $layername end-1 end] == "//" } {
	return 0
    } elseif { [string range $layername 0 1] == "//" } {
	return 0
    } elseif { $layername == "" } {
	return 0
    }
    return 1
}

proc FillTableInfoLayers { T {parent ""} {parentitem root} } {
    if { $::LAYERS_TREE } {
	if { [winfo exists $T] } {

	    UpdateRootNodeVisibility $T

	    array unset stateitems
	    array unset selectionitems
	    #try to mantain the previous state: open, selected
	    foreach item [$T item range 0 end] {
		set fullnameencoded [$T item tag names $item]
		set stateitems($fullnameencoded) [$T item isopen $item]
		set selectionitems($fullnameencoded) [$T selection includes $item]
	    }
	    $T item delete all ;#coult try not to delete and rebuild all...
	    FillTableInfoLayers_New $T $parent $parentitem
	    foreach item [$T item range 0 end] {
		set fullnameencoded [$T item tag names $item]
		if { [info exists stateitems($fullnameencoded)] &&
		    !$stateitems($fullnameencoded) } {
		    $T item collapse $item
		}
		if { [info exists selectionitems($fullnameencoded)] &&
		    $selectionitems($fullnameencoded) } {
		    $T selection add $item
		}
	    }
	    set id [$T item id "tag [EncodeName [GiD_Info Project LayerToUse]]"]
	    if { $id != "" } {
		$T activate $id
	    }
	    if { [$T selection count] == 1 } {
		$T see [$T selection get]
	    }
	    set ::TreeCtrl::Priv(DirCnt,$T) [$T item count] ;#to use drag file-list bindings
	}
    } else {
	FillTableInfoLayers_Old $T
    }
}

proc LayersCreateListWin { w } {
    if { $::LAYERS_TREE } {
	return [LayersCreateListWin_New $w]
    } else {
	return [LayersCreateListWin_Old $w]
    }
}

proc SetSelection { T } {
    if { $::LAYERS_TREE } {
	return [SetSelection_New $T]
    } else {
	return [SetSelection_Old $T]
    }
}

proc SetSelection_New { T } {
    if { ![winfo exists $T] } { return }
    set LayersSelection ""
    set sel [$T selection get]
    foreach item $sel {
	if { [IsItemFolderOfLayers $T $item] } {
	    #folder of layers
	    foreach child [$T item descendants $item] {
		if { [IsItemFolderOfLayers $T $child] } {
		    #not necessary to recurse more, descendants include all childs and grandchilds...
		} else {
		    lappend LayersSelection [DecodeName [$T item tag names $child]]
		}
	    }
	} else {
	    lappend LayersSelection [DecodeName [$T item tag names $item]]
	}
    }
    set LayersSelection [lsort -unique -dictionary $LayersSelection]
    return $LayersSelection
}

proc FillTableInfoLayers_New { T {parent ""} {parentitem root} } {

#     if { ![winfo exists $T] } { return }

    set LayerNames [GetChildLayerNames $parent]
    set LayerToUse [GiD_Info Project LayerToUse]

    set BackLayerNames [GiD_Info back_layers]

    if { $BackLayerNames != "" } {
	regsub -all {_\*back\*} $BackLayerNames {} pp
	set BackLayerNames $pp
    }

    foreach i $LayerNames {
	if { $parent != "" } {
	    set fullname $parent$i
	} else {
	    set fullname $i
	}
	if { [IsFolderOfLayers $i] } {
	    #is a folder of layers
	    set item [$T item create -button yes -tags [EncodeName $fullname]]
	    $T item lastchild $parentitem $item
	    $T item style set $item C0 styAnyRead
	    $T item element configure $item C0 elemTxtRead -text [string range $i 0 end-2]
	    #only set color,etc if all childs are equal
	    #invoke recursivelly
	    FillTableInfoLayers_New $T $fullname $item
	} else {
	    set item [$T item create -button no -tags [EncodeName $fullname]]
	    $T item lastchild $parentitem $item
	    $T item style set $item C0 styAnyRead
	    $T item element configure $item C0 elemTxtRead -text $i
	
	    $T item style set $item C1 styAnyImage
	    if { $fullname == $LayerToUse } {
#                 $T item element configure $item C0 elemTxtRead -text $i + elemImgAny -image [GetImage layer_touse.gif]
		$T item element configure $item C1 elemImgAny -image [GetImage layer_touse.gif]
#                 $T item state set $item touse
	    } else {
#                 $T item element configure $item C0 elemTxtRead -text $i
		$T item element configure $item C1 elemImgAny -image [GetImage layer_nottouse.gif];#void image
#                 $T item state set $item !touse
	    }
	
	    set LayersProp [lindex [GiD_Info layers $fullname] 0]
	    set cur_col [format "#%02x%02x%02x" [lindex $LayersProp 2] \
		    [lindex $LayersProp 3] [lindex $LayersProp 4]]
	    $T item style set $item C2 styRectColor
	    $T item element configure $item C2 elemRectColor -fill $cur_col

	    $T item style set $item C3 styAnyImage
	    if { [lindex $LayersProp 0] } {
		$T item element configure $item C3 elemImgAny -image [GetImage layer_on.gif]
#                 $T item state set $item on
	    } else {
		$T item element configure $item C3 elemImgAny -image [GetImage layer_off.gif]
#                 $T item state set $item !on
	    }
		

	    $T item style set $item C4 styAnyImage
	    if { [lindex $LayersProp 1] } {
		$T item element configure $item C4 elemImgAny -image [GetImage layer_freeze.gif]
#                 $T item state set $item freeze
	    } else {
		$T item element configure $item C4 elemImgAny -image [GetImage layer_unfreeze.gif]
#                 $T item state set $item !freeze
	    }
	
	    $T item style set $item C5 styAnyImage
	    if { [lindex $LayersProp 5] == 255 } {
		$T item element configure $item C5 elemImgAny -image [GetImage layer_transparent_off.gif]
#                 $T item state set $item transparent
	    } else {
		$T item element configure $item C5 elemImgAny -image [GetImage layer_transparent_on.gif]
#                 $T item state set $item !transparent
	    }
	
	    $T item style set $item C6 styAnyImage
	    if { [ lsearch $BackLayerNames $fullname] != -1 } {
		$T item element configure $item C6 elemImgAny -image [GetImage layer_bringtofront.gif]
#                 $T item state set $item back
	    } else {
		$T item element configure $item C6 elemImgAny -image [GetImage layer_sendtoback.gif] ;#void image
#                 $T item state set $item !back
	    }
		
	}
    }
    if { [$T column cget C0 -arrow] == "up" } {       
	$T item sort 0 -dictionary -decreasing
    } else {
	$T item sort 0 -dictionary -increasing
    }
}


proc GetOldAndNewLayernamesRecursive { T dragged_list dstname parentname } {
    set cmd ""
    foreach item $dragged_list {
	if { [IsItemFolderOfLayers $T $item] } {
	    lappend cmd {*}[GetOldAndNewLayernamesRecursive $T [$T item children $item] $dstname [$T item text $item 0]//]
	} else {
	    set fullname [DecodeName [$T item tag names $item]]
	    set newname $dstname$parentname[$T item text $item 0]
	    lappend cmd ChangeName $fullname $newname
	}
    }
    return $cmd
}

proc ReceiveDragLayers { T dragged_list dst } {
    set dstname [DecodeName [$T item tag names $dst]]
    if { [IsItemFolderOfLayers $T $dst] } {
	#folder of layers
	set cmd [GetOldAndNewLayernamesRecursive $T $dragged_list $dstname ""]
	GiD_Process Layers {*}$cmd escape
    } else {
	WarnWin "$dstname is not a folder"
    }
}


proc SetLayersToRename_New { T item newtext } {
    if { $newtext == ""} return
    if { $item == 0 } {
	WarnWin [_ "Root folder can't be edited"]
	return
    }
    set fullname [DecodeName [$T item tag names $item]]
    if { [IsFolderOfLayers $fullname] } {
	#must rename all children
	set splitted [split2 $fullname //]
	set index [expr {[llength $splitted]-2}]
	if { [lindex $splitted $index] != $newtext } {
	    set layers ""
	    foreach child [$T item descendants $item] {
		set fullnamechild [DecodeName [$T item tag names $child]]
		if { [IsFolderOfLayers $fullnamechild] } {
		    #is not a real layer
		} else {
		    lappend layers $fullnamechild
		}
	    }
	    foreach fullnamechild $layers {
		set parts [split2 $fullnamechild //]
		lset parts $index $newtext
		set newname [join $parts //]
		GiD_Process Layers ChangeName $fullnamechild $newname
	    }
	    GiD_Process escape
	}
    } else {
	set parts [split2 $fullname //]
	lset parts end $newtext
	set newname [join $parts //]
	if { $fullname != $newname } {
	    GiD_Process Layers ChangeName $fullname $newname escape
	}
    }
}


proc BeginEditLayers { T } {
    set I [$T item id active]
    set C 0
    set E elemTxtRead
    ::TreeCtrl::FileListEdit $T $I $C $E
}

proc UpdateRootNodeVisibility { T } {
    #show root node if exist layer names with tree structure
    set showroot 0
    foreach layer [GiD_Info layers] {
	if { [string first // $layer] != -1 } {
	    set showroot 1
	    break
	}
    }
    $T configure -showroot $showroot
}

proc LayersCreateListWin_New { w } {

    set USE_TreePlusScrollbarsInAFrame 0
    if { $USE_TreePlusScrollbarsInAFrame } {
	TreePlusScrollbarsInAFrame $w.f 1 1
	set T $w.f.t
    } else {
	set vsb $w.vsb1
	set hsb $w.hsb1
	set T [treectrl $w.t -xscrollcommand [list $hsb set] -yscrollcommand [list $vsb set]]
	ttk::scrollbar $vsb -orient vertical   -command [list $T yview]
	ttk::scrollbar $hsb -orient horizontal -command [list $T xview]
    }

    set height [font metrics [$T cget -font] -linespace]
    if {$height < 18} {
	set height 18
    }

    $T configure -indent 15 -itemheight $height -selectmode extended \
	-showroot 0 -showrootbutton 0 -showbuttons 1 -showlines 1 \
	-highlightthickness 0 -borderwidth 0 \
	-xscrollincrement 20 -yscrollincrement 20
	#-scrollmargin 16 -xscrolldelay "500 50" -yscrolldelay "500 50"


    set coltxt [list [_ "Name"] { } [_ "C#C#Color"] [_ "I/O#C#On/Off"] \
	    [_ "F/U#C#Freeze/Unfreeze"] [_ "Tr#C#transparent"] [_ "B#C#back"]]

#     $T state define touse
#     $T state define on
#     $T state define freeze
#     $T state define transparent
#     $T state define back
	
    set i 0
    foreach txt $coltxt {
	$T column create -text $txt -tags C$i -weight 0
	incr i
    }
    $T column configure C0 -weight 1 -arrow down
    #     $T column configure tail -width 0
    #     $T column configure tail -visible 0
    $T configure -treecolumn C0

    #create images
    #     image create photo layer_folder -file {C:\gid project\bitmaps\folder.gif}

    # Create elements
    $T element create elemImgAny image
    $T element create elemTxtRead text -fill [list $::SystemHighlightText {selected focus}] \
	-lines 1
    $T element create elemRectSel rect -fill [list $::SystemHighlight {selected focus} gray {selected !focus}] \
		-showfocus yes
    $T element create elemRectColor rect -width 30
    # Create styles using the elements
    set S [$T style create styAnyRead]
    $T style elements $S {elemRectSel elemTxtRead elemImgAny}
    $T style layout $S elemImgAny -expand ns
    $T style layout $S elemTxtRead -padx 4 -expand ns -squeeze x
    $T style layout $S elemRectSel -union [list elemTxtRead] -iexpand ns -ipadx 2

    set S [$T style create styRectColor]
    $T style elements $S {elemRectColor elemTxtRead}
    $T style layout $S elemRectColor -union [list elemTxtRead] -iexpand ns -ipadx 2

    set S [$T style create styAnyImage]
    $T style elements $S {elemImgAny}
    $T style layout $S elemImgAny -expand ns
	

    set item root
    $T item configure $item -button yes
    $T item style set $item C0 styAnyRead
    $T item element configure $item C0 elemTxtRead -text [_ "Root"]
    #$T item element configure $item C0 elemImgAny -image [GetImage layer16x16.gif]


    # List of lists: {column style element ...} specifying text elements
    # the user can edit
    TreeCtrl::SetEditable $T {
	{C0 styAnyRead elemTxtRead}
    }

    # List of lists: {column style element ...} specifying elements
    # the user can click on or select with the selection rectangle
    TreeCtrl::SetSensitive $T {
	{C0 styAnyRead elemTxtRead}
    }


    # List of lists: {column style element ...} specifying elements
    # added to the drag image when dragging selected items
    TreeCtrl::SetDragImage $T {
		{C0 styAnyRead elemTxtRead}
    }

#     $T notify install <Drag-begin>
#     $T notify install <Drag-end>
    $T notify install <Drag-receive>

#     $T notify install <Edit-begin>
#     $T notify install <Edit-end>
    $T notify install <Edit-accept>

#     $T notify bind DragTag <Drag-begin> { DragEvent begin %T %l %I }
#     $T notify bind DragTag <Drag-end> { DragEvent end %T %l %I }
    $T notify bind DragTag <Drag-receive> { ReceiveDragLayers %T %l %I}

    $T notify bind EditTag <Edit-accept> { SetLayersToRename_New %T %I %t }

    bind $T <Button-1> [list ClickTableList_New %x %y $T]
    bind $T <Double-Button-1> [list DoubleClickTableList_New %x %y $T]
    bind $T <Return> [list SetLayersTo TOUSE $T]
    bind $T <Key-Delete> [list SetLayersToDelete $T]
    bind $T <Alt_L> [list InvertSelectionTableList $T]
    bind $T <Alt_R> [list InvertSelectionTableList $T]
    bind $T <Meta_L> [list InvertSelectionTableList $T]
    bind $T <Meta_R> [list InvertSelectionTableList $T]
    bind $T <F2> [list BeginEditLayers $T]

    bind $T <Button-3> "[list MenuContextualLayer %W %x %y] ; break"


    if { $USE_TreePlusScrollbarsInAFrame } {
	grid $T -sticky nsew
	grid $w.f -sticky nsew
    } else {
	grid $T $vsb -sticky nsew
	grid configure $vsb -sticky ns
	grid $hsb -sticky ew
	grid remove $vsb $hsb
	bind $T <Configure> [list ConfigureListScrollbars $T $hsb $vsb]
    }

    grid columnconfigure $w 0 -weight 1
    grid rowconfigure $w 0 -weight 1

#     bindtags $T [list $T DemoExplorer TreeCtrlFileList TreeCtrl [winfo toplevel $T] all]
    bindtags $T [list $T TreeCtrlFileList TreeCtrl [winfo toplevel $T] all]

    return $T
}


proc LayersCreateListWin_Old { w} {
    global GidPriv

    set vsb $w.vsb1
    set hsb $w.hsb1
    set tbl [tablelist::tablelist $w.tbl \
	    -exportselection 0 \
	    -columns [list \
	    0 [_ "C#C#Color"] left \
	    5 [_ "Name"] left \
	    0 [_ "I/O#C#On/Off"] left \
	    0 [_ "F/U#C#Freeze/Unfreeze"] left \
	    0 [_ "Tr#C#transparent"] left \
	    0 [_ "B#C#back"] left \
	    ] \
	    -selectbackground black -selectforeground white \
	    -background $GidPriv(Color,BackgroundListbox) \
	    -stretch 1 \
	    -selectmode extended \
	    -labelborderwidth 1 \
	    -labelcommand tablelist::sortByColumn \
	    -labelpady 0 \
	    -showarrow 0 \
	    -showseparators 0 \
	    -xscrollcommand [list $hsb set] \
	    -yscrollcommand [list $vsb set] \
	    -stripebackground [CCColorSombra $GidPriv(Color,BackgroundListbox)] \
	    -resizablecolumns 0]
	    #-labelcommand tablelist::sortByColumn -showarrow 1
  #-highlightthickness 0

    ttk::scrollbar $vsb -orient vertical   -command [list $tbl yview]
    ttk::scrollbar $hsb -orient horizontal -command [list $tbl xview]

    # bind [$tbl bodypath] <Button-1> [list ClickTableList %x %y %W $tbl]
    # correcion for tablelist 4.1, else the binding of the image inside the label is lost
    bind [$tbl bodytag] <Button-1> "\
	    foreach {tablelist::W tablelist::x tablelist::y} \
	    \[tablelist::convEventFields %W %x %y] {}
    ClickTableList \$tablelist::x \$tablelist::y %W $tbl"

    #bind [$tbl bodypath] <Double-Button-1> [list DoubleClickTableList %x %y %W $tbl]
    bind [$tbl bodytag] <Double-Button-1> "\
	    foreach {tablelist::W tablelist::x tablelist::y} \
	    \[tablelist::convEventFields %W %x %y] {}
    DoubleClickTableList \$tablelist::x \$tablelist::y %W $tbl"

    bind [$tbl bodytag] <Return> [list SetLayersTo TOUSE $tbl]
    bind [$tbl bodytag] <Key-Delete> [list SetLayersToDelete $tbl]


    bind [$tbl bodypath] <Motion> [list MotionTableList %x %y %W $tbl]

    bind [$tbl bodypath] <Alt_L> [list InvertSelectionTableList $tbl]
    bind [$tbl bodypath] <Alt_R> [list InvertSelectionTableList $tbl]
    bind [$tbl bodypath] <Meta_L> [list InvertSelectionTableList $tbl]
    bind [$tbl bodypath] <Meta_R> [list InvertSelectionTableList $tbl]

    bind [$tbl bodypath] <F2> [list SetLayersToRename_Old $tbl]

    grid $tbl $vsb -sticky nsew
    grid configure $vsb -sticky ns
    grid $hsb -sticky ew
    grid remove $vsb $hsb
    bind $tbl <Configure> [list ConfigureListScrollbars $tbl $hsb $vsb]

    grid columnconfigure $w 0 -weight 1
    grid rowconfigure $w 0 -weight 1
    return $tbl
}

proc ChangeLayers { {w .gid.central.wlay} } {
    if { $::LAYERS_TREE } {
	if { ![info exists ::GidPriv(LayerLayout)] } {
	    set ::GidPriv(LayerLayout) OUTSIDE ;
	    #not use INSIDERIGTH because sharing gid.ini cause problems for previous GiD's in ReadDefaultValues
	}
	if { $::GidPriv(LayerLayout) == "RIGHT" } {
	    set inside 1
	} else {
	    set inside 0
	}
	#problems with panedwindow: without after idle have bad size when restarting
	#gid with Layers window inside
	after idle ChangeLayers_New $w $inside
    } else {
	ChangeLayers_Old $w
    }
}


proc WriteLayerGeomToVar { w what geomname {InitComm ""}} {
    global GidPriv
    set trans 1
    update idletasks
    if { $::GidPriv(LayerLayout) == "RIGHT" } {
	#maintain all dimensions except the new width
	set prevgeom [lindex $GidPriv($geomname) 1]
	foreach {width height x y} [split $prevgeom x+] break
	set width [winfo width $w]
    } else {
	#"OUTSIDE"
	set width [winfo width $w]
	set height [winfo height $w]
	set x [winfo x $w]
	set y [winfo y $w]
    }
    set geom ${width}x${height}+${x}+${y}
    set GidPriv($geomname) "$what $geom $trans $InitComm"
#     WarnWinText $GidPriv($geomname)
}

########################
#   Some changes to take account GidPriv(pwCentral), a panedwindow to manage several windows
########################
proc ChangeLayers_New { w {inside 1} } {
    global GidPriv
    
    if { $inside } {
	open_window_inside $w
	if { ![winfo exists $w] } return ;# windows disabled || usemorewindows == 0
	if { [winfo class $w] == "Toplevel" } return 

	#######
	# pwCentral is a panedwindow to manage $grWindow with other windows in GiD
	if {![info exist GidPriv(pwCentral)]} return
	if {![winfo exist $GidPriv(pwCentral)]} return
	#######
	
	set p [frame $w.gg]
	
	
	ChangeLayers_New2 $p
	#set size
	pack $p -expand 1 -fill both
	update idletasks
	if { [info exists ::GidPriv(LayersWindowGeom)] } {
	    set w1 [lindex [split [lindex $::GidPriv(LayersWindowGeom) 1] x] 0]
	} else {
	    set w1 [winfo reqwidth $p]
	}
	set sepw 4
	set w0 [expr [winfo width $GidPriv(pwCentral)]-$w1-$sepw]
	if { $w0<=0 || $w1<=0 } {
	    set w0 750
	    set w1 250
	}

	set wsash [expr [lsearch [$GidPriv(pwCentral) panes] $w]-1]
	if {$wsash < 0} {
	    set wsash 0
	}

	
#         $w add $p ;#raise reshape_cb of togl with unexpected width!!
	if { [winfo class $GidPriv(pwCentral)] == "Panedwindow" } {
	    #set w0 [expr {$w0-[$w cget -sashwidth]}]
	    #problems with panedwindow: without update idletasks have bad size when
	    #reopening again the layers window inside
	    update idletasks
	    $GidPriv(pwCentral) sash place $wsash $w0 0 ;#raise reshape_cb of togl
	    #$GidPriv(pwCentral) paneconfigure $p -width $w1
	} elseif { [winfo class $GidPriv(pwCentral)] == "TPanedwindow" } {
	    #ttk::pane
	    #$GidPriv(pwCentral) pane 0 -weight $w0
	    #$GidPriv(pwCentral) pane 1 -weight $w1
	    $GidPriv(pwCentral) sashpos $wsash $w0
	}

	set ::GidPriv(LayerLayout) "RIGHT"
	#WriteLayerGeomToVar $p OPEN LayersWindowGeom ChangeLayers
	bind $p <Configure> [list WriteLayerGeomToVar $p OPEN LayersWindowGeom ChangeLayers]
    } else {
	open_window_outside $w
	if { ![winfo exists $w] } return ;# windows disabled || usemorewindows == 0
	set ::GidPriv(LayerLayout) "OUTSIDE"
	ChangeLayers_New2 $w
	#add lower buttons
	ttk::frame $w.buts
	ttk::button $w.buts.can -text [_ "Close"] -command [list destroy $w]
	grid $w.buts.can -sticky ew -padx 5 -pady 6
	grid $w.buts -sticky ews
	grid anchor $w.buts center
	bind $w <Alt-c> "destroy $w"
	bind $w <Escape> "destroy $w"
    }
}

proc ChangeLayers_New2 { w } {
    global wChangeLay LayersNoRedraw GidPriv
    set wChangeLay $w
    set LayersNoRedraw 0
	

    if { ![info exists GidPriv(LayersAlsoLower)] } {
	set GidPriv(LayersAlsoLower) 1
    }
    if { ![info exists GidPriv(LayersBackOpposite)] } {
	set GidPriv(LayersBackOpposite) 0
    }

    ttk::frame $w.caption -relief solid -borderwidth 1
    #ttk::label $w.caption.title -foreground grey60
    label $w.caption.title -foreground grey60

#     label $w.caption -bg $::SystemBackgroundHi -bd 1 -relief solid -height 1 -cursor hand2 -fg white -text $txt -pady 2
    GidHelp $w.caption [_ "Double-click toggle window or inline visualization"]
    setTooltip $w.caption [_ "Double-click toggle window or inline visualization"]
    if { [winfo class $w] != "Toplevel" } {
	$w.caption.title configure -text [_ "Layers"]
	button $w.caption.close -image [GetImage close17.gif] -command [list close_window_inside [winfo parent $w]] -bd 0
	#bind $w.caption.close <1> [list close_window_inside [winfo parent $w]]
	$w.caption.title configure -text [_ "Double click here to tear off the window"]
    } else {
	$w.caption.title configure -text [_ "Double click here to integrate the window"]
    }

    ttk::frame $w.middle
    set T [LayersCreateListWin $w.middle]
    set GidPriv(Layers,table) $T

    ttk::frame $w.top
    #new16x16.gif
    ttk::button $w.top.newlayer -image [GetImage newtopic.gif]  -command [list CreateNewLayer $T] -style Toolbutton
    GidHelp $w.top.newlayer [_ "Create a new layer."]
    ttk::button $w.top.newfolder -image [GetImage newfolder.gif]  -command [list CreateNewLayerFolder $T] -style Toolbutton
    GidHelp $w.top.newfolder [_ "Create a new folder with one layer."]
    #ttk::button $w.top.rename -image [GetImage rename2.gif]  -command [list BeginEditLayers $T]
    #GidHelp $w.top.rename [_ "Rename a layer."]
    ttk::button $w.top.delete -image [GetImage erase-cross.gif]  -command [list SetLayersToDelete $T] -style Toolbutton
    GidHelp $w.top.delete [_ "Delete a layer."]
    ttk::menubutton $w.top.sendto -image [GetImage sendto.gif] -menu $w.top.sendto.m \
	-takefocus 1 -style Toolbutton
    GidHelp $w.top.sendto \
	[_ "Send the selected entities to a layer.\
	   If 'Also lower entities' is checked, the subentities are also send to the layer"]
    CreateMenuSendTo $w.top.sendto.m $T
    ttk::menubutton $w.top.toback -image [GetImage sendtoback.gif] -text [_ "To back"] -menu $w.top.toback.m \
	-takefocus 1 -style Toolbutton
    GidHelp $w.top.toback \
	[_ "Send entities to back/front of a layer (entities in back are not drawn and frozen)"]
    CreateMenuBack $w.top.toback.m $T

    ttk::menubutton $w.top.entitylayer -menu $w.top.entitylayer.m -width 0 -image [GetImage layerofentity.gif] -style Toolbutton
    GidHelp $w.top.entitylayer [_ "This option permmits to select one entity and set its layer as layer to use."]
    menu $w.top.entitylayer.m -postcommand [list LayerToUseEntities $w.top.entitylayer.m]

    ttk::button $w.top.filter -image [GetImage filter.gif]  -command [list SelectLayersByRegexp $T] -style Toolbutton
    GidHelp $w.top.filter \
	[_ "Select layers with filters (for example: 'b*' select all the layers with name beginning by 'b')"]

    #grid $w.top.newlayer $w.top.newfolder $w.top.rename  $w.top.delete ...
    grid $w.top.newlayer $w.top.newfolder $w.top.delete \
	 $w.top.sendto $w.top.toback $w.top.entitylayer $w.top.filter -sticky w


    if { [winfo class $w] != "Toplevel" } {
	grid $w.caption.title $w.caption.close -sticky ew
	grid configure $w.caption.close  -sticky w
    } else {
	grid $w.caption.title -sticky ew
    }
    grid $w.caption -sticky ew
    grid columnconfigure $w.caption 0 -weight 1
    grid $w.top -sticky ew
    grid $w.middle -sticky wens


    grid columnconfigure $w 0 -weight 1
    grid rowconfigure $w 2 -weight 1

    FillInfoLayers

    bind $w.caption.title <Double-Button-1> [list switch_window_inside_outside]

    focus $T
}

proc ChangeLayers_Old { w } {
    global wChangeLay LayersNoRedraw GidPriv
    set wChangeLay $w
    set LayersNoRedraw 0
    InitWindow $w [_ "Layers"] LayersWindowGeom ChangeLayers
    if { ![winfo exists $w] } return ;# windows disabled || usemorewindows == 0

    ttk::frame $w.middle

    set tbl [LayersCreateListWin $w.middle]
    set GidPriv(Layers,table) $tbl
#     GidHelp $w.middle \
#         [_ "Layer properties:\n Color, Name, Visible/Hidden, Frozen/Unfrozen, Exists back entities."]
    pack $w.middle -expand yes -fill both

    ttk::frame $w.options -relief sunken
    ttk::button $w.options.col -text [_ "Color"]... -command [list SetLayersToColor $tbl]
    GidHelp $w.options.col [_ "Changes the color used to draw the entities in render mode."]
    ttk::button $w.options.on -text [_ "On#C#verb"] -command [list SetLayersTo ON $tbl] -underline 0
    GidHelp $w.options.on [_ "Entities belonging to a layer ON will be drawn."]
    ttk::button $w.options.off -text [_ "Off#C#verb"] -command [list SetLayersTo OFF $tbl] -underline 1
    GidHelp $w.options.off [_ "Entities belonging to a layer OFF will not be drawn."]
    grid $w.options.on $w.options.off $w.options.col -sticky e -padx 2 -pady 1
    grid rowconfigure $w.options 0 -weight 1
    grid columnconfigure $w.options {0 1 2} -weight 1

    ttk::frame $w.options2 -relief sunken

    ttk::button $w.options2.fr -command [list SetLayersTo FREEZE $tbl] -image [GetImage Lock-on.gif]
    GidHelp $w.options2.fr [_ "Entities belonging to a freeze layer will be drawn but cannot be selected or modified."]
    ttk::button $w.options2.ufr  -command [list SetLayersTo UNFREEZE $tbl] -image [GetImage Lock-off.gif]
    GidHelp $w.options2.ufr [_ "Entities belonging to a unfreeze layer will be selected and modified."]

    ttk::button $w.options2.opaque -command [list SetLayersTo OPAQUE $tbl] -image [GetImage opaco.gif]
    GidHelp $w.options2.opaque [_ "Turns the selected layers opaque."]
    ttk::button $w.options2.transparent -command [list SetLayersTo TRANSPARENT $tbl] -image [GetImage transparente.gif]
    GidHelp $w.options2.opaque [_ "Turns the selected layers transparent."]

    ttk::button $w.options2.sel -text [_ "Sel"]... -command [list SelectLayersByRegexp $tbl]
    GidHelp $w.options2.sel \
	[_ "Select layers with filters (for example: 'b*' select all the layers with name beginning by 'b')"]
    grid $w.options2.fr $w.options2.ufr $w.options2.opaque $w.options2.transparent $w.options2.sel -sticky w
    grid configure $w.options2.sel -sticky e
    grid rowconfigure $w.options2 0 -weight 1
    grid columnconfigure $w.options2 {4} -weight 1

    bind $w <Alt-o> "$w.options.on invoke; break"
    bind $w <Alt-f> "$w.options.off invoke; break"
    bind $w <Alt-e> "$w.options2.fr invoke; break"
    bind $w <Alt-u> "$w.options2.ufr invoke; break"

    bind $w <Control-o> "$w.options.on invoke; break"
    bind $w <Control-f> "$w.options.off invoke; break"
    bind $w <Control-e> "$w.options2.fr invoke; break"
    bind $w <Control-u> "$w.options2.ufr invoke; break"

    ttk::frame $w.touse
    ttk::button $w.touse.b -text [_ "Layer To use"]: -command [list SetLayersTo TOUSE $tbl]
    GidHelp $w.touse.b [_ "Set the current layer (the new entities are created in this layer)."]

    ttk::menubutton $w.touse.m -menu $w.touse.m.m -width 0 -text "<<" -direction flush
    GidHelp $w.touse.m [_ "This option permmits to select one entity and retrieve its layer."]
    menu $w.touse.m.m -postcommand [list LayerToUseEntities $w.touse.m.m]
    label $w.touse.l -foreground blue

    set LayerToUse [GiD_Info Project LayerToUse]
    $w.touse.l conf -text $LayerToUse
    grid $w.touse.b -row 0 -column 0 -sticky e -padx 0 -pady 1
    grid $w.touse.m -row 0 -column 1 -sticky wns -padx 0 -pady 1
    grid $w.touse.l -row 0 -column 2 -sticky w -padx 3 -pady 1

    grid rowconfigure $w.touse 0 -weight 1
    grid columnconfigure $w.touse 2 -weight 1

    ttk::frame $w.ent
    ttk::entry $w.ent.e
    ttk::button $w.ent.b -text [_ "New#C#layer"] -command [list NewLayerFromEntry $tbl $w]
    GidHelp $w.ent.b [_ "Create a new layer."]

    #SetWidgetsWidthFromText $w.options.col $w.options.on $w.options.off $w.options2.sel $w.touse.b $w.ent.b

    bind $w.ent.e <Return> [list NewLayerFromEntry $tbl $w]
#     "update idletasks; $w.ent.b invoke"


    grid $w.ent.e $w.ent.b -sticky ew -padx 2 -pady 1
    grid rowconfigure $w.ent 0 -weight 1
    grid columnconfigure $w.ent 0 -weight 1

    ttk::frame $w.dr
    ttk::button $w.dr.bdel -text [_ "Delete"] -command [list SetLayersToDelete $tbl]
    GidHelp $w.dr.bdel [_ "Delete a layer."]
    ttk::button $w.dr.bren -text [_ "Rename"] -command [list SetLayersToRename_Old $tbl]
    GidHelp $w.dr.bren [_ "Rename a layer."]

    #SetWidgetsWidthFromText $w.dr.bdel $w.dr.bren

    ttk::checkbutton $w.dr.bred -text [_ "alphabetic"] -variable GidPriv(LayersAlphabetic) \
	    -command FillInfoLayers
    GidHelp $w.dr.bred [_ "Sort the list of layers by name."]

    grid $w.dr.bdel $w.dr.bren $w.dr.bred -sticky we -padx 1 -pady 1

    ttk::frame $w.buts

    if { ![info exists GidPriv(LayersAlsoLower)] } {
	set GidPriv(LayersAlsoLower) 1
    }
    if { ![info exists GidPriv(LayersBackOpposite)] } {
	set GidPriv(LayersBackOpposite) 0
    }

    ttk::menubutton $w.buts.back -text [_ "To back"] -menu $w.buts.back.m -takefocus 1
    GidHelp $w.buts.back \
	[_ "Send entities to back/front of a layer (entities in back are not drawn and frozen)"]
    CreateMenuBack $w.buts.back.m $tbl

    ttk::menubutton $w.buts.act -text [_ "Send To"] -menu $w.buts.act.m -takefocus 1
    GidHelp $w.buts.act \
	[_ "Send the selected entities to a layer.\
	   If 'Also lower entities' is checked, the subentities are also send to the layer"]

    CreateMenuSendTo $w.buts.act.m $tbl

    ttk::button $w.buts.can -text [_ "Close"] -command [list destroy $w]

    #SetWidgetsWidthFromText $w.buts.back $w.buts.act $w.buts.can

    grid $w.buts.back $w.buts.act $w.buts.can -sticky ew -padx 5 -pady 6

    grid $w.middle -sticky wens -pady 0
    grid $w.touse -sticky we -pady 0
    grid $w.options -sticky we -pady 0
    grid $w.options2 -sticky we -pady 0
    grid $w.ent -sticky we -pady 0
    grid $w.dr -sticky ew -pady 0
    grid $w.buts -sticky ews
    if { $::tcl_version >= 8.5 } { grid anchor $w.buts center }
    grid columnconfigure $w 0 -weight 1
    grid rowconfigure $w 0 -weight 1

    FillInfoLayers

    bind $w <FocusIn> "focus $w.ent.e ; bind $w <FocusIn> {}"
    bind $w <Alt-c> "destroy $w"
    bind $w <Escape> "destroy $w"
}

proc CreateMenuBack { m T } {
    global GidPriv
    menu $m -postcommand [list SendLayerEntitiesToBack $m $T]
    $m add check -label [_ "Also lower entities"] -var GidPriv(LayersAlsoLower)
    $m add check -label [_ "Opposite"] -var GidPriv(LayersBackOpposite)
    $m conf -activeborderwidth 1
}

proc CreateMenuSendTo { m T } {
    global GidPriv
    menu $m -postcommand [list SendLayerEntities $m $T]
    $m add check -label [_ "Also lower entities"] -var GidPriv(LayersAlsoLower)
    $m conf -activeborderwidth 1
}

proc MenuContextualLayer { T x y } {
    set w $T.menucontextuallayer
    if { [winfo exists $w] } {
	destroy $w
    }

    menu $w

    $w add command -label [_ "New layer#C#layer"] -command [list CreateNewLayer $T]
    $w add command -label [_ "New folder#C#layer"] -command [list CreateNewLayerFolder $T]

    set nfolder 0
    set nsingle 0
    foreach item [$T item range 0 end] {
	if { [$T selection includes $item] } {
	    if { [IsItemFolderOfLayers $T $item] } {
		incr nfolder
	    } else {
		incr nsingle
	    }
	}
    }
    if { [expr {$nsingle+$nfolder}] == 1 } {
	$w add command -label [_ "Rename"] -command [list BeginEditLayers $T] -state normal
    } else {
	$w add command -label [_ "Rename"] -state disabled
    }
    if { [expr {$nsingle+$nfolder}] > 0 } {
	$w add command -label [_ "Delete"] -command [list SetLayersToDelete $T] -state normal
    } else {
	$w add command -label [_ "Delete"] -state disabled
    }
    $w add separator

    if { [expr {$nsingle+$nfolder}] > 0 } {
	set state normal
    } else {
	set state disabled
    }
    $w add command -label [_ "Color"]... -command [list SetLayersToColor $T] -state $state
    $w add command -label [_ "On#C#verb"] -command [list SetLayersTo ON $T] -state $state
    $w add command -label [_ "Off#C#verb"] -command [list SetLayersTo OFF $T] -state $state
    $w add command -label [_ "Freeze#C#verb"] -command [list SetLayersTo FREEZE $T] -state $state
    $w add command -label [_ "Unfreeze#C#verb"] -command [list SetLayersTo UNFREEZE $T] -state $state
    $w add command -label [_ "Opaque#C#verb"] -command [list SetLayersTo OPAQUE $T] -state $state
    $w add command -label [_ "Transparent#C#verb"] -command [list SetLayersTo TRANSPARENT $T] -state $state
    $w add separator
    if { $nsingle == 1 && $nfolder == 0 } {
	$w add cascade -label [_ "Send to"] -menu $w.sendto -state normal
    } else {
	$w add cascade -label [_ "Send to"] -menu $w.sendto -state disabled
    }
    CreateMenuSendTo $w.sendto $T
    $w add cascade -label [_ "To back"] -menu $w.back
    CreateMenuBack $w.back $T
    $w add separator
    $w add command -label [_ "Filter"] -command [list SelectLayersByRegexp $T]

    set x [expr [winfo rootx $T]+$x+2]
    set y [expr [winfo rooty $T]+$y]
    GiD_PopupMenu $w $x $y
}

proc switch_window_inside_outside {  { w .gid.central.wlay } } {
    if { ![winfo exists $w] } {
	ChangeLayers_New $w 1
    } elseif { [winfo class $w] eq "Toplevel" } {
	close_window_outside $w
	ChangeLayers_New $w 1
    } else {
	close_window_inside $w
	ChangeLayers_New $w 0
    }
}

proc open_window_outside { w } {
    if { [winfo exists $w] } {
	close_window_outside $w
    }
    InitWindow $w [_ "Layers"] LayersWindowGeom ChangeLayers
    return $w
}

proc close_window_outside { w } {
    destroy $w
}


########################
#   This procedure take account new panedwindow GidPriv(pwCentral) to manage several windows
########################
proc close_window_inside { p } {
    global GidPriv
    
    if {![winfo exist $p]} return

    #######
    # pwCentral is a panedwindow to manage $grWindow with other windows in GiD
    if {![info exist GidPriv(pwCentral)]} {
	destroy $p
	return
    }
    #######
	
    if {[winfo exist $p.gg]} {
	bind $p.gg <Configure> ""
	WriteLayerGeomToVar $p.gg NONE LayersWindowGeom ChangeLayers
    }

    if { [winfo exists $GidPriv(pwCentral)] } {
	if { [winfo class $GidPriv(pwCentral)] in "Panedwindow TPanedwindow" } {
	    set panes [$GidPriv(pwCentral) panes]
	    set _pos [lsearch $panes $p]
	    if {$_pos >= 0} {
	    if { [llength $panes] == 2 } {
		    set oldwin [lreplace $panes $_pos $_pos]
		    set mg [winfo manager $GidPriv(pwCentral)]
		    set mginfo [$mg info $GidPriv(pwCentral)]        
		    $GidPriv(pwCentral) forget $p
		    $mg $oldwin {*}$mginfo
		    update idletask
		    destroy $GidPriv(pwCentral)
		    unset GidPriv(pwCentral)
		}
	    }
	    }
	} else {
	unset GidPriv(pwCentral)
	}
	destroy $p
    }

#create a new panedwindw named $p and a new ttk::frame wich name is returned
proc open_window_inside_prev { p } {
    if { [winfo exists $p] } {
	close_window_inside $p
    }

#     if { [lsearch -exact [list GEOMETRYUSE MESHUSE] [GiD_Info Project ViewMode]] == -1 } {
#         return
#     }
    panedwindow $p -bd 0 -showhandle 0 -sashpad 1 -sashrelief sunken -opaqueresize 0;#-bg #880000
    #ttk::panedwindow $p -orient horizontal

    set focus [focus]

    if { [winfo exists .gid.central.wins] } {
	set grWindow .gid.central.wins
    } else {
	set grWindow .gid.central.s
    }
    if { [grid slaves .gid.central] ne "" } {
	eval grid $p [grid info $grWindow]
    } elseif { [pack slaves .gid.central] ne "" } {
	pack $p -in .gid.central -before $grWindow \
	    -anchor center -expand 1 -fill both -side left
    } else {
	destroy $p
	return ""
    }

    $p add $grWindow

    raise $grWindow $p
    raise .gid.central.s

    set f [ttk::frame $p.gg]
    $p add $f

    #tkTabToWindow $p.gg
    if { $focus ne "" } {
	update
	catch { focus -force $focus }
    }
    return $f
}


########################
#   Procedure to open window inside, include creation of panedwindow to manage several windows
########################
proc open_window_inside { w } {
    global GidPriv
    
    if { [winfo exists $w] } {
	close_window_inside $w
    }

    if { [lsearch -exact {GEOMETRYUSE MESHUSE} [GiD_Info Project ViewMode]] == -1 } {
	return
    }
    
    frame $w

    set focus [focus]

    if { [winfo exists .gid.central.wins] } {
	set grWindow .gid.central.wins
    } else {
	set grWindow .gid.central.s
    }

    #######
    # pwCentral is a panedwindow to manage $grWindow with other windows in GiD
    if {![info exist GidPriv(pwCentral)]} {
	set GidPriv(pwCentral) .gid.central.pwCentral
    }
    #######

    if {![winfo exist $GidPriv(pwCentral)]} {
	set mg [winfo manager $grWindow]
	set mginfo [$mg info $grWindow]        
	$mg forget $grWindow

	panedwindow $GidPriv(pwCentral) -bd 0 -showhandle 0 -sashpad 1 -sashrelief sunken -opaqueresize 0;#-bg #880000
	#ttk::panedwindow $GidPriv(pwCentral) -orient horizontal

	$mg $GidPriv(pwCentral) {*}$mginfo
	$GidPriv(pwCentral) add $grWindow $w -minsize 60
	raise $grWindow
	if {$grWindow == ".gid.central.wins"} {
	    raise .gid.central.s
	}

	$GidPriv(pwCentral) paneconfigure $grWindow -stretch always 

	raise $w
    } else {
	set panes  [$GidPriv(pwCentral) panes]
	set maxpane [expr [llength $panes]-1]
	# Are there any window after grWindow?
	if {[lsearch $panes $grWindow] < $maxpane} {
	    close_window_inside $w
	    ChangeLayers_New $w 0
	} else {
	    # Save sash position
	    for {set i 0} {$i < $maxpane} {incr i} {
		set _wsash($i) [lindex [$GidPriv(pwCentral) sash coord $i] 0]
	    }
	    $GidPriv(pwCentral) add $w -after $grWindow -minsize 60
	    # Restore sash position
	    foreach i [array names _wsash]  {
		update idletask
		$GidPriv(pwCentral) sash place $i $_wsash($i) 0
	    }
	}
    }

    if { $focus ne "" } {
	update
	catch { focus -force $focus }
    }
    return $w
}
