
set PCR_ResultsGrouped 1

proc CreateMenuAnalyisStep { w cmd_in} {
    global GidPriv RMenuRPriv

    rmenuinit
    set all_anal [lsort -dictionary [GiD_Info postprocess get all_analysis]]
    # set all_anal [ GiD_Info postprocess get all_analysis]
    set cur_anal [ GiD_Info postprocess get cur_analysis]

    $w del 0 end

    set cur_step [ GiD_Info postprocess get cur_step $cur_anal]

    if { $all_anal == ""} {
	$w add command -label [_ "Please load a MODEL first"] -state disabled
	return
    }

    foreach i $all_anal {
	regsub -all {_} [= $i] { } menu_txt
	regsub -all {\.} $i {_} menu_name

	$w add cascade -label $menu_txt -menu $w.m$menu_name -compound left

	set all_step [ GiD_Info postprocess get all_steps $i]
	set cur_step [ GiD_Info postprocess get cur_step $i]
	if { [ winfo exists $w.m$menu_name ]} {
	    $w.m$menu_name del 0 end
	} else {
	    menu $w.m$menu_name
	}
	set words ""

	set add_more 0
	if { ( [ llength $all_step] > $RMenuRPriv(MaxAbsolute)) && ( $RMenuRPriv(MaxAbsolute) > 2)} {
	    set all_step [ lrange $all_step 0 [ expr $RMenuRPriv(MaxAbsolute) - 2]]
	    set add_more 1
	}

	foreach j $all_step {
	    if { ( $cur_anal != $i) || ( $cur_step != $j) } {
		lappend words [ list $j "$cmd_in $i $j"]
	    } else {
		lappend words [ list $j "$cmd_in $i $j" check]
	    }
	}
	rmenuconf $w.m$menu_name $words

	if { $add_more} {
	    $w.m$menu_name add separator
	    $w.m$menu_name add command -label ...[_ "More"]... -command PostGeom::Create
	}
    }
}

proc UpdateAnalysisText { number w args } {
    global PCRPriv
    set PCRPriv(analysis$number) [ lindex $args 0]
    set PCRPriv(step$number) [ lindex $args 1]
    regsub -all {_} [= $PCRPriv(analysis$number)] { } txt_a
    $w conf -text "$txt_a / $PCRPriv(step$number)"
}

proc PCRSelectAnalStep { number w text_id} {
    global PCRPriv

    if { ![ winfo exists $w.m]} {
	menu $w.m
    }
    CreateMenuAnalyisStep $w.m [list UpdateAnalysisText $number $text_id]
    $w.m post [ winfo pointerx $w] [ winfo pointery $w]
}

proc CreateMenuNodalResults { w cmd_in} {
    global GidPriv RMenuRPriv


    # without components
    # Result_Surface are nodal results
    # set wich_res "Result_Surface"
    # in nodos and in gauss points
    set wich_res "Contour_Fill"

    rmenuinit
    set all_anal [lsort -dictionary [ GiD_Info postprocess get all_analysis]]
    # set all_anal [ GiD_Info postprocess get all_analysis]
    if { $all_anal == ""} {
	set msg [_ "Please load a MODEL first"]
    } else {
	set msg [_ "Please select a STEP first"]
    }
    set cur_anal [ GiD_Info postprocess get cur_analysis]
    if { $cur_anal == ""} {
	$w del 0 end
	$w add command -label $msg -state disabled
	return
    }

    set cur_step [ GiD_Info postprocess get cur_step $cur_anal]
    if { $cur_step == ""} {
	$w add command -label [_ "Please select a STEP first"] -state disabled
	$w del 0 end
	return
    }

    if { $wich_res != "Deformation" } {
	set all_res [ GiD_Info postprocess get cur_results_list $wich_res ]
    } else {
	set all_res [ GiD_Info postprocess get show_geom_all_deform $cur_anal \
	    $cur_step ]
    }
    if { $all_res == ""} {
	$w del 0 end
	$w add command -label [_ "No useful results found"] -state disabled
	return
    }

    $w del 0 end

    if { $wich_res != "Deformation" } {
	set cur_res [ GiD_Info postprocess get cur_result]
	set cur_comp [ GiD_Info postprocess get cur_component]
    } else {
	set cur_res [ GiD_Info postprocess get main_geom_cur_deform]
	set cur_comp ""
    }

    set add_more 0
    if { [ llength $all_res] > $RMenuRPriv(MaxNumCol)} {
	set all_res [ lrange $all_res 0 [ expr $RMenuRPriv(MaxNumCol) - 2]]
	set add_more 1
    }

    foreach i $all_res {

	set wm [ GetResultMenuFromPath $w $i ""]

	regsub -all {_} [= $i] { } txt
	if { $wm != $w} {
	    if { [ regexp {^/?/?(.*)//(.*?)$} $i dum path res_txt]} {
		regsub -all {_} [= $res_txt] { } txt
	    }
	}

	$wm add command -label $txt -command "$cmd_in $i" -compound left
	
    }

    if { $add_more} {
	$w add separator
	$w add command -label ...[_ "More"]... -command PostGeom::Create
    }

}

proc UpdateResultText { number w args} {
    global PCRPriv
    set PCRPriv(result$number) [ lindex $args 0]

    regsub -all {_} [= $PCRPriv(result$number)] { } txt
    if { [ regexp {^/?/?(.*)//(.*?)$} $PCRPriv(result$number) dum path res_txt]} {
	regsub -all {_} [= $res_txt] { } txt
    }
    $w conf -text $txt
}


proc PCRSelectResult { number w text_id} {
    global PCRPriv

    if { ![ info exists PCRPriv(analysis$number)] || ![ info exists PCRPriv(step$number)]} {
	WarnWin [_ "Please, select an analysis and a step first."] $w
	return
    }

    GiD_Process escape escape escape escape results analysissel $PCRPriv(analysis$number) $PCRPriv(step$number)

    # Result_Surface are nodal results
    if { ![ winfo exists $w.m]} {
	menu $w.m
    }
    CreateMenuNodalResults $w.m [list UpdateResultText $number $text_id]
    $w.m post [ winfo pointerx $w] [ winfo pointery $w]
}

proc CreateMenuOperators { w cmd_in} {
    global GidPriv RMenuRPriv

    rmenuinit
    #set all_operadores "+ - * / y1x2z1" ;#"y1x2z1" is an example selecting mixed components of both
    set all_operadores "+ - * /"

    $w del 0 end
    foreach i $all_operadores {
	regsub -all {_} $i { } txt
	$w add command -label $txt -command "$cmd_in $i" -compound left
    }
}

proc UpdateOperationText { w args} {
    global PCRPriv
    set PCRPriv(operador) [ lindex $args 0]
    $w conf -text "   $PCRPriv(operador)   "
}

proc PCRSelectOperacion { w text_id} {
    global PCRPriv

    if { ![ winfo exists $w.m]} {
	menu $w.m
    }
    CreateMenuOperators $w.m "UpdateOperationText $text_id"
    $w.m post [ winfo pointerx $w] [ winfo pointery $w]
}

proc PCRApply { } {
    global PCRPriv

    foreach d "analysis1 step1 result1 operador analysis2 step2 result2 resultdest" \
	txt "{First Analysis} {First Step} {First Result} Operator {Second Analysis} {Second Step} {Second Result} {Result destination name}" \
	txt_trad [_ "{First Analysis} {First Step} {First Result} Operator {Second Analysis} {Second Step} {Second Result} {Result destination name}"] {
	if { ![ info exists PCRPriv($d)]} {
	    set txt1 [_ "Please, choose a correct '"]
	    append txt1 $txt_trad"'"
	    WarnWin $txt1 $PCRPriv(wid)
	    return
	}
    }
    
    if { $PCRPriv(analysis1) == $PCRPriv(analysis2) } {
	set analysis_dest $PCRPriv(analysis1)                
    } else {
	set analysis_dest [GiD_Info postprocess get cur_analysis]      
    }
    if { $PCRPriv(step1) == $PCRPriv(step2) } {
	set step_dest $PCRPriv(step1)
    } else {
	set step_dest [GiD_Info postprocess get cur_step [GiD_Info postprocess get cur_analysis]]
    }    

    if { $::PCRPriv(AllSteps)} {
	if { $PCRPriv(analysis1) != $PCRPriv(analysis2) } {
	    set all_steps1 [ GiD_Info postprocess get all_steps $PCRPriv(analysis1)]
	    set all_steps2 [ GiD_Info postprocess get all_steps $PCRPriv(analysis2)]
	    if { $all_steps1 != $all_steps2 } {
		WarnWin [_ "the steps of both analysis must be equal to set all steps"] $PCRPriv(wid)
		return                
	    } else {
		set all_steps $all_steps1
	    }                    
	} else {
	    set all_steps [ GiD_Info postprocess get all_steps $PCRPriv(analysis1)]            
	}
	foreach number {1 2} {
	    set ipos($number) [lsearch -sorted -real $all_steps $::PCRPriv(step$number)]
	    if { $ipos($number) == -1 } {
		WarnWin [_ "step %s not found in steps list" $::PCRPriv(step$number)] $PCRPriv(wid)
		return
	    }
	}
	#calculate relative position
	set imin [::tcl::mathfunc::min $ipos(1) $ipos(2)]
	set i_step_incre1 [expr {$ipos(1)-$imin}]
	set i_step_incre2 [expr {$ipos(2)-$imin}]
    } else {
	set all_steps $step_dest
    }
    set n_steps [ llength $all_steps]
    set i_step 0
    foreach step_dest $all_steps {
	if { $::PCRPriv(AllSteps)} {
	    set i_step1 [expr {$i_step+$i_step_incre1}]
	    set i_step2 [expr {$i_step+$i_step_incre2}]
	    if { $i_step1 >= $n_steps || $i_step2 >= $n_steps } {
		break
	    }
	    set step1 [lindex $all_steps $i_step1]
	    set step2 [lindex $all_steps $i_step2]
	} else {
	    set step1 $::PCRPriv(step1)            
	    set step2 $::PCRPriv(step2)
	}
	set err [ catch {
	    set res_info1 [GiD_Result get [ list $PCRPriv(resultdest) $analysis_dest $step_dest]]
	} errtxt ]
	if { !$err} {
	    # set resp [tk_dialogRAM $PCRPriv(wid).___pa_ww [_ "Warning"] \
	    #         [_ "Result '%s' already exists. Delete it?" $PCRPriv(resultdest)] \
	    #         gidquestionhead 1 [_ "Yes"] [_ "No#C#I don't want to do that"]]
	    # switch $resp {
	    #         0 {
		         GiD_Result delete [ list $PCRPriv(resultdest) $analysis_dest $step_dest]
		         GidUtils::SetWarnLine [_ "Result '%s' for step %s deleted" $PCRPriv(resultdest) $step_dest]
	    #         }
	    #         1 {
	    #             return
	    #         }
	    # }
	}
	
	#$PCRPriv(chivato) conf -text "Processing ..."
	
	set err [ catch {
	    set res_info1 [ GiD_Result get [ list $PCRPriv(result1) $PCRPriv(analysis1) $step1]]
	} errtxt ]
	if { $err} {
	    WarnWin [_ "Result '%s' does not exist: %s" $PCRPriv(result1) $errtxt] $PCRPriv(wid)
	    return
	}
	set err [ catch {
	    set res_info2 [ GiD_Result get [ list $PCRPriv(result2) $PCRPriv(analysis2) $step2]]
	} errtxt ]
	if { $err} {
	    WarnWin [_ "Result '%s' does not exist: %s" $PCRPriv(result2) $errtxt] $PCRPriv(wid)
	    return
	}

	# set header
	regsub -all { } $PCRPriv(resultdest) {_} dst_txt
	set cabecera [ lindex $res_info1 0]
	set res_dst [list [list Result $PCRPriv(resultdest) $analysis_dest $step_dest {*}[lrange $cabecera 4 end]]]

	# me salto las cabeceras
	set res_info1 [ lrange $res_info1 1 end]
	set res_info2 [ lrange $res_info2 1 end]

	set max_idx 0

	# create array first result
	foreach r1 $res_info1 {
	    set idx [ lindex $r1 0]
	    if { ![ regexp {[0-9]+} $idx]} {
		# if is not an index jump it
		# WarnWin "r1: $idx is not an index"
		continue
	    }
	    if { $idx > $max_idx} { set max_idx $idx}
	    if { $::PCR_ResultsGrouped} {
		set res [ lindex $r1 1]
	    } else {
		set res [ lrange $r1 1 end]
	    }
	    set result1(n_comp) [ llength $res]
	    for { set ic 0 } { $ic < $result1(n_comp)} { incr ic} {
		set result1($idx,$ic) [ lindex $res $ic]
	    }
	}
	# create array second result
	foreach r2 $res_info2 {
	    set idx [ lindex $r2 0]
	    if { ![ regexp {[0-9]+} $idx]} {
		# if is not an index jump it
		# WarnWin "r2: $idx is not an index"
		continue
	    }
	    if { $idx > $max_idx} { set max_idx $idx}
	    if { $::PCR_ResultsGrouped} {
		set res [ lindex $r2 1]
	    } else {
		set res [ lrange $r2 1 end]
	    }
	    set result2(n_comp) [ llength $res]
	    for { set ic 0 } { $ic < $result2(n_comp)} { incr ic} {
		set result2($idx,$ic) [ lindex $res $ic]
	    }
	}

	unset res_info1
	unset res_info2

	if { $result1(n_comp) != $result2(n_comp)} {
	    WarnWin [_ "Different number of components, please check the results selected"] $PCRPriv(wid)
	    return
	}

	if { $result1(n_comp) > 6} {
	    set result1(n_comp) 6
	    set result2(n_comp) 6
	} elseif { $result1(n_comp) == 4} {
	    set result1(n_comp) 3
	    set result2(n_comp) 3
	}

	#$PCRPriv(chivato) conf -text "Processing node 0 of $max_idx"
	set n_chivato 0

	set lst_res1_no_def {}
	set lst_res2_no_def {}

	for { set i 1} { $i <= $max_idx} { incr i} {
	    if { ![ info exists result1($i,0)]} {
		#WarnWinText [_ "result index %d does not exist for result '%s'. skipping it" \
		#                  [ expr $i + 1] $PCRPriv(result1)] $PCRPriv(wid)
		lappend lst_res1_no_def $i
		continue
	    }
	    if { ![ info exists result2($i,0)]} {
		#WarnWinText [_ "result index %d does not exist for result '%s'. skipping it" \
		#                  [ expr $i + 1] $PCRPriv(result2)] $PCRPriv(wid)
		lappend lst_res2_no_def $i
		continue
	    }
	    set dstres $i
	    #set lstres {}
	    if { $PCRPriv(operador) == "y1x2z1"} {
		# swap de x e y
		if { $result1(n_comp) >= 3} {
		    lappend dstres $result1($i,1)
		    lappend dstres $result2($i,0)
		    lappend dstres $result1($i,2)
		    for { set ic 3 } { $ic < $result1(n_comp)} { incr ic} {
		        lappend dstres $result1($i,3)
		    }
		}
	    } else {
		for { set ic 0 } { $ic < $result1(n_comp)} { incr ic} {
		    set res [ expr $result1($i,$ic) $PCRPriv(operador) $result2($i,$ic)]
		    #lappend lstres $res
		    lappend dstres $res
		}
	    }
	    #lappend dstres $lstres
	    lappend res_dst $dstres

	    incr n_chivato
	    if { $n_chivato == 1000} {
		# $PCRPriv(chivato) conf -text "Processing node $i of $max_idx"
		# set n_chivato 0
		set ::PCRPriv(var) [ expr int( ( ( ( 100.0 * $i) / $max_idx) * $i_step / $n_steps))]
		update idletasks
		update
	    }
	}

	# warn about not existent indices
	set len [ llength $lst_res1_no_def]
	if { $len != 0} {
	    if { $len == 1} {
		WarnWinText [_ "result index %s does not exist for result '%s'. skipping it" \
		                  [ lindex $lst_res1_no_def 0] $PCRPriv(result1)] $PCRPriv(wid)
	    } else {
		WarnWinText [_ "result indices %s \ndo not exist for result '%s'. skipping it" \
		                  [ join $lst_res1_no_def ", "] $PCRPriv(result1)] $PCRPriv(wid)
	    }
	}

	set len [ llength $lst_res2_no_def]
	if { $len != 0} {
	    if { $len == 1} {
		WarnWinText [_ "result index %s does not exist for result '%s'. skipping it" \
		                  [ lindex $lst_res2_no_def 0] $PCRPriv(result2)] $PCRPriv(wid)
	    } else {
		WarnWinText [_ "result indices %s \ndo not exist for result '%s'. skipping it" \
		                  [ join $lst_res2_no_def ", "] $PCRPriv(result2)] $PCRPriv(wid)
	    }
	}

	GiD_Result create {*}$res_dst
	GidUtils::SetWarnLine [_ "Result '%s' for %s step %s created" $PCRPriv(resultdest) $analysis_dest $step_dest]
	
	unset result1
	unset result2
	unset res_dst

	set ::PCRPriv(var) [ expr int( 100.0 * $i_step / $n_steps)]
	update idletasks
	update
	incr i_step
    }

    set ::PCRPriv(var) 100
    #$PCRPriv(chivato) conf -text "Done"
    #after 3000 "$PCRPriv(chivato) conf -text {}"
    after 3000 set ::PCRPriv(var) 0

    if { [ regexp {([^0-9]*)([0-9]+)$} $PCRPriv(resultdest) dum raiz num]} {
	set new_name "$raiz[ expr $num + 1]"
    } else {
	set new_name "$PCRPriv(resultdest) 2"
    }
    set PCRPriv(resultdest) $new_name
}

proc PCRClose { } {
    global PCRPriv
    destroy $PCRPriv(wid)
}

proc PCRIniVariables { } {
    global PCRPriv
    foreach d "analysis1 step1 result1 operador analysis2 step2 result2 resultdest" {
	if { [ info exists PCRPriv($d)]} {
	    unset PCRPriv($d)
	}
    }
}

proc PCRDeleteResult { } {
    global PCRPriv

    foreach number {1 2} {
	if { ![ info exists PCRPriv(analysis$number)] || ![ info exists PCRPriv(step$number)]} {
	    WarnWin [_ "Please, select an analysis and a step first."] $PCRPriv(wid)
	    return
	}
    }

    set err [ catch {
	    if { $PCRPriv(analysis1) == $PCRPriv(analysis2) } {
		set analysis_dest $PCRPriv(analysis1)                
	    } else {
		set analysis_dest [GiD_Info postprocess get cur_analysis]
	    }
	    if { $PCRPriv(step1) == $PCRPriv(step2) } {
		set step_dest $PCRPriv(step1)
	    } else {
		set step_dest [GiD_Info postprocess get cur_step [GiD_Info postprocess get cur_analysis]]
	    }
	    set res_info1 [ GiD_Result get [ list $PCRPriv(resultdest) $analysis_dest $step_dest]]
    } errtxt ]
    if { $err} {
	WarnWin [_ "Result '%s' does not exist: %s" $PCRPriv(resultdest) $errtxt] $PCRPriv(wid)
	return
    }

    if { ![ llength $res_info1] || ( [ lindex $res_info1 0] == "GiD_Result:")} {
	if { [ llength $res_info1]} {
	    WarnWin $res_info1 $PCRPriv(wid)
	} else {
	    WarnWin [_ "Result '%s' does not exist." $PCRPriv(resultdest)] $PCRPriv(wid)
	}
    }

    if { [ llength $res_info1] && ( [ lindex $res_info1 0] != "GiD_Result:")} {
	set resp [tk_dialogRAM $PCRPriv(wid).___pa_ww [_ "Warning"] \
		[_ "Do you really want to delete the Result '%s'?" $PCRPriv(resultdest)] \
		gidquestionhead 1 [_ "Yes"] [_ "No#C#I don't want to do that"]]
	switch $resp {
	    0 {
		if { $::PCRPriv(AllSteps)} {                                        
		    set all_steps [ GiD_Info postprocess get all_steps $analysis_dest]
		} else {
		    set all_steps $step_dest
		}
		foreach step_dest $all_steps {                   
		    catch {
		        GiD_Result delete [ list $PCRPriv(resultdest) $analysis_dest $step_dest]
		    }
		    GidUtils::SetWarnLine [_ "Result '%s' for step %s deleted" $PCRPriv(resultdest) $step_dest]
		}
	    }
	    1 {
		return
	    }
	}
    }
}

proc PostCreateResult { { w .gid.wPostCrearResultado}} {
    global PCRPriv GidPriv GIDDEFAULT

    # package require progressbar

    PCRIniVariables

    InitWindow $w [_ "Create Result in Tcl"] PostCreareResultWindowGeom PostCreateResult
    if { ![winfo exists $w] } return ;# windows disabled || usemorewindows == 0

    set PCRPriv(wid) $w

    set color_fondo [ CCGetRGB $w [ $w cget -background]]
    set color_sel [ CCColorSombra $color_fondo 65]
    set color_realzado [ CCColorActivo $color_fondo]

    set f [ttk::frame $w.f]

    set f0 [frame $f.f0 -border 2 -relief groove]
    label $f0.l0 -text [concat [_ "Analysis & step"] " 1"]
    ttk::button $f0.b0 -text [concat [_ "Select analysis & step"] " 1"] -command [ list PCRSelectAnalStep 1 $w $f0.b0]
    label $f0.l1 -text [concat [_ "Result"] " 1:"]
    ttk::button $f0.b1 -text [_ "Select result"] -command [ list PCRSelectResult 1 $w $f0.b1]
    grid $f0.l0 $f0.b0 -sticky ew
    grid $f0.l1 $f0.b1 -sticky ew
    grid columnconfigure $f0 "0 1" -weight 1

    set f2 [ttk::frame $f.f2]
    label $f2.l -text [_ "Operator"]:
    ttk::button $f2.b -text [_ "Select operator"] -command [ list PCRSelectOperacion $w $f2.b]
    grid $f2.l $f2.b -sticky ew
    grid configure $f2.b -sticky w
    grid columnconfigure $f2 "2" -weight 1

    set f3 [frame $f.f3 -border 2 -relief groove]
    label $f3.l0 -text [concat [_ "Analysis & step"] " 2"]
    ttk::button $f3.b0 -text [concat [_ "Select analysis & step"] " 2"] -command [ list PCRSelectAnalStep 2 $w $f3.b0]    
    label $f3.l1 -text [concat [_ "Result"] " 2:"]
    ttk::button $f3.b1 -text [_ "Select result"] -command [ list PCRSelectResult 2 $w $f3.b1]
    grid $f3.l0 $f3.b0 -sticky ew
    grid $f3.l1 $f3.b1 -sticky ew
    grid columnconfigure $f3 "0 1" -weight 1

    set fr [ttk::frame $f.fr]
    label $fr.l -text [_ "Result name"]:
    ttk::entry $fr.e -textvariable PCRPriv(resultdest) -width 20

    set filename "erase-cross.gif"
    ttk::button $fr.b -image [GetImage $filename] -command PCRDeleteResult
    ttk::checkbutton $fr.cbAll -text [_ "All steps"] -variable ::PCRPriv(AllSteps)
    set ::PCRPriv(AllSteps) 1

    #label $fr.l2 -text ""
    # ::progressbar::progressbar $fr.l2 -variable ::PCRPriv(var) \
    #         -width 200 -background $color_fondo
    ttk::progressbar $fr.l2 -variable ::PCRPriv(var) -length 200
    set ::PCRPriv(var) 0

    grid $fr.l $fr.e $fr.b -sticky news
    grid $fr.cbAll -sticky news -columnspan 3
    grid $fr.l2 -sticky news -columnspan 3
    grid columnconfigure $fr 1 -weight 1
    set PCRPriv(resultdest) [_ "New result"]

    set PCRPriv(chivato) $fr.l2


    grid $f0 -sticky news -padx 2 -pady 2
    grid $f2 -sticky news -padx 2 -pady 2
    grid $f3 -sticky news -padx 2 -pady 2
    grid $fr -sticky news -padx 2 -pady 2

    grid columnconfigure $f 0 -weight 1
    grid rowconfigure $f 4 -weight 1
    # marco de los botones
    frame $w.but -bd 0 -background $color_realzado

    ttk::button $w.but.apply -text [_ "Apply"] -command PCRApply -takefocus 1
    ttk::button $w.but.close -text [_ "Close"] -command PCRClose -takefocus 1

    SetWidgetsWidthFromText $w.but.apply $w.but.close

    grid $w.but.apply -sticky ews -padx 4 -pady 10
    grid $w.but.close -sticky ews -padx 4 -pady 10 -row 0 -column 1

    #enpaquetamos marcos
    grid $w.f -sticky news -padx 2 -pady 2
    grid $w.but -sticky wes
    if { $::tcl_version >= 8.5 } { grid anchor $w.but center }

    grid columnconfigure $w 0 -weight 1
    grid rowconfigure $w 0 -weight 1
    focus $w.but.apply
    bind $w <Return> "$w.but.apply invoke"
    bind $w <Alt-c> "$w.but.close invoke"
    bind $w <Escape> "$w.but.close invoke"

    #update idletasks
}

########################################################################

namespace eval PCreateResult {
    variable _wid
    variable _lst_variables   {}
    variable _widget_variables
    variable _txt_analysis
    variable _analysis
    variable _step
    variable _txt_result
    variable _result
    variable _txt_componente
    variable _componente
    variable _txt_variable
    variable _lst_variables
}

# package requirements
package require tablelist_tile

proc PCreateResult::IsList { var} {
    set idx [ string first \{ $var]
    set is_list 0
    # find if something open it:
    if { $idx != -1 } {
	if { $idx == 0} {
	    set is_list 1
	} elseif { [ string index $var [ expr $idx - 1]] != "\\"} {
	    set is_list 1
	}
    }
    if { $is_list} {
	# find if something close it:
	set idx [ string first \} $var $idx]
	if { $idx == -1} {
	    set is_list 0
	}
    }
    return $is_list
}

proc PCreateResult::ReplaceUnderineBySpace { lst { is_list 0}} {
    set txt_lst {}
    if { !$is_list} {
	regsub -all {_} $lst { } txt_lst
    } else {
	foreach elem $lst {
	    regsub -all {_} $elem { } txt
	    lappend txt_lst $txt
	}
    }
    return $txt_lst
}

proc PCreateResult::ReplaceSpaceByUnderine { lst { is_list 0}} {
    set txt_lst {}
    if { !$is_list} {
	regsub -all { } $lst {_} txt_lst
    } else {
	foreach elem $lst {
	    regsub -all { } $elem {_} txt
	    lappend txt_lst $txt
	}
    }
    return $txt_lst
}

proc PCreateResult::Apply { } {
    variable _wid
    #destroy $_wid
}

proc PCreateResult::Close { } {
    variable _wid
    destroy $_wid
}

proc PCreateResult::UpdateAnalysisList { w} {
    variable _txt_analysis
    variable _analysis

    $w configure -values [ ReplaceUnderineBySpace [ lsort -dictionary [ GiD_Info postprocess get all_analysis]] 1]
}

proc PCreateResult::UpdateListSteps { w} {
    variable _txt_analysis
    variable _analysis
    variable _step

    if { $_txt_analysis == "" } { return}
    set _analysis [ ReplaceSpaceByUnderine $_txt_analysis 0]

    if { $_analysis != ""} {
	set all_steps [ GiD_Info postprocess get all_steps $_analysis]
	$w configure -values $all_steps
	set current_analysis [ GiD_Info postprocess get cur_analysis]
	if { $current_analysis == $_analysis} {
	    set current_step [ GiD_Info postprocess get cur_step $current_analysis]
	} else {
	    set current_step [ lindex $all_steps 0]
	}
	set _step $current_step
    }
}


proc PCreateResult::Get_Results {} {
    set ret {}
    set all [ GiD_Info postprocess get cur_results_list "Result_Surface" ]
    foreach res $all {
	lappend ret $res
    }
    return $ret
}

proc PCreateResult::UpdateResultsList { w wc} {
    variable _analysis
    variable _step
    variable _txt_result
    variable _result

    if { $_analysis == ""} { return }
    if { $_step == ""} { return}

    set ret [ GiD_Info postprocess get results_list "Result_Surface"  $_analysis $_step]
    $w configure -values [ ReplaceUnderineBySpace $ret 1]
    if { [ lsearch $ret $_txt_result] == -1} {
	if { [ llength $ret] != 0} {
	    set _result [ lindex $ret 0]
	} else {
	    set _result ""
	}
	regsub -all {_} $_result { } _txt_result
	UpdateComponentsList $wc
    }
}

proc PCreateResult::UpdateComponentsList { w} {
    variable _txt_result
    variable _analysis
    variable _step
    variable _result
    variable _txt_componente

    if { $_analysis == ""} { return }
    if { $_step == ""} { return}

    if { $_txt_result == ""} { return }
    set _result [ ReplaceSpaceByUnderine $_txt_result 0]

    set ret [ GiD_Info postprocess get components_list "Result_Surface" \
		  $_result $_analysis $_step]
    $w configure -values [ ReplaceUnderineBySpace $ret 1]
    if { [ lsearch $ret $_txt_componente] == -1} {
	if { [ llength $ret] != 0} {
	    set _txt_componente [ lindex $ret 0]
	} else {
	    set _txt_componente ""
	}
	UpdateComponentName
    }
}

proc PCreateResult::UpdateComponentName { } {
    variable _txt_componente
    variable _componente

    if { $_txt_componente == ""} { return}
    set _componente [ ReplaceSpaceByUnderine $_txt_componente 0]

}

proc PCreateResult::ValidateName { name_var} {
    upvar $name_var var
    # valid variable names start by letter and follos letters, 
    # numbers and some of this characters    
    # _ # ! + - * / % & '
    # \#!+-*/%&'
    set valid 0
    if { [ regexp {^([[:alpha:]][\w\#!+\-*/%&\']+)$} $var dum pepe]} {
	set valid 1
    }
    return $valid
}

proc PCreateResult::ValidateVariableAndClose { w old_name} {
    variable _txt_variable

    if { ![ ValidateName PCreateResult::_txt_variable]} {
	WarnWin [ concat [_ "Invalid variable name. \
	    \nShould begin with a letter and followed\
	    \nby a digit, letter or"] " \# ! + - * / % & '"] $w
	return
    } else {
	
    }

    if { $old_name != ""} {
	# quitamos $old_name
    }

    # add new variable to list
    destroy $w
}

proc PCreateResult::EditVariableWindow { { w .gid.wPCreateResultEditVariable} { name ""}} {
    variable _txt_analysis
    variable _analysis
    variable _step
    variable _txt_result
    variable _result
    variable _txt_componente
    variable _componente
    variable _txt_variable
    variable _lst_variables

    if { [ winfo exists $w]} {
	raise $w
	return
    }

    InitWindow $w [_ "Add variable"] PostEditVariableWindowGeom \
	PCreateResult::EditVariableWindow
    if { ![ winfo exists $w]} return

    ttk::frame $w.f

    if { $name == ""} {
	set _txt_variable [_ "No_Name"]_[ llength _lst_variables]
    } else {
	set _txt_variable $name
    }

    label $w.f.lv -text [_ "Variable name"]:
    ttk::entry $w.f.ev -textvariable PCreateResult::_txt_variable -width 30

    label $w.f.la -text [_ "Analysis"]:
    ComboBox $w.f.ma -textvariable PCreateResult::_txt_analysis -editable 0 -width 30 -bd 1 \
	-modifycmd [ list PCreateResult::UpdateListSteps $w.f.fp.mp]

    label $w.f.lp -text [_ "Step"]:
    ComboBox $w.f.mp -textvariable PCreateResult::_step -editable 0 -width 8 -bd 1 \
	-modifycmd [ list PCreateResult::UpdateResultsList $w.f.m1 $w.f.m2]

    label $w.f.l1 -text [_ "Result"]:
    ComboBox $w.f.m1 -textvariable PCreateResult::_txt_result -editable 0 -width 30 -bd 1 \
	-modifycmd [ list PCreateResult::UpdateComponentsList $w.f.m2]

    label $w.f.l2 -text [_ "Component"]:
    ComboBox $w.f.m2 -textvariable PCreateResult::_txt_componente -editable 0 -width 30 -bd 1 \
	-modifycmd PCreateResult::UpdateComponentName

    # rellenamos los widget con valores
    UpdateAnalysisList $w.f.ma

    set _analysis [ GiD_Info postprocess get cur_analysis]
    regsub -all {_} $_analysis { } _txt_analysis
	
    UpdateListSteps $w.f.mp
    $w.f.m1 configure -values [ PCreateResult::Get_Results]
    UpdateResultsList $w.f.m1 $w.f.m2

    # name variable
    grid $w.f.lv $w.f.ev -sticky e -padx 2 -pady 1
    grid configure $w.f.ev -sticky ew
    # analysis
    grid $w.f.la $w.f.ma -sticky e -padx 2 -pady 1
    grid configure $w.f.ma -sticky ew
    # step
    grid $w.f.lp $w.f.mp -sticky e -padx 2 -pady 1
    grid configure $w.f.mp -sticky w
    # result
    grid $w.f.l1 $w.f.m1 -sticky e -padx 2 -pady 1
    grid configure $w.f.m1 -sticky ew
    # componente
    grid $w.f.l2 $w.f.m2 -sticky e -padx 2 -pady 1
    grid configure $w.f.m2 -sticky ew

    grid rowconfigure $w.f 0 -weight 1
    grid rowconfigure $w.f 1 -weight 1
    grid rowconfigure $w.f 2 -weight 1
    grid rowconfigure $w.f 3 -weight 1
    grid rowconfigure $w.f 4 -weight 1
    grid columnconfigure $w.f 0 -weight 1
    grid columnconfigure $w.f 1 -weight 1

    set def_back [ $w cget -background]
    frame $w.buts -bg [ CCColorActivo $def_back]

    ttk::button $w.buts.apply -text [_ "Create"] -command \
	[ list PCreateResult::ValidateVariableAndClose $w $name]
    ttk::button $w.buts.close -text [_ "Cancel"] -command [ list destroy $w]

    SetWidgetsWidthFromText $w.buts.apply $w.buts.close

    grid $w.buts.apply $w.buts.close -sticky ew -padx 5 -pady 6

    # parte superior
    grid $w.f -sticky nsew
    # parte inferior
    grid $w.buts -sticky sew
    if { $::tcl_version >= 8.5 } { grid anchor $w.buts center }
    grid columnconfigure $w 0 -weight 1
    grid rowconfigure $w 0 -weight 1

    bind $w <Escape> "$w.buts.close invoke"


    tkwait window $w

    set ret_val ""
    if { ( $_txt_analysis != "") && ( $_step != "") && \
	     ( $_txt_result != "") && ( $_txt_componente != "")} {
	set ret_val [ list $_txt_variable Scalar Result \
		          $_txt_analysis $_step $_txt_result $_txt_componente]
    }
    return $ret_val
}

proc PCreateResult::AnyadeVariable { lista} {
    set w $lista.def

    set info_var [ EditVariableWindow $w]
    if { [ winfo exists $lista]} {
	if { $info_var != ""} {
	    $lista insert end $info_var
	}
    }
}

proc PCreateResult::QuitaVariable { lista} {
    WarnWin "Aade variable"
}

proc PCreateResult::CreaListaVariables { f} {
    variable _widget_variables

    set _widget_variables $f.list

    ttk::frame $f
    ttk::scrollbar $f.xscroll -orient horizontal -command [ list $f.list xview]
    ttk::scrollbar $f.yscroll -orient vertical -command [ list $f.list yview]
    listbox $f.list -height 5 -width 20 \
	-xscroll [ list $f.xscroll set] \
	-yscroll [ list $f.yscroll set] \
	-selectmode browse -relief sunken -bd 1
	
    grid $f.list -sticky ewns
    grid $f.xscroll -sticky ew
    grid $f.yscroll -sticky ns -rowspan 2 -row 0 -column 1
    grid rowconfigure $f 0 -weight 1
    grid columnconfigure $f 0 -weight 1
    return $f
}

proc PCreateResult::EventosWidgetVariables { f add_cmd del_cmd edit_cmd} {
    variable _widget_variables

    bind $_widget_variables <Double-Button-1> $edit_cmd
    bind $_widget_variables <Return> $edit_cmd
    bind $_widget_variables <Delete> $del_cmd
    bind $_widget_variables <Insert> $add_cmd
    bind $_widget_variables <plus> $add_cmd
    bind $_widget_variables <minus> $del_cmd
    #bind $_widget <KeyPress> "WarnWin %K"
}

proc PCreateResult::CreaTablaVariables { f} {
    global GidPriv
    variable _widget_variables

    ttk::frame $f

    #  table name
    set tbl $f.tbl
    set _widget_variables $f.tbl

    ttk::scrollbar $f.xscroll -orient horizontal -command [ list $tbl xview]
    ttk::scrollbar $f.yscroll -orient vertical -command [ list $tbl yview]
    tablelist::tablelist $tbl -exportselection 0 \
	-columns [ list \
		       10 [_ "Name"] left \
		       5 [_ "Type"] left \
		       5 [_ "Class"] left \
		       15 [_ "Analysis"] left \
		       4 [_ "Step"] left \
		       15 [_ "Result"] left \
		       15 [_ "Component"] left \
		       ] \
	-selectbackground black \
	-selectforeground white \
	-background $GidPriv(Color,BackgroundListbox) \
	-stretch all \
	-selectmode single \
	-labelborderwidth 1 \
	-labelcommand tablelist::sortByColumn \
	-labelpady 0 \
	-showarrow 1 \
	-showseparators 0 \
	-xscrollcommand [ list $f.xscroll set] \
	-yscrollcommand [ list $f.yscroll set] \
	-resizablecolumns 1 \
	-stripebackground [ CCColorSombra $GidPriv(Color,BackgroundListbox)] \
	-activestyle frame \
	-selecttype row
	#-highlightthickness 0
	
    # for { set i 0} { $i < 7} { incr i} {
    #         $tbl columnconfigure $i -selectbackground $GidPriv(Color,BackgroundListbox) \
    #             -selectforeground black
    # }

    grid $f.tbl -sticky ewns
    grid $f.xscroll -sticky ew
    grid $f.yscroll -sticky ns -rowspan 2 -row 0 -column 1
    grid rowconfigure $f 0 -weight 1
    grid columnconfigure $f 0 -weight 1
    return $f
}

proc PCreateResult::Crear { { w .gid.wPostCrearResultado2}} {
    variable _wid
    variable _widget_variables

    PCRIniVariables

    InitWindow $w [_ "Create Result in Tcl"] PostCreateResultWindowGeom PCreateResult::Crear
    if { ![winfo exists $w] } return ;# windows disabled || usemorewindows == 0

    set _wid $w

    set color_fondo [ CCGetRGB $w [ $w cget -background]]
    set color_sel [ CCColorSombra $color_fondo 65]
    set color_realzado [ CCColorActivo $color_fondo]

    ttk::frame $w.ventana
    # -relief ridge

    # lista de variables
    #TitleFrame $w.ventana.lv -text [_ "Variable list"]
    #set f [ $w.ventana.lv getframe]
    set f [ttk::labelframe $w.ventana.lv -text [_ "Variable list"]]

    #CreaListaVariables $f.lb
    CreaTablaVariables $f.lb

    # control de lista de variables: mas menos y edit
    ttk::frame $f.bot
    ttk::button $f.bot.mas -image [ GetImage Mas16.gif] \
	-command [ list PCreateResult::AnyadeVariable $_widget_variables]
    ttk::button $f.bot.edit -text [_ "Edit"] \
	-command [ list WarnWin "edita variable"]
    ttk::button $f.bot.menos -image [ GetImage Menos16.gif] \
	-command [ list PCreateResult::QuitaVariable $_widget_variables]

    grid $f.bot.mas $f.bot.edit $f.bot.menos -sticky w -padx 4 -pady 2
    grid rowconfigure $f.bot 0 -weight 1

    EventosWidgetVariables $f.lb \
	[ $f.bot.mas cget -command] [ $f.bot.menos cget -command] [ $f.bot.edit cget -command]

    grid $f.lb -sticky nsew -padx 4 -pady 2
    grid $f.bot -sticky ew
    grid rowconfigure $f 0 -weight 1
    grid columnconfigure $f 0 -weight 1

    # text with formulae
    # TitleFrame $w.ventana.tf -text [_ "Formula"]
    # set f [ $w.ventana.tf getframe]
    set f [ttk::labelframe $w.ventana.tf -text [_ "Formula"]]

    ttk::frame $f.tt
    ttk::scrollbar $f.tt.xscroll -orient horizontal -command "$f.tt.text xview"
    ttk::scrollbar $f.tt.yscroll -orient vertical -command "$f.tt.text yview"
    text $f.tt.text -width 20 -height 5 \
	-xscroll "$f.tt.xscroll set" \
	-yscroll "$f.tt.yscroll set" \
	-relief sunken -bd 1

    grid $f.tt.text -sticky ewns
    grid $f.tt.xscroll -sticky ew
    grid $f.tt.yscroll -sticky ns -rowspan 2 -row 0 -column 1
    grid rowconfigure $f.tt 0 -weight 1
    grid columnconfigure $f.tt 0 -weight 1

    grid $f.tt -sticky news -padx 4 -pady 2
    grid rowconfigure $f 0 -weight 1
    grid columnconfigure $f 0 -weight 1


    grid $w.ventana.lv -sticky news -padx 2
    grid $w.ventana.tf -sticky news -padx 2
    grid rowconfigure $w.ventana 0 -weight 1
    grid rowconfigure $w.ventana 1 -weight 1
    grid columnconfigure $w.ventana 0 -weight 1

    # apply y close
    # marco de los botones
    frame $w.but -bd 0 -background $color_realzado

    ttk::button $w.but.apply -text [_ "Apply"] -command PCreateResult::Apply \
	-takefocus 1
    ttk::button $w.but.close -text [_ "Close"] -command PCreateResult::Close \
	-takefocus 1

    # SetWidgetsWidthFromText $w.but.apply $w.but.close

    grid $w.but.apply -sticky ews -padx 4 -pady 10
    grid $w.but.close -sticky ews -padx 4 -pady 10 -row 0 -column 1

    #enpaquetamos marcos
    grid $w.ventana -sticky news
    ttk::frame $w.sep -height 4

    #-pady 3 -padx 4
    grid $w.sep -sticky we
    grid $w.but -sticky wes
    if { $::tcl_version >= 8.5 } { grid anchor $w.but center }

    grid columnconfigure $w 0 -weight 1
    grid rowconfigure $w 0 -weight 1
    bind $w <Return> "+$w.but.apply configure -relief sunken; \
	    update; \
	    $w.but.apply invoke; \
	    $w.but.apply configure -relief raised"
    bind $w <Escape> "focus $w.but.close; \
	    $w.but.close configure -relief sunken; \
	    $w.but.close invoke"

    update idletasks
}

namespace eval ::PostOperate2Scalars {
}

#Fills the values of the scalars list
proc ::PostOperate2Scalars::Get_Results_Nodales_Escalares { } {
    set ret {}
    set all [ GiD_Info postprocess get cur_results_list "Result_Surface" ]
    foreach res $all {
	if { [ llength [ GiD_Info postprocess get cur_components_list $res ]] == 1} {
	    lappend ret $res
	}
    }
    return $ret
}
#Switch between spaces and "_"
proc ::PostOperate2Scalars::ReplaceUnderineBySpace { lst} {
    set txt_lst {}
    foreach elem $lst {
	regsub -all {_} $elem { } txt
	lappend txt_lst $txt
    }
    return $txt_lst
}
proc ::PostOperate2Scalars::ReplaceSpaceByUnderine { lst} {
    set txt_lst {}
    foreach elem $lst {
	regsub -all { } $elem {_} txt
	lappend txt_lst $txt
    }
    return $txt_lst
}
#Vector's creation code
proc ::PostOperate2Scalars::ConvertOperationToScalars { w} {
    ::PostOperate2Scalars::UpdateScalarsList $w all
    set i 1
    while { $i <= $::PostOperate2Scalars::Number_vectors } {
	set scalar_name "::PostOperate2Scalars::ScalarToConvert$i"
	if { [subst $$scalar_name] == ""} {
	    WarnWin [_ "Please, select a valid scalar result for Scalar result $i"] $w
	    return
	}
	incr i
    }
    regsub -all { } $::PostOperate2Scalars::TxtVectorName {_} ::PostOperate2Scalars::VectorName
    if { $::PostOperate2Scalars::VectorName == ""} {
	WarnWin [_ "Please, enter a name for the vector result"] $w
	return
    }
    set current_analysis $::PostOperate2Scalars::Analysis
    set all_steps $::PostOperate2Scalars::Step
    if { $::PostOperate2Scalars::AllSteps} {
	set all_steps [ GiD_Info postprocess get all_steps $::PostOperate2Scalars::Analysis]
    }
    set texto_output ""
    foreach current_step $all_steps {
	set cabecera [ list Result $::PostOperate2Scalars::VectorName $current_analysis $current_step "Vector" "OnNodes"]
	set res_output {}
	lappend res_output $cabecera
	
	set dict_values ""
	set all_exist 1
	#We fill a dictionary with the values of all the vectors for this step
	foreach i [list 1 2 3] {
	    if {$i > $::PostOperate2Scalars::Number_vectors} break
	    set scalar_name "::PostOperate2Scalars::ScalarToConvert$i"
	    set res_info ""
	    set err [ catch {
		set res_info [ GiD_Result get [ list [subst $$scalar_name] $current_analysis $current_step]]
	    } ]
	    if { $err} {
		set all_exist 0
		break
	    }
	    foreach input [ lrange $res_info 1 end] {
		if { ( [ llength $input] == 2) && ( [ scan $input "%d %g" idx_nodo scalar] == 2) } {
		    foreach coordinate [list x y z] {
		        set vector_name "::PostOperate2Scalars::VectorToUse$i\($coordinate\)"
		        if {[dict exists $dict_values $idx_nodo $coordinate] == "0"} {
		            set old_value 0
		        } else {
		            set old_value [dict get $dict_values $idx_nodo $coordinate]
		        }
		        set new_value [expr ($scalar * [subst $$vector_name])]
		        dict set dict_values $idx_nodo $coordinate [ expr ($old_value + $new_value)]
		    }
		}
	    }
	}
	
	#If some vector is missing for this step, we evaluate the next one
	if {$all_exist == "1"} {
	    #We convert our dictionary to the proper GiD format
	    foreach nodeid [dict keys $dict_values] {
		set input_list [list $nodeid]
		foreach position [list x y z] {
		    lappend input_list [dict get $dict_values $nodeid $position]
		}
		lappend res_output $input_list
	    }
	    
	    eval GiD_Result create $res_output
	    
	    #We update the output text and we delete the ancient value if needed
	    append texto_output [_ "\nVector result '%s'  ( '%s', step '%g') created   \n from" \
		                      $::PostOperate2Scalars::TxtVectorName [ ::PostOperate2Scalars::ReplaceUnderineBySpace $current_analysis] $current_step]
	    set i 1
	    while { $i <= $::PostOperate2Scalars::Number_vectors } {
		set scalar_name "::PostOperate2Scalars::TxtScalarToConvert$i"
		
		if {$::PostOperate2Scalars::Delete_scalars == "1"} {
		    set analysis_name ""
		    regsub -all {_} $current_analysis { } analysis_name
		    set erasing_list [list [subst $$scalar_name] $analysis_name $current_step]
		    GiD_Result delete $erasing_list
		}
		set vector_x "::PostOperate2Scalars::VectorToUse$i\(x\)"
		set vector_y "::PostOperate2Scalars::VectorToUse$i\(y\)"
		set vector_z "::PostOperate2Scalars::VectorToUse$i\(z\)"
		if {$::PostOperate2Scalars::Number_vectors != "1" && $i == $::PostOperate2Scalars::Number_vectors} {
		    append texto_output [_ " and"]
		}
		append texto_output [_ " scalar '%s' using vector \( %s, %s, %s\)" \
		                          [subst $$scalar_name] [subst $$vector_x] [subst $$vector_y] [subst $$vector_z]]
		incr i
	    }
	}
    }
    if {$::PostOperate2Scalars::Delete_scalars == "1"} {
	append texto_output [_ "\n\nOriginal scalar results have been deleted"]
    }
    WarnWinText $texto_output
}
#Fills the list of analysis
proc ::PostOperate2Scalars::ActualizaListaAnalysis { w} {
    $w configure -values [ ::PostOperate2Scalars::ReplaceUnderineBySpace [lsort -dictionary [GiD_Info postprocess get all_analysis]]]
}
#Dependencies of the analysis selection
proc ::PostOperate2Scalars::UpdateListSteps { w w_local} {
    if { "$::PostOperate2Scalars::TxtAnalysis" == "" } { return }
    set ::PostOperate2Scalars::Analysis [ ::PostOperate2Scalars::ReplaceSpaceByUnderine [ list $::PostOperate2Scalars::TxtAnalysis]]
    if { "$::PostOperate2Scalars::Analysis" != ""} {
	set all_steps [ GiD_Info postprocess get all_steps $::PostOperate2Scalars::Analysis]
	$w_local configure -values $all_steps
	set current_analysis [ GiD_Info postprocess get cur_analysis]
	if { $current_analysis == $::PostOperate2Scalars::Analysis} {
	    set current_step [ GiD_Info postprocess get cur_step $current_analysis]
	} else {
	    set current_step [ lindex $all_steps 0]
	}
	set ::PostOperate2Scalars::Step $current_step
	
	if {$::PostOperate2Scalars::Scalars_initialized == "1" } {
	    ::PostOperate2Scalars::UpdateScalarsList $w all
	}
    }
}
#Update the values of the scalar lists
proc ::PostOperate2Scalars::UpdateScalarsList { w index} {
    if {$index == "all"} {
	set index_list [list 1 2 3]
	set windows_list [list $w.f.m11 $w.f.m12 $w.f.m13]
    } else {
	set index_list [list $index]
	set windows_list [list $w]
    }
    
    if { $::PostOperate2Scalars::Analysis != ""} {
	if { $::PostOperate2Scalars::Step != ""} {
	    set ret {}
	    set all [ GiD_Info postprocess get results_list "Result_Surface"  $::PostOperate2Scalars::Analysis $::PostOperate2Scalars::Step]
	    foreach res $all {
		if { [ llength [ GiD_Info postprocess get components_list "Result_Surface"  $res $::PostOperate2Scalars::Analysis $::PostOperate2Scalars::Step]] == 1} {
		    lappend ret $res
		}
	    }
	    foreach window $windows_list {
		$window configure -values [ ::PostOperate2Scalars::ReplaceUnderineBySpace $ret]
	    }
	    foreach position $index_list {
		set var_name "::PostOperate2Scalars::ScalarToConvert$position"
		set var_txt_name "::PostOperate2Scalars::TxtScalarToConvert$position"
		if { [ lsearch $ret [subst $$var_name]] == -1} {
		    if { [ llength $ret] != 0} {
		        set $var_name [lindex $ret 0]
		    } else {
		        set $var_name ""
		    }
		    set $var_name [::PostOperate2Scalars::ReplaceSpaceByUnderine [subst $$var_txt_name]]
		    
		    if {$position == "1"} {
		        ::PostOperate2Scalars::UpdateVectorName
		    }
		}
	    }
	}
    }
}
#Dependencies of the all steps check button
proc ::PostOperate2Scalars::EnableDisableLista { w} {
    if { $::PostOperate2Scalars::AllSteps} {
	$w configure -state disabled
    } else {
	$w configure -state normal
    }
}

#Define the dependencies of the dimension selector
proc ::PostOperate2Scalars::Updatedimension { w } {
    foreach i [list 3 2] {
	if { "$::PostOperate2Scalars::Number_vectors" >= "$i"} {
	    $w.f.m1$i configure -state normal
	    if {"$::PostOperate2Scalars::Conversion_mode" == [_ "User_defined"]} {
		foreach j [list x y z] { $w.f.fv$i.e$j configure -state normal }
	    }
	} else {
	    $w.f.m1$i configure -state disabled
	    foreach j [list x y z] { $w.f.fv$i.e$j configure -state disable }
	}
    }
    ::PostOperate2Scalars::UpdateVectorName
}

#Define the dependencies of the Conversion mode selector
proc ::PostOperate2Scalars::Updatestatevectors { w } {
    if { "$::PostOperate2Scalars::Conversion_mode" == [_ "User_defined"] } {
	foreach i [list 1 2 3] {
	    if {"$::PostOperate2Scalars::Number_vectors" >= "$i"} {
		foreach j [list x y z] {
		    $w.f.fv$i.e$j configure -state normal
		}
	    }
	}
    } else {
	::PostOperate2Scalars::Initializevectors
	foreach i [list 1 2 3] {
	    foreach j [list x y z] {
		$w.f.fv$i.e$j configure -state disabled
	    }
	}
    }
}

#Gives to the three vectors the main axes values
proc ::PostOperate2Scalars::Initializevectors {} {
    set ::PostOperate2Scalars::VectorToUse1(x) 1.0
    set ::PostOperate2Scalars::VectorToUse1(y) 0.0
    set ::PostOperate2Scalars::VectorToUse1(z) 0.0
    set ::PostOperate2Scalars::VectorToUse2(x) 0.0
    set ::PostOperate2Scalars::VectorToUse2(y) 1.0
    set ::PostOperate2Scalars::VectorToUse2(z) 0.0
    set ::PostOperate2Scalars::VectorToUse3(x) 0.0
    set ::PostOperate2Scalars::VectorToUse3(y) 0.0
    set ::PostOperate2Scalars::VectorToUse3(z) 1.0
}

#Initialize the values related with the three scalars
proc ::PostOperate2Scalars::Initializscalars {} {
    set ::PostOperate2Scalars::ScalarToConvert1 ""
    set ::PostOperate2Scalars::ScalarToConvert2 ""
    set ::PostOperate2Scalars::ScalarToConvert3 ""
}

#We suggest a name while dealing with a single vector
proc ::PostOperate2Scalars::UpdateVectorName {} {
    if {$::PostOperate2Scalars::Number_vectors == "1"} {
	if { "$::PostOperate2Scalars::TxtScalarToConvert1" == "" } {
	    set ::PostOperate2Scalars::ScalarToConvert1 ""
	    set ::PostOperate2Scalars::VectorName ""
	    set ::PostOperate2Scalars::TxtVectorName ""
	    return
	}
	set ::PostOperate2Scalars::ScalarToConvert1 [ ::PostOperate2Scalars::ReplaceSpaceByUnderine [ list $::PostOperate2Scalars::TxtScalarToConvert1]]
	if { "$::PostOperate2Scalars::ScalarToConvert1" != ""} {
	    set ::PostOperate2Scalars::VectorName ${::PostOperate2Scalars::ScalarToConvert1}_vector
	} else {
	    set ::PostOperate2Scalars::VectorName ""
	}
	regsub -all {_} $::PostOperate2Scalars::VectorName { } ::PostOperate2Scalars::TxtVectorName
    }
}

#Build the window of the macro
proc ::PostOperate2Scalars::Opera2Escalares {  { w .gid.wOpera2Escalares}} {
    set ::PostOperate2Scalars::Analysis ""
    set ::PostOperate2Scalars::Step ""
    set ::PostOperate2Scalars::AllSteps 0
    set ::PostOperate2Scalars::TxtVectorName [_ "Vector1"]
    set ::PostOperate2Scalars::Number_vectors "3"
    ::PostOperate2Scalars::Initializscalars
    ::PostOperate2Scalars::Initializevectors
    set ::PostOperate2Scalars::Scalars_initialized 0
    set ::PostOperate2Scalars::Conversion_mode [_ "Main_axes"]
    set ::PostOperate2Scalars::Delete_scalars 0
    InitWindow $w [_ "Nodal scalars 2 vector"] PostOpera2EscalaresWindowGeom ::PostOperate2Scalars::Opera2Escalares
    frame $w.f -bd 0
    #Analysis
    label $w.f.la -text [_ "Analysis"]:
    ComboBox $w.f.ma -textvariable ::PostOperate2Scalars::TxtAnalysis -editable 0 -width 30 -bd 1  -modifycmd "::PostOperate2Scalars::UpdateListSteps $w $w.f.fp.mp"
    ::PostOperate2Scalars::ActualizaListaAnalysis $w.f.ma
    #Step
    label $w.f.lp -text [_ "Step"]:
    frame $w.f.fp -bd 0
    ComboBox $w.f.fp.mp -textvariable ::PostOperate2Scalars::Step -editable 0 -width 8 -bd 1  -modifycmd "::PostOperate2Scalars::UpdateScalarsList $w all"
    ttk::checkbutton $w.f.fp.cb -text [_ "all steps"] -variable ::PostOperate2Scalars::AllSteps  -command "::PostOperate2Scalars::EnableDisableLista $w.f.fp.mp"
    grid $w.f.fp.mp $w.f.fp.cb -sticky w
    grid rowconfigure $w.f.fp 0 -weight 1
    grid columnconfigure $w.f.fp 0 -weight 1
    grid columnconfigure $w.f.fp 1 -weight 1
    #Vector Name selection
    ::PostOperate2Scalars::UpdateVectorName
    label $w.f.l2 -text [_ "To vector result"]:
    entry $w.f.e2 -textvariable ::PostOperate2Scalars::TxtVectorName -width 30
    #Conversion mode
    label $w.f.lc1 -text [_ "Conversion mode"]:
    frame $w.f.fc -bd 0
    label $w.f.fc.lc2 -text [_ "Number of vectors"]:
    ComboBox $w.f.fc.mc1 -textvariable ::PostOperate2Scalars::Conversion_mode -editable 0 -width 12 -bd 1 -values [list [_ "Main_axes"] [_ "User_defined"]] -modifycmd "::PostOperate2Scalars::Updatestatevectors $w"
    ComboBox $w.f.fc.mc2 -textvariable ::PostOperate2Scalars::Number_vectors -editable 0 -width 4 -bd 1 -values [list 1 2 3] -modifycmd "::PostOperate2Scalars::Updatedimension $w"
    grid $w.f.fc.mc1 $w.f.fc.lc2 $w.f.fc.mc2 -sticky e
    grid rowconfigure $w.f.fc 0 -weight 1
    grid columnconfigure $w.f.fc 0 -weight 1
    grid columnconfigure $w.f.fc 1 -weight 1
    grid columnconfigure $w.f.fc 2 -weight 1

    set ::PostOperate2Scalars::Analysis [ GiD_Info postprocess get cur_analysis]
    regsub -all {_} $::PostOperate2Scalars::Analysis { } ::PostOperate2Scalars::TxtAnalysis
    ::PostOperate2Scalars::UpdateListSteps $w $w.f.fp.mp
    
    #We define the graphical environment for the three scalars
    foreach index [list 1 2 3] {
	label [subst $w.f.l1$index] -text [_ "Scalar result $index"]:
	if {$index == "1"} {
	    ComboBox [subst $w.f.m1$index] -textvariable "::PostOperate2Scalars::TxtScalarToConvert$index" -editable 0 -width 30 -bd 1 -modifycmd ::PostOperate2Scalars::UpdateVectorName
	} else {
	    ComboBox [subst $w.f.m1$index] -textvariable "::PostOperate2Scalars::TxtScalarToConvert$index" -editable 0 -width 30 -bd 1
	}
	[subst $w.f.m1$index] configure -values [ ::PostOperate2Scalars::Get_Results_Nodales_Escalares]

	::PostOperate2Scalars::UpdateScalarsList [subst $w.f.m1$index] $index
	
	label [subst $w.f.l3$index] -text [_ "Using Vector $index"]:
	frame [subst $w.f.fv$index] -bd 0
	label [subst $w.f.fv$index.lx] -text "x:"
	entry [subst $w.f.fv$index.ex] -textvariable "::PostOperate2Scalars::VectorToUse$index\(x\)" -width 5 -state disabled
	label [subst $w.f.fv$index.ly] -text "y:"
	entry [subst $w.f.fv$index.ey] -textvariable "::PostOperate2Scalars::VectorToUse$index\(y\)" -width 5 -state disabled
	label [subst $w.f.fv$index.lz] -text "z:"
	entry [subst $w.f.fv$index.ez] -textvariable "::PostOperate2Scalars::VectorToUse$index\(z\)" -width 5 -state disabled
	
	grid [subst $w.f.fv$index.lx] [subst $w.f.fv$index.ex] [subst $w.f.fv$index.ly] [subst $w.f.fv$index.ey] [subst $w.f.fv$index.lz] \
	    [subst $w.f.fv$index.ez] -sticky e -padx 2 -pady 1
	grid configure [subst $w.f.fv$index.ex] -sticky w
	grid configure [subst $w.f.fv$index.ey] -sticky w
	grid configure [subst $w.f.fv$index.ez] -sticky w
	grid rowconfigure [subst $w.f.fv$index] 0 -weight 1
	grid columnconfigure [subst $w.f.fv$index] 1 -weight 1
	grid columnconfigure [subst $w.f.fv$index] 3 -weight 1
	grid columnconfigure [subst $w.f.fv$index] 5 -weight 1
    }
    
    # Scalars deletion option
    label $w.f.ldel -text [_ "Delete original results"]:
    ttk::checkbutton $w.f.cbdel -variable ::PostOperate2Scalars::Delete_scalars

    set ::PostOperate2Scalars::Scalars_initialized 1
    ::PostOperate2Scalars::UpdateScalarsList $w all

    # analysis
    grid $w.f.la $w.f.ma -sticky e -padx 2 -pady 1
    grid configure $w.f.ma -sticky ew
    # step
    grid $w.f.lp $w.f.fp -sticky e -padx 2 -pady 1
    grid configure $w.f.fp -sticky w
    # vector's name
    grid $w.f.l2 $w.f.e2 -sticky e -padx 2 -pady 1
    grid configure $w.f.e2 -sticky ew
    # conversion
    grid $w.f.lc1 $w.f.fc -sticky e -padx 2 -pady 1
    grid configure $w.f.fc -sticky w
    #We display all the vectors
    foreach index [list 1 2 3] {
	# Scalar result
	grid [subst $w.f.l1$index] [subst $w.f.m1$index] -sticky e -padx 2 -pady 1
	grid configure [subst $w.f.m1$index] -sticky ew
	# Vector
	grid [subst $w.f.l3$index] [subst $w.f.fv$index] -sticky e -padx 2 -pady 1
	grid configure [subst $w.f.fv$index] -sticky w
    }
    # Scalars deletion
    grid $w.f.ldel $w.f.cbdel -sticky e -padx 2 -pady 1
    grid configure $w.f.cbdel -sticky w
    
    # General displaying
    grid rowconfigure $w.f 0 -weight 2
    grid rowconfigure $w.f 1 -weight 2
    grid rowconfigure $w.f 2 -weight 2
    grid rowconfigure $w.f 2 -weight 2
    grid rowconfigure $w.f 2 -weight 2
    grid rowconfigure $w.f 3 -weight 2
    grid rowconfigure $w.f 3 -weight 2
    grid rowconfigure $w.f 3 -weight 2
    
    grid columnconfigure $w.f 0 -weight 1
    grid columnconfigure $w.f 1 -weight 1
    
    set def_back [ $w cget -background]
    frame $w.buts -bg [ CCColorActivo $def_back]
    ttk::button $w.buts.apply -text [_ "Create"] -command "::PostOperate2Scalars::ConvertOperationToScalars $w"
    ttk::button $w.buts.close -text [_ "Close"] -command "destroy $w"
    SetWidgetsWidthFromText $w.buts.apply $w.buts.close
    
    grid $w.buts.apply $w.buts.close -sticky ew -padx 5 -pady 6
    # Upper part
    grid $w.f -sticky nsew
    # Lower part
    grid $w.buts -sticky sew
    if { $::tcl_version >= 8.5 } { grid anchor $w.buts center }
    grid columnconfigure $w 0 -weight 1
    grid rowconfigure $w 0 -weight 1
    
    bind $w <Escape> "$w.buts.close invoke"
}
