

namespace eval RetrievePT {
    variable w
    variable progressbar
    variable progresslabel
    variable ftpid ""
    variable FileSize
    variable LastTimeMiliSec 0
    variable modulesinfo
    variable images
    variable lists
    variable currenttab

    variable server gid.cimne.com
    variable relativedir [file join pub gidmodules]
    variable localdir [file join $::GIDDEFAULT problemtypes]
    variable blocksize 4096 ;#default value 4096
    variable contcallbacks 0
}

proc RetrievePT::Close {} {
    variable w
    variable ftpid
    variable progresslabel

    if { [info exists ftpid] && $ftpid != "" } {
	set progresslabel [_ "Closing"]...
	update
	ftp::Close $ftpid
	set ftpid ""
    }
    destroy $w
}


proc RetrievePT::Callback { bytes } {
    variable FileSize
    variable progressbar
    variable progresslabel
    variable LastTimeMiliSec
    variable blocksize
    variable contcallbacks
    if { $contcallbacks == [expr int($FileSize/$blocksize/15)] } {
	set contcallbacks 0
	set percent [expr 10+90*$bytes/double($FileSize)]
	set progressbar $percent
	
	set TimeMiliSec [clock clicks -milliseconds]
	set speed [expr $bytes/1024.0/($TimeMiliSec-$LastTimeMiliSec)*1000.0]
	
	set progresslabel [format "$bytes/$FileSize (%.3g Kb/s)" $speed]
	update idletasks
    } else {
	incr contcallbacks
    }
}

proc RetrievePT::Connect {} {
    variable w
    variable ftpid
    variable server
    variable progressbar
    variable progresslabel
    variable relativedir
    variable InitialPercentage
    variable blocksize

    set progressbar 0
    $w conf -cursor watch
    update

    if { ![info exists ftpid] } {
	set ftpid ""
    }

    if { $ftpid != "" && [catch {
	ftp::Pwd $ftpid
    }] } {
	set ftpid ""
    }

    if { $ftpid == "" } {
	set ftpid [ftp::Open $server ftp gidretriever@  -mode passive \
		        -progress ::RetrievePT::Callback -timeout 30 -blocksize $blocksize]

	if { $ftpid == -1 } {
	    WarnWin [_ "Could not connect to server"]
	    set progressbar 0
	    set progresslabel ""
	    $w conf -cursor ""
	    return -1
	}
	ftp::Cd $ftpid $relativedir
    }
    set progresslabel [_ "Connected"]
    update
    return 0
}

proc RetrievePT::Retrievelist {} {
    variable w
    variable ftpid
    variable progressbar
    variable progresslabel
    variable FileSize
    variable modulesinfo
    variable modulesarray
    variable modules

    if { [Connect] } { return }
    set FileSize [ftp::FileSize $ftpid modulesinfo]

    set progresslabel 0/$FileSize
    update
    ftp::Type $ftpid ascii
    ftp::Get $ftpid modulesinfo -variable RetrievePT::modulesinfo
    if { ![info exists modulesinfo] } {
	set progresslabel [_ "Error retrieving list"]
	$w conf -cursor ""
	update
	after 1000 {set ::RetrievePT::progresslabel ""}
	return
    }

    regsub -lineanchor -all {^#[^\n]*} $modulesinfo {} modulesinfo
    catch { unset modulesarray }
    set modules ""
    set Name ""
    catch {
	foreach "name value" $modulesinfo {
	    if { $name == "Name" } {
		set Name $value
		lappend modules $value
	    } else {
		set modulesarray($Name,$name) $value
	    }
	}
    }
    CheckModulesInstalled

    set progressbar 100
    set progresslabel ""
    $w conf -cursor ""
    update
    after 1000 set RetrievePT::progressbar 0
}

proc RetrievePT::FillTable { list i } {
    variable modules
    variable modulesarray
    variable images

    if { ![winfo exists $list] } { return }

    foreach {currentos currentbits} [GetCurrentPlatform] break

    $list delete 0 end
    foreach module $modules {
	foreach {os bits} $modulesarray($module,Platform) break
	if { ($os == "All" || $os == $currentos) &&
	    ($bits == "All" || $bits == $currentbits ) } {
	    if { $i == 1 } continue
	} else {
	    if { $i == 0 } continue
	}
	if { [info exists modulesarray($module,Version)] } {
	    set version $modulesarray($module,Version)
	} else {
	    set version ""
	}
	if { [info exists modulesarray($module,ShortDescription)] } {
	    set shortdesc [string trim $modulesarray($module,ShortDescription)]
	} else {
	    set shortdesc ""
	}
	$list insert end [list $module $version "" $shortdesc]
    }

    set row 0
    foreach module $modules {
	foreach {os bits} $modulesarray($module,Platform) break
	if { ($os == "All" || $os == $currentos) &&
	    ($bits == "All" || $bits == $currentbits ) } {
	    if { $i == 1 } continue
	} else {
	    if { $i == 0 } continue
	}
	if { $modulesarray($module,IsInstalled) == 1 } {
	    $list cellconfigure $row,2 -image $images(checked-on)
	} elseif { $modulesarray($module,IsInstalled) == -1 } {
	    #GiD and the module are not compatible
	    $list cellconfigure $row,2 -image $images(cross)
	} else {
	    $list cellconfigure $row,2 -image $images(checked-off)
	}
	incr row
    }
}

proc RetrievePT::IsVersionOK { module } {
    variable modulesarray
    if { [info exists modulesarray($module,MinimumGiDVersion)] } {
	set moduleversion $modulesarray($module,MinimumGiDVersion)
    } else {
	return 1
    }
    set comp [GidUtils::VersionCmp $moduleversion]
    if { $comp < 0 } {
	#GiD is Old
	return 0
    } else {
	#GiD is OK
	return 1
    }
}

proc RetrievePT::CheckModulesInstalled {} {
    variable modules
    variable modulesarray
    variable localdir

    foreach module $modules {
	set InstalledFile [file join $localdir $modulesarray($module,InstalledFile)]
	set isinstalled 0
	if { [file exists $InstalledFile] } {
	    set isinstalled 1
	} elseif { [GidUtils::IsLink $InstalledFile.lnk] } {
	    set realfile [GidUtils::ReadLink $InstalledFile.lnk 1]
	    if { [string equal -nocase [file tail $realfile] $modulesarray($module,InstalledFile)] } {
		set isinstalled 1
	    }
	}
	if { $isinstalled } {
	    set modulesarray($module,IsInstalled) 1
	} elseif { ![IsVersionOK $module] } {
	    set modulesarray($module,IsInstalled) -1
	} else {
	    set modulesarray($module,IsInstalled) 0
	}
    }
}

proc RetrievePT::RetrieveModule {} {
    variable w
    variable modulesarray
    variable localdir
    variable ftpid
    variable progressbar
    variable progresslabel
    variable FileSize
    variable LastTimeMiliSec
    variable lists
    variable currenttab

    set list $lists($currenttab)

    if { [llength [$list curselection]] == 0 } {
	WarnWin [_ "Select first one Problem type"] $list
	return
    }

    if { [Connect] } { return }

    set GidVersion [GiD_Info GiDVersion]

    foreach i [$list curselection] {
	set module [lindex [$list get $i] 0]

	if { ![IsVersionOK $module] } {
	    set moduleversion $modulesarray($module,MinimumGiDVersion)
	    set text [_ "Warning: Problem type '%s' requires %s version %s and current version is %s. There can be incompatibilities. Proceed?" \
		$module $::GidPriv(ProgName) $moduleversion $GidVersion]
	    set retval [tk_messageBox -default ok -icon question -message $text -type okcancel]
	    if { $retval == "cancel" } {
		set progressbar 0
		set progresslabel ""
		$w conf -cursor ""
		return
	    }
	}
	set InstalledFile [file join $localdir $modulesarray($module,InstalledFile)]
	if { [file exists $InstalledFile] } {
	    set text [_ "Warning: Problem type '%s' is already installed in the local system. Are you sure to proceed?" $module]
	    set retval [tk_messageBox -default ok -icon question -message $text -type okcancel]
	    if { $retval == "cancel" } {
		set progressbar 0
		set progresslabel ""
		$w conf -cursor ""
		return
	    }
	}

	if { ![file exists $localdir] } {
	    file mkdir $localdir
	}

	if { ![file exists $localdir] || ![file isdirectory $localdir] } {
	    WarnWin [_ "Directory %s does not exist" $localdir]
	    set progressbar 0
	    set progresslabel ""
	    catch { $w conf -cursor "" }
	    return
	}

	set DownloadFile $modulesarray($module,DownloadFile)
	ftp::Type $ftpid binary ;#must set binary before asking file size
	set FileSize [ftp::FileSize $ftpid $DownloadFile]

	set LastTimeMiliSec [clock clicks -milliseconds]
	ftp::Get $ftpid $DownloadFile [file join $localdir $DownloadFile]
	if { ![file exists [file join $localdir $DownloadFile]] } {
	    WarnWin [_ "Problems retrieving the Problem type"]
	    set progressbar 0
	    set progresslabel ""
	    catch { $w conf -cursor "" }
	    return
	}
	set progresslabel "$FileSize/$FileSize"
	$w conf -cursor watch
	update
	if { [file size [file join $localdir $DownloadFile]] == $FileSize } {
	    set pwd [pwd]
	    cd $localdir

	    #anulado, en su lugar arranco un proceso sin bloqueo con callback
	    if { 0 } {
		set unzip [auto_execok unzip]
		if { $unzip == "" } {
		    set unzip [file join $::GIDDEFAULT unzip]
		    set unzip [auto_execok $unzip]
		}
		if { $unzip != "" } {
		    if { $::tcl_platform(platform) == "windows"} {
		        #eval exec $unzip -o [list $DownloadFile]
		    } else {
		        # flags para ficheros ASCII (CR LF => LF)
		        eval exec $unzip -a -o [list $DownloadFile]
		    }
		} else {
		    WarnWin [_ "Error: to execute this function it is necessary to have unzip"]
		}
	    }
		
	    if { $::tcl_platform(platform) == "windows"} {
		set exe "\"[file join $::GIDDEFAULT unzip.exe]\" -x -o \"$DownloadFile\""
		runExe run $exe -blocking false -callback [list RetrievePT::AfterUnzip %m $w $DownloadFile] -timeout 3600
	    } else {
		# flags para ficheros ASCII (CR LF => LF)
		set exe "unzip -a -o $DownloadFile"
		runExe run $exe -blocking false -callback [list RetrievePT::AfterUnzip %m $w $DownloadFile] -timeout 3600
	    }

	    cd $pwd
	} elseif { [winfo exists $w] } {
	    WarnWin [_ "Error retrieving the Problem type"] $w
	}
    }
}

proc RetrievePT::AfterUnzip { mode args } {
    variable lists
    variable currenttab

    switch -exact -- $mode {
	prepare  {
	    set ::RetrievePT::progresslabel [_ "Unzipping"]...
	    update
	}
	output {
	    #set ::RetrievePT::progresslabel [lindex $args 0]
	}
	finished {
	    set w [lindex $args 0]
	    set DownloadFile [lindex $args 1]
	    file delete [file join $::RetrievePT::localdir $DownloadFile]
	    ::RetrievePT::CheckModulesInstalled
	    ClearCacheFormProblemtypeMenuDir
	    #if the problemtypes menu must be deleted to use FastCacheFromProblemtypeMenu
	    #FastCacheFromProblemtypeMenu
	    catch { $w conf -cursor "" }
	    set ::RetrievePT::progresslabel [_ "Problem type downloaded"]
	    set ::RetrievePT::progressbar 0
	    set list $lists($currenttab)
	    RetrievePT::FillTable $list $currenttab
	    update
	    after 2000 { set ::RetrievePT::progresslabel "" }
	}
    }
}

proc RetrievePT::DisplayText { parent textinfo } {

    set w $parent.retrievepttext

    InitWindow $w [_ "Display info"] PreRetrievePTDisplayHelpWindowGeom
    if { ![winfo exists $w] } return ;# windows disabled || usemorewindows == 0

    set sw [ScrolledWindow $w.lf \
	    -auto both -relief sunken -borderwidth 2]
    set text [text $sw.text -width 60 -height 20]
    bind $text <1> "focus $text"
    $sw setwidget $text

    button $w.close -text [_ "Close"] -width 10 -command "destroy $w"
    SetWidgetsWidthFromText $w.close
    grid $sw -sticky nsew
    grid $w.close

    grid columnconf $w 0 -weight 1
    grid rowconf $w 0 -weight 1

    $text ins end $textinfo
    $text conf -state disabled
}

proc RetrievePT::ModuleInfo {} {
    variable modulesarray
    variable localdir
    variable lists
    variable currenttab

    set list $lists($currenttab)

    if { [llength [$list curselection]] == 0 } {
	WarnWin [_ "Select first one Problem type"] $list
	return
    }
    if { [llength [$list curselection]] > 1 } {
	WarnWin [_ "Select ONLY one Problem type"] $list
	return
    }
    set module [lindex [$list get [$list curselection]] 0]

    set desc ""
    if { ![IsVersionOK $module] } {
	set moduleversion $modulesarray($module,MinimumGiDVersion)
	set GidVersion [GiD_Info GiDVersion]
	set desc [_ "Warning: This Problem type requires %s version %s and current version is %s. There can be incompatibilities" \                
		$::GidPriv(ProgName) $moduleversion $GidVersion]
    }
    set InstalledFile [file join $localdir $modulesarray($module,InstalledFile)]
    if { [file exists $InstalledFile] } {
	append desc [_ "This Problem type is already installed"]\n
    }
    if { [info exists modulesarray($module,Description)] } {
	append desc $modulesarray($module,Description)
# WarnWinText -$modulesarray($module,Description)-
# WarnWinText -->$desc<--
    } elseif { [info exists modulesarray($module,ShortDescription)] } {
	append desc $modulesarray($module,ShortDescription)
    } else {
	WarnWin [_ "This Problem type has no description"] $list
	return
    }
    RetrievePT::DisplayText $list $desc
}

proc RetrievePT::ModuleNews {} {
    variable modulesarray
    variable localdir
    variable lists
    variable currenttab

    set list $lists($currenttab)

    if { [llength [$list curselection]] == 0 } {
	WarnWin [_ "Select first one Problem type"] $list
	return
    }
    if { [llength [$list curselection]] > 1 } {
	WarnWin [_ "Select ONLY one Problem type"] $list
	return
    }
    set module [lindex [$list get [$list curselection]] 0]

    set desc ""
    if { ![IsVersionOK $module] } {
	set GidVersion [GiD_Info GiDVersion]
	set moduleversion $modulesarray($module,MinimumGiDVersion)
	set desc [_ "Warning: This Problem type requires %s version %s and current version is %s. There can be incompatibilities" $::GidPriv(ProgName) $moduleversion $GidVersion]
    }
    set InstalledFile [file join $localdir $modulesarray($module,InstalledFile)]
    if { [file exists $InstalledFile] } {
	append desc [_ "This Problem type is already installed"]\n
    }
    if { [info exists modulesarray($module,NewsInVersion)] } {
	append desc $modulesarray($module,NewsInVersion)
    } else {
	WarnWin [_ "This Problem type has no news information"] $list
	return
    }
    RetrievePT::DisplayText $list $desc
}

proc RetrievePT::GetCurrentPlatform {} {
    if { $::tcl_platform(platform) == "windows" } {
	if { $::tcl_platform(machine) == "amd64" } {
	    set myplatform "Windows 64"
	} else {
	    set myplatform "Windows 32"
	}
    } elseif { $::tcl_platform(os)        == "Linux" } {
	if { $::tcl_platform(machine) == "x86_64" } {
	    set myplatform "Linux 64"
	} else {
	    set myplatform "Linux 32"
	}
    } elseif { $::tcl_platform(os) == "Darwin" } {
	if { $::tcl_platform(machine) == "Power Macintosh" || $::tcl_platform(machine) == "i386" } {
	    set myplatform "MacOSX 32"
	} else {
	    set myplatform "MacOSX 64"
	}
    } else {
	#AIX , HP-UX, IRIX, SunOS, DragonFly, FreeBSD, NetBSD, OpenBSD, OSF1
	set myplatform "$::tcl_platform(os) ?"
    }
    return $myplatform
}

proc RetrievePT::OpenGUI { { parent .gid } } {
    variable w
    variable progressbar
    variable progresslabel
    variable images
    variable lists
    variable currenttab

    set images(checked-on) [image create photo -data {
	R0lGODlhEAAQAIQAAIKCgrxdXe4AANnZ2fePj+0BAdVOWdl6fugGBuSgpeimpu0AANvW1up8
	fOwAANympv///+ttbe0fH+0UFNyrq+TAwOa1tf//////////////////////////////////
	/yH+Dk1hZGUgd2l0aCBHSU1QACH5BAEKAB8ALAAAAAAQABAAAAVL4CeOZGmeJqCuLFC2bSCQ
	6mDfA1EYNIDbB0Ti5VMsGIOG4wEhDhbHiEMBafYGAoFkQhlUndlFxfa9WhaWW3mkqrrfr7fc
	iqrb7/cQADs=
    }]
    set images(checked-off) [image create photo -data {
	R0lGODlhEAAQAKEAAP///4KCgv///////yH+Dk1hZGUgd2l0aCBHSU1QACH5BAEKAAIALAAA
	AAAQABAAAAImlI+py50AowRvylBXwAhqfnjKlo2gISZkR5mlegrQRtdPjcfO3hQAOw==
    }]
    set images(cross) [image create photo -data {
	R0lGODlhEAAQAIQAAIKCgrxdXe4AANnZ2fePj+0BAdVOWdl6fugGBuSgpeimpu0AANvW1up8
	fOwAANympv///+ttbe0fH+0UFNyrq+TAwOa1tf//////////////////////////////////
	/yH+Dk1hZGUgd2l0aCBHSU1QACH5BAEKAB8ALAAAAAAQABAAAAVJ4CeOZGmeJqCuLFC2bSCQ
	qiAMOG4bNDDYOVvi5VMIGD/BA0IcLBY2gQLC7CWjA2ozWrFprTaLTlAd1chUqu0FQafVqLh8
    }]

    package require ftp
    package require tablelist_tile
    package require progressbar

    set w $parent.retrievept

    catch { destroy $parent.retrievept }

    InitWindow $w [_ "Retrieve analysis Problem types"] PreRetrievePTWindowGeom RetrievePT::OpenGUI
    if { ![winfo exists $w] } return ;# windows disabled || usemorewindows == 0

    wm protocol $w WM_DELETE_WINDOW RetrievePT::Close


    label $w.l1 -text \
	[_ "Note: In order to use this function, it is necessary to be connected to internet"] \
	    -font BoldFont

    set currentplatform [GetCurrentPlatform]

    set nb [ttk::notebook $w.nb]
    ttk::notebook::enableTraversal $nb

    set f1 [ttk::frame $nb.f1]
    $nb add $f1 -text $currentplatform -underline 0
    set lists(0) [RetrievePT::CreateTable $f1]

    set f2 [ttk::frame $nb.f2]
    $nb add $f2 -text [_ "Other OS"] -underline 0
    set lists(1) [RetrievePT::CreateTable $f2]


    set wp [::progressbar::progressbar $w.f -variable ::RetrievePT::progressbar -width 300 \
	    -background [ $w cget -background]]
    label $w.l2 -textvariable RetrievePT::progresslabel
    set progressbar 0
    set progresslabel ""

    grid $w.l1 -sticky ew
    grid $nb -sticky nsew
    grid $w.l2 -sticky ew
    grid $wp -sticky ew
    grid columnconfigure $w 0 -weight 1
    grid rowconfigure $w 1 -weight 1

    set def_back [$w cget -background]
    frame $w.buts -bg [CCColorActivo $def_back]

    button $w.buts.b2 -text [_ "Retrieve Problem type"] -command RetrievePT::RetrieveModule
    button $w.buts.b3 -text [_ "Problem type info"] -command RetrievePT::ModuleInfo
    button $w.buts.b4 -text [_ "Problem type news"] -command RetrievePT::ModuleNews
    button $w.buts.b5 -text [_ "Close"] -command RetrievePT::Close
    #SetWidgetsWidthFromText $w.buts.b2 $w.buts.b3 $w.buts.b4 $w.buts.b5
    grid $w.buts.b2 $w.buts.b3 $w.buts.b4 $w.buts.b5 -padx 2 -pady 3

    grid $w.buts -sticky ew
    grid columnconf $w.buts "0 1 2 3" -weight 1
    if { $::tcl_version >= 8.5 } { grid anchor $w.buts center }
    grid columnconfigure $w 0 -weight 1
    grid rowconfigure $w 1 -weight 1

    update idletasks

    set progresslabel [_ "Retrieving Problem type list"]...
    update
    RetrievePT::Retrievelist

    RetrievePT::FillTable $lists(0) 0
    RetrievePT::FillTable $lists(1) 1

    if { ![info exists currenttab] } {
	set currenttab 0
    }
    $nb select $currenttab

    set progresslabel ""
    update
    bind $w <<NotebookTabChanged>> [list RetrievePT::TabChanged $nb]
}

proc RetrievePT::TabChanged { nb } {
    variable currenttab
    set currenttab [$nb index current]
}

proc RetrievePT::CreateTable { w } {
    set sw [ScrolledWindow $w.lf \
	    -auto both -relief sunken -borderwidth 2]
    set list [tablelist::tablelist $sw.lb \
	    -exportselection 0 \
	    -columns [list \
		18 [_ "Problem type"]        left \
		7 [_ "Version"]        left \
		4 [_ "Installed"] center \
		20 [_ "Characteristics"]        left \
		] \
	    -labelcommand tablelist::sortByColumn \
	    -background white \
	    -selectbackground navy -selectforeground white \
	    -stretch end -selectmode extended]
    #-highlightthickness 0

    $sw setwidget $list

    bind [$list bodytag] <Button-3> "[list RetrievePT::MenuContextualRetrievePT %W %x %y] ; break"
    grid $sw -sticky nsew
    grid columnconfigure $w 0 -weight 1
    grid rowconfigure $w 0 -weight 1
    return $list
}

proc RetrievePT::MenuContextualRetrievePT { T x y } {
    set txt [[winfo parent $T] cellcget active -text]
    if { $txt == "" } return
    set w $T.menucontextualretrievept
    if { [winfo exists $w] } {
	destroy $w
    }
    menu $w
    $w add command -label [_ "Retrieve %s" $txt] -command RetrievePT::RetrieveModule
    $w add command -label [_ "%s info" $txt] -command RetrievePT::ModuleInfo
    $w add command -label [_ "%s news" $txt] -command RetrievePT::ModuleNews
    set x [expr [winfo rootx $T]+$x+2]
    set y [expr [winfo rooty $T]+$y]
    GiD_PopupMenu $w $x $y
}

