
set GIDDEFAULTTCL [file dirname [info script]]
set ::env(TK_LIBRARY) [file join $GIDDEFAULTTCL tk]
lappend auto_path $GIDDEFAULTTCL $GIDDEFAULTTCL/tcl $GIDDEFAULTTCL/tk
set gidexe [file join $GIDDEFAULTTCL .. gid.exe]

package require Tk 8.5
package require dialogwin
package require msgcat

set SoftwareOpenGL 0
set Language en

set ret [exec $gidexe -outputinfo defaults]
regexp -line {GidDefaults=(.*)} $ret {} inifile
set ret [exec $gidexe -outputinfo]
set ret [regexp -line {Name:\s+(\S+)} $ret {} machine]
if { !$ret } {
    set ret [regexp -line {^\s*(\S+)} $ret {} machine]
}

set inifile0 [file join $GIDDEFAULTTCL gidDefaults]

foreach file [list $inifile0 $inifile] {
    if { [file exists $file] } {
	set fin [open $file r]
	set data [read $fin]
	close $fin
	regexp -line {^SoftwareOpenGL\s+(\d+)} $data {} SoftwareOpenGL
	regexp -line {^Language\s+(\S+)} $data {} Language
    }
}

msgcat::mclocale $Language
msgcat::mcload [file join $GIDDEFAULTTCL msgs]
interp alias "" _ "" msgcat::mc

set Languages [list en]
foreach file [glob -nocomplain -dir [file join $GIDDEFAULTTCL msgs] *.msg] {
    lappend Languages [file root [file tail $file]]
}
set Languages [lsort -dictionary -unique $Languages]

wm withdraw .

set w [dialogwin_snit ._ask -title [_ "Safe mode"] -cancelname [_ Exit]]
set f [$w giveframe]

set f1 [ttk::labelframe $f.f1 -text OpenGL]

ttk::checkbutton $f1.cb -text [_ "Activate Software OpenGL"] -variable SoftwareOpenGL

label $f1.l1 -text [_ "Note: If OpenGL by software is activated, graphics will be slower.\
	Only activate it if you are experiencing problems with graphics visualization"] \
    -wraplength 250 -justify left

grid $f1.cb -sticky w -padx 3 -pady 3
grid $f1.l1 -sticky w -padx 3 -pady 3

set f2 [ttk::labelframe $f.f2 -text [_ "Preferences"]]

ttk::checkbutton $f2.cb -text [_ "Reset preferences"] -variable reset_prefs
set reset_prefs 0

label $f2.l1 -text [_ "Note: If preferences are reset. Program will start as if it was\
	installed for the first time. All preferences data will be lost"] \
    -wraplength 250 -justify left

grid $f2.cb -sticky w -padx 3 -pady 3
grid $f2.l1 -sticky w -padx 3 -pady 3

set f3 [ttk::labelframe $f.f3 -text [_ "Languages"]]

ttk::label $f3.l1 -text [_ "Language"]:

array set lans [list en English es Spanish ja Japanese fr French ca Catalan \
	de German ko Korean]
set Language_p $Language
set Languages_p ""
foreach i $Languages {
    if { [info exists lans($i)] } {
	set i_p $lans($i)
    } else {
	set i_p $i
    }
    lappend Languages_p $i_p
    if { $i eq $Language } {
	set Language_p $i_p
    }
}


ttk::combobox $f3.cb -textvariable Language_p -state readonly -values $Languages_p

grid $f3.l1 $f3.cb -padx 3 -pady 3

grid $f1 -sticky nwe
grid $f2 -sticky nwe
grid $f3 -sticky nwe

set action [$w createwindow]

if { $action < 1 } { exit }

set Language $Language_p
foreach "n v" [array get lans] {
    if { $Language_p eq $v } {
	set Language $n
	break
    }
}

proc InitTemporalVariables { machine } {
    set addline 1
    set string "$machine IDIDIDIDIDIDIDID"
    set filename [file join $::GIDDEFAULTTCL TemporalVariables]
    set fin [open $filename r]
    if { $fin != "" } {
	set len [string length $string]
	foreach line [split [read $fin] \n] {
	    if { ![string compare -length $len $string $line] } {
		set addline 0
		break
	    }
	}
	close $fin
    }
    if { $addline } {
	set fout [open $filename a]
	if { $fout != "" } {
	    set date [clock format [clock seconds] -format "%b %d %Y"]
	    puts $fout "$string # Registration data: $date."
	    close $fout
	}
    }
}


if { $reset_prefs } {
    set inidir [file dirname $inifile]
    file delete -force $inidir~
    if { [file exists $inidir] } {
	file rename $inidir $inidir~
    }
    file mkdir $inidir

    InitTemporalVariables $machine
}

set fout [open $inifile a]
puts $fout "SoftwareOpenGL $SoftwareOpenGL"
puts $fout "Language $Language"
close $fout


eval [list exec $gidexe] [lrange $argv 2 end] &

exit




















