
proc ChangeColorFondoGiD { idx_var { actualiza_color 0}} {
    global ChangeVarInfo

    set wb $ChangeVarInfo($idx_var,wid)
    if { ![ winfo exists $wb]} {
	return
    }
    if { $actualiza_color} {
	$wb conf -bg $ChangeVarInfo($idx_var)
    } else {
	set nuevo [ GIDChooseColor $wb.mc -title [_ "Select color"] -color $ChangeVarInfo($idx_var)]
	if { [ string length $nuevo]} {
	    set ChangeVarInfo($idx_var) $nuevo
	    $wb conf -bg $nuevo
	}
    }
}

proc ChangeColorSenseFondoGiD { sentido} {
    global ChangeVarInfo

    set ChangeVarInfo(BackColorSense) $sentido
    set wb $ChangeVarInfo(BackColorSense,wid)
    if { ![ winfo exists $wb]} {
	return
    }
    $wb configure -image $ChangeVarInfo(BackColorSense,$sentido)

    $ChangeVarInfo(BackColorSense,TxtArriba,wid) configure -text $ChangeVarInfo(BackColorSense,TxtArriba,$sentido)

    $ChangeVarInfo(BackColorSense,TxtAbajo,wid) configure -text $ChangeVarInfo(BackColorSense,TxtAbajo,$sentido)
}

proc ChangeColorEnableDisableWidgets { } {
    global ChangeVarInfo

    if { ![ info exists ChangeVarInfo(BackColorType,lst_wid)]} { return}
    foreach wb $ChangeVarInfo(BackColorType,lst_wid) {
	if { ![ winfo exists $wb]} {
	    continue
	}
	if { [ winfo class $wb] == "Label"} {
	    if { $ChangeVarInfo(BackColorType)} {
		$wb configure -fg black
	    } else {
		$wb configure -fg grey60
	    }
	} else {
	    if { $ChangeVarInfo(BackColorType)} {
		$wb configure -state normal
	    } else {
		$wb configure -state disabled
	    }
	}
    }
}

proc pref_graphical_ui {base args} {
    global ChangeVarInfo GidPriv

    ttk::labelframe $base.fsoftwareopengl -text [_ "OpenGL"]
    ttk::frame $base.frame#3 -borderwidth 2 -relief ridge
    ttk::frame $base.frmBackgroundColor
    ttk::labelframe $base.frame#1 -text [_ "Curve precision"]
    ttk::labelframe $base.frame#1b -text [_ "Surface precision (Render)"]


    ttk::radiobutton $base.fsoftwareopengl.rd1 -text [_ "Fast visualization mode."] \
	-value 0 -variable ::ChangeVarInfo(SoftwareOpenGL)
    setTooltip $base.fsoftwareopengl.rd1 \
	[_ "Models will be drawn quicker but some issues could appear. I'm confident about my Graphic Card and I want to use its capabilities."]

    ttk::radiobutton $base.fsoftwareopengl.rd2 -text [_ "Safe visualization mode."] \
	-value 1 -variable ::ChangeVarInfo(SoftwareOpenGL)
    setTooltip $base.fsoftwareopengl.rd2 \
	[_ "Models will be drawn slower but the artifacts would disappear. I'm experiencing troubles with my Graphic Card and I want to use OpenGL by software."]

    #"Emulate Front Buffer ( dynamic lines hack)"
    ttk::checkbutton $base.fsoftwareopengl.cbEmulateFB -text [_ "Selection lines by software (emulateFrontBuffer)."] \
	-variable ChangeVarInfo(OGL_emulateFrontBuffer)
    # GidHelp
    setTooltip $base.fsoftwareopengl.cbEmulateFB [_ "On some graphic cards, when a selection is done or a line is created, \nthe used dynamic lines are drawn with problems, or are not drawn at all \n(for instance, on Intel GMA graphic cards).\nTo avoid this problem select this option which does not use the \nOpenGL FrontBuffer to drawn these kind of lines."]

    grid $base.fsoftwareopengl.rd1 $base.fsoftwareopengl.rd2 -sticky ew -padx 2
    grid $base.fsoftwareopengl.cbEmulateFB -sticky w -padx 2 -columnspan 2


    # on Mac OS X, if the software renderer is selected, better to use emulateFrontBuffer too
    # because there are some problems with dynamic lines
    if { $::tcl_platform(os) == "Darwin"} {
	$base.fsoftwareopengl.rd1 configure -command {
	    set ::ChangeVarInfo(OGL_emulateFrontBuffer) $::ChangeVarInfo(SoftwareOpenGL)
	}
	$base.fsoftwareopengl.rd2 configure -command {
	    set ::ChangeVarInfo(OGL_emulateFrontBuffer) $::ChangeVarInfo(SoftwareOpenGL)
	}
    }

    ttk::checkbutton $base.checkbutton#1 -text [_ "Smoothed elems(deg)"]: \
	-variable ChangeVarInfo(Lightsmoothedelements)

    ttk::entry $base.entry#1 -textvariable ChangeVarInfo(AngleSmoothed) -width 8

    GidHelp "$base.checkbutton#1 $base.entry#1" [_ "If set and when rendering\
	    a mesh, the intersection between elements with a small angle\
	    between their normals will be illuminated as if it were\
	    a continuous solid. If not, illumination is made considering\
	    every element as planar.\
	    \nThe angle (in degrees) is the maximum angle between normals\
	    of two elements to allow smooth lighting between them.\
	    If not, one edge is drawn between them."]

    ttk::label $base.label#35 -text [_ "Surface drawing type"]: -justify left

    TTKComboBox $base.menubutton#1 -textvariable ::ChangeVarInfo(DrawSurfaceMode) \
	-labels [ list [_ "None"] [_ "Boundary lines"] [_ "Isoparametric lines"] [_ "Both modes"]] \
	-values {None "Boundary lines" "Isoparametric lines" "Both modes"} \
	-state readonly

    GidHelp "$base.label#35 $base.menubutton#1" [_ "Lets the user choose how to\
	    draw the surfaces when in wireframe (normal) mode. Options are:\
	    \n'None': Surfaces are not drawn.\
	    \n'Boundary lines': One magenta line is drawn for every\
	    contour line. This set of lines have a small offset towards\
	    the interior of the surface.\
	    \n'Isoparametric lines': Two yellow lines are drawn for every\
	    NURBS surface. The one for 'u=0.5' and the one for 'v=0.5'.\
	    \n'Both': Draws both modes."]

    ttk::label $base.labelQuadratic -text [_ "Draw quadratic elements"]: -justify left
    TTKComboBox $base.menubuttonQuadratic -textvariable ::ChangeVarInfo(DrawQuadratic) \
	-labels [ list [_ "No#C#Nothing, None"] [_ "As lines"] [_ "As curves"]] \
	-values {No AsLines AsCurve} \
	-state readonly
    GidHelp "$base.labelQuadratic $base.menubuttonQuadratic"  [_ "With this option \
	    user can specify how to visualize quadratic elements: as linear (No), as \
	    quadratic but with straight edges between element nodes (As lines), or \
	    as quadratic curved edges (As curves)."]
    ttk::label $base.labelLabelOptions -text [_ "Entity labels show"]: -justify left
    TTKComboBox $base.menubuttonLabelOptions -textvariable ::ChangeVarInfo(LabelOptions) \
	-labels [ list [_ "Num"] [_ "Layer"] [_ "Num & layer"]] \
	-values {1 2 3} \
	-state readonly
    GidHelp "$base.labelLabelOptions $base.menubuttonLabelOptions"  [_ "With this option \
	    user can specify the information showed by entity labels: entity number and/or its layer name."]

    global MenuColorsForceChecked
    set MenuColorsForceChecked 1

    ttk::menubutton $base.menuEntityColor \
	-menu $base.menuEntityColor.m -image [GetImage color.gif]

    menu $base.menuEntityColor.m -tearoff 0
    $base.menuEntityColor.m add checkbutton -label [_ "Background"]... \
	-variable MenuColorsForceChecked \
	-command {SelectColorEntity BackgroundColor 0 ; set MenuColorsForceChecked 1}
    $base.menuEntityColor.m add checkbutton -label [_ "Points"]... \
	-variable MenuColorsForceChecked \
	-command {SelectColorEntity ColorPoints 1 ; set MenuColorsForceChecked 1}
    $base.menuEntityColor.m add checkbutton -label [_ "Lines"]... \
	-variable MenuColorsForceChecked \
	-command {SelectColorEntity ColorLines 2 ; set MenuColorsForceChecked 1}
    $base.menuEntityColor.m add checkbutton -label [_ "PolyLines"]... \
	-variable MenuColorsForceChecked \
	-command {SelectColorEntity ColorPolyLines 3 ; set MenuColorsForceChecked 1}
    $base.menuEntityColor.m add checkbutton -label [_ "Surfaces"]... \
	-variable MenuColorsForceChecked \
	-command {SelectColorEntity ColorSurfaces 4 ; set MenuColorsForceChecked 1}
    $base.menuEntityColor.m add checkbutton -label [_ "Surfaces Iso"]... \
	-variable MenuColorsForceChecked \
	-command {SelectColorEntity ColorSurfsIsoparametric 5 ; set MenuColorsForceChecked 1}
    $base.menuEntityColor.m add checkbutton -label [_ "Volumes"]... \
	-variable MenuColorsForceChecked \
	-command {SelectColorEntity ColorVolumes 6 ; set MenuColorsForceChecked 1}
    $base.menuEntityColor.m add checkbutton -label [_ "Nodes"]... \
	-variable MenuColorsForceChecked \
	-command {SelectColorEntity ColorNodes 7 ; set MenuColorsForceChecked 1}
    $base.menuEntityColor.m add checkbutton -label [_ "Elements"]... \
	-variable MenuColorsForceChecked \
	-command {SelectColorEntity ColorElements 8 ; set MenuColorsForceChecked 1}

    GidHelp "$base.menuEntityColor" \
	[_ "User can choose with this option the entity color of\
	    the graphical window."]

    ttk::checkbutton $base.checkbutton#4 -text [_ " GUI attenuated"] \
	-variable ChangeVarInfo(GUIAppearance)

    GidHelp "$base.checkbutton#4" \
	[_ "Set the GUI appearance to standard or attenuated color.\n\
	    (the change is updated restarting %s)" $GidPriv(ProgName)]

    ttk::frame $base.tcg
    ttk::checkbutton $base.tcg.checkbutton#4b -text [_ "Two colours background"] \
	-variable ChangeVarInfo(BackColorType) \
	-command ChangeColorEnableDisableWidgets

    ChangeColorEnableDisableWidgets

    GidHelp "$base.tcg.checkbutton#4b" \
	[_ "Shows the background with a two colour degradation"]

    ttk::label $base.tcg.lArriba -text [_ "Top"]:
    button $base.tcg.colorArriba -width 3 -command {ChangeColorFondoGiD BackColorTop}
    set ChangeVarInfo(BackColorTop,wid) $base.tcg.colorArriba

    button $base.tcg.intercambio \
	-image [GetImage "ArrowLeftRight.gif"] \
	-command {
	global ChangeVarInfo
	set tmp $ChangeVarInfo(BackColorTop)
	set ChangeVarInfo(BackColorTop) $ChangeVarInfo(BackColorBottom)
	set ChangeVarInfo(BackColorBottom) $tmp
	ChangeColorFondoGiD BackColorTop 1
	ChangeColorFondoGiD BackColorBottom 1
    } -bd 0

    ttk::label $base.tcg.lAbajo -text [_ "Bottom"]:
    button $base.tcg.colorAbajo -width 3 -command {ChangeColorFondoGiD BackColorBottom}
    set ChangeVarInfo(BackColorBottom,wid) $base.tcg.colorAbajo



    ttk::label $base.tcg.sentido -text [_ "Sense"]:
    ttk::menubutton $base.tcg.cbs -menu $base.tcg.cbs.m
    menu $base.tcg.cbs.m -tearoff no -bd 0 -activeborderwidth 0
    set ChangeVarInfo(BackColorSense,wid) $base.tcg.cbs
    set ChangeVarInfo(BackColorSense,TxtArriba,wid) $base.tcg.lArriba
    set ChangeVarInfo(BackColorSense,TxtAbajo,wid) $base.tcg.lAbajo

    set ChangeVarInfo(BackColorType,lst_wid) [ list $base.tcg.lArriba $base.tcg.colorArriba \
	    $base.tcg.lAbajo $base.tcg.colorAbajo \
	    $base.tcg.intercambio \
	    $base.tcg.sentido $base.tcg.cbs \
	    ]

    set primera 0
    foreach img "ArrowDown16.gif ArrowUp16.gif ArrowRight16.gif ArrowLeft16.gif ArrowUpLeft16.gif ArrowUpRight16.gif ArrowDownRight16.gif ArrowDownLeft16.gif" \
	val "d u r l ul ur dr dl" \
	txtArriba [list [_ "Top"]: [_ "Bottom"]: [_ "Left"]: [_ "Right"]: \
	    [_ "Bottom right"]: [_ "Bottom left"]: [_ "Top left"]: [_ "Top right"]:] \
	txtAbajo  [list [_ "Bottom"]: [_ "Top"]: [_ "Right"]: [_ "Left"]: \
	    [_ "Top left"]: [_ "Top right"]: [_ "Bottom right"]: [_ "Bottom left"]:] {
	set img_id [GetImage $img]
	$base.tcg.cbs.m add command -image $img_id -hidemargin 1 -command [list ChangeColorSenseFondoGiD $val]
	if { !$primera} {
	    $base.tcg.cbs configure -image $img_id
	    set primera 1
	}
	set ChangeVarInfo(BackColorSense,$val) $img_id
	set ChangeVarInfo(BackColorSense,TxtArriba,$val) $txtArriba
	set ChangeVarInfo(BackColorSense,TxtAbajo,$val) $txtAbajo
    }

    grid $base.tcg.checkbutton\#4b -columnspan 5
    grid $base.tcg.lArriba $base.tcg.colorArriba $base.tcg.intercambio \
	$base.tcg.lAbajo $base.tcg.colorAbajo \
	$base.tcg.sentido $base.tcg.cbs -padx 2

    if { $::GidPriv(AllowBGChange)} {
	ttk::frame $base.changeBG
	ttk::button $base.changeBG.b -image [GetImage HSV_color.gif] \
	    -command [list GetColorForGUI $base $base.changeBG Background]
	ttk::label $base.changeBG.l1 -width 1 -text " "
	ttk::label $base.changeBG.l2 -text [_ "Change GUI Background"]:
	grid $base.changeBG.l1 $base.changeBG.l2 $base.changeBG.b
    }

    button $base.button#4 -text +

    scale $base.scale#4 -variable ChangeVarInfo(CurvePrecision) \
	-from 1.0 -to 0.0 -resolution 0.1 \
	-borderwidth 2 -relief groove \
	-length 1 -orient h  -command {}
    
    if { ![info exists ::CurvePrecisionDynamic] } {
	set ::CurvePrecisionDynamic 0
    }
    if { $::CurvePrecisionDynamic == 1 } { 
	$base.scale#4 configure -command DynamicPrecision
    } 

    button $base.button#3 -text -

    ttk::checkbutton $base.checkbutton#3 -text [_ "Dynamic"] -variable ::CurvePrecisionDynamic \
	-command " \
	if { \$::CurvePrecisionDynamic == 1 } {  \
	    $base.scale#4 conf -command DynamicPrecision \
	} else { $base.scale#4 conf -command {} }"
    #use ::CurvePrecisionDynamic instead ::ChangeVarInfo(Dynamic) the array is unset when 
    #destroying the window #and is not stored as a GiD_Set variable

    GidHelp "$base.frame#1 $base.button#4 $base.scale#4 $base.button#3 \
	$base.checkbutton#3" [_ "This option gives the precision used to\
	    draw curves. The internal definition of curves does not change.\
	    With the dynamic button, it is possible to dynamically change\
	    the drawing of a curve to test the precision."]


    button $base.button#4b -text +

    scale $base.scale#4b \
	-variable ChangeVarInfo(SurfacePrecision) \
	-from 1.0 -to 0.0 -resolution 0.1 \
	-borderwidth 2 -relief groove \
	-length 1 -orient h

    button $base.button#3b -text -

    GidHelp "$base.frame#1b $base.button#4b $base.scale#4b $base.button#3b" \
	[_ "This option gives the precision used to\
	    draw surfaces when in render mode"]

    ttk::frame $base.rotation

    ttk::checkbutton $base.rotation.checkbutton#2 -text [_ "Fast rotation"] \
	-variable ChangeVarInfo(FastRotation)

    GidHelp $base.rotation.checkbutton#2 [_ "When 'Fast rotation' is set, drawing will be\
	    faster for both the geometry and the mesh when rotating the\
	    model. The way of increasing speed depends on the options."]

    ttk::checkbutton $base.rotation.checkbutton#22 -text [_ "Automatic rotation center"] \
	-variable ChangeVarInfo(AutomaticRotationCenter)

    GidHelp $base.rotation.checkbutton#22 [_ "When 'Automatic rotation center' is set,\
	    the nearest geometrical/mesh point to the screen center will be used\
	    as rotation center after each 'Zoom In' or 'Zoom Out'."]

    grid $base.rotation.checkbutton#2 $base.rotation.checkbutton#22 -padx 2

    set pp [ $base cget -background]

    checkbutton $base.checkbutton\#15 -image [GetImage "point16x16.gif"] \
	-indicatoron 0 \
	-variable ChangeVarInfo(FRPoints) -bd 1 -selectcolor $pp

    checkbutton $base.checkbutton\#13 \
	-disabledforeground grey60 \
	-image [GetImage "line16x16.gif"] \
	-indicatoron 0 \
	-variable ChangeVarInfo(FRLines) -bd 1 -selectcolor $pp

    checkbutton $base.checkbutton\#7 \
	-disabledforeground grey60 \
	-image [GetImage "coonsurf16x16.gif"] \
	-indicatoron 0 \
	-variable ChangeVarInfo(FRSurfaces) -bd 1 -selectcolor $pp

    checkbutton $base.checkbutton\#11 \
	-disabledforeground grey60 \
	-image [GetImage "volume16x16.gif"] \
	-indicatoron 0 \
	-variable ChangeVarInfo(FRVolumes) -bd 1 -selectcolor $pp

    GidHelp $base.checkbutton\#15 $base.checkbutton#13 $base.checkbutton#7 \
	$base.checkbutton\#11 [_ "To draw or not to draw this type of entity\
	    when rotating."]

    checkbutton $base.checkbutton\#10 \
	-disabledforeground grey60 \
	-text [_ "Always geometry"] \
	-variable ChangeVarInfo(AlwaysGeom)

    GidHelp $base.checkbutton\#10 \
	[_ "If set, when viewing the mesh and rotating, the geometry is drawn instead."]

    checkbutton $base.checkbutton\#16 \
	-disabledforeground grey60 \
	-text [_ "Graphic objects"] \
	-variable ChangeVarInfo(DrawGraphicObjects)

    GidHelp $base.checkbutton\#16 [_ "If not set, when rotating the geometry,\
	    some graphical and temporal objects like normals or materials\
	    or conditions symbols are not drawn. "]

    # Geometry management
    grid $base.fsoftwareopengl -sticky ew -columnspan 2 -padx 2
    grid columnconfigure $base.fsoftwareopengl "0 1" -weight 1
    grid $base.checkbutton#1 $base.entry#1 -sticky w -padx 2
    grid configure $base.entry#1 -sticky ew
    grid $base.label#35 $base.menubutton#1 -sticky w -padx 2
    grid configure $base.menubutton#1 -sticky ew
    grid $base.labelQuadratic $base.menubuttonQuadratic -sticky w -padx 2
    grid configure $base.menubuttonQuadratic -sticky ew
    grid $base.labelLabelOptions $base.menubuttonLabelOptions -sticky w -padx 2
    grid configure $base.menubuttonLabelOptions -sticky ew
    grid $base.frmBackgroundColor -columnspan 2 -padx 2 -sticky w
    grid $base.frame#1  -columnspan 2 -padx 2 -sticky ew
    grid $base.frame#1b -columnspan 2 -padx 2 -sticky ew
    grid $base.rotation -columnspan 2 -padx 2 -sticky w
    grid $base.frame#3  -columnspan 2 -padx 2 -sticky ew

    grid $base.button#4 -in $base.frame#1        -row 1 -column 1
    grid $base.scale#4 -in $base.frame#1        -row 1 -column 2  -sticky ew
    grid $base.button#3 -in $base.frame#1        -row 1 -column 3
    grid $base.checkbutton#3 -in $base.frame#1        -row 2 -column 2 -sticky w


    grid $base.button#4b -in $base.frame#1b        -row 1 -column 1
    grid $base.scale#4b -in $base.frame#1b        -row 1 -column 2 -sticky ew
    grid $base.button#3b -in $base.frame#1b        -row 1 -column 3

    grid $base.menuEntityColor -in $base.frmBackgroundColor -row 1 -column 0
    grid $base.checkbutton#4 -in $base.frmBackgroundColor -row 1 -column 1
    grid $base.tcg -in $base.frmBackgroundColor -row 2 -column 0 -columnspan 3

    if { $::GidPriv(AllowBGChange)} {
	grid $base.changeBG -in $base.frmBackgroundColor -row 1 -column 2 -sticky e
    }

    grid $base.checkbutton#15 $base.checkbutton#13 $base.checkbutton#10 -in $base.frame#3 -sticky w
    grid $base.checkbutton#7 $base.checkbutton#11 $base.checkbutton#16 -in $base.frame#3 -sticky w


    # Resize behavior management
    grid columnconfigure $base.frame#3 3 -weight 1

    grid columnconfigure $base "0 1" -weight 1

    grid columnconfigure $base.frame#1 2 -weight 1
    grid columnconfigure $base.frame#1b 2 -weight 1
    # additional interface code


    proc ChangeFastRotation {name1 name2 op} {
	global ChangeVarInfo
	
	set slaves [grid slaves $ChangeVarInfo(FRFrame)]
	foreach i $slaves {
	    set slaves [concat $slaves [grid slaves $i]]
	}
	
	foreach i $slaves {
	    if { $ChangeVarInfo(FastRotation) == 0 } {
		catch {
		    $i conf -state disabled
		    if { [winfo class $i] == "Scale" } { $i conf -fg grey60 }
		}
	    } else {
		catch {
		    $i conf -state normal
		    if { [winfo class $i] == "Scale" } { $i conf -fg black }
		}
	    }
	}
    }

    proc selectEntityColor { entity } {
	global ChangeVarInfo
	
	# Select and Update interface
	#set retcol [tk_chooseColor -initialcolor $ChangeVarInfo($entity)]
	set retcol [GIDChooseColor $ChangeVarInfo(PG).selectcolor -title [_ "Select color"] -color $ChangeVarInfo($entity)]
	
	# Set preference
	if { $retcol != "" } {
	    set  ChangeVarInfo($entity) $retcol
	    set res 1
	} else {
	    set res 0
	}
	
	return $res
    }

    proc ChangeColor  {name elem op} {
	global ChangeVarInfo
	set indice 0
	foreach i "BackgroundColor ColorPoints ColorLines ColorPolyLines \
	    ColorSurfaces ColorSurfsIsoparametric ColorVolumes \
	    ColorNodes ColorElements" {
	    if [ info exists ChangeVarInfo($i) ] {
		$ChangeVarInfo(PG).menuEntityColor.m  entryconfigure $indice -selectcolor $ChangeVarInfo($i)
	    }
	    incr indice
	}
    }

    proc SelectColorEntity { entity indice} {
	global ChangeVarInfo
	if { [selectEntityColor $entity] } {
	    $ChangeVarInfo(PG).menuEntityColor.m  entryconfigure $indice -selectcolor $ChangeVarInfo($entity)
	}
    }

    global ChangeVarInfo

    set ChangeVarInfo(PG) $base
    set ChangeVarInfo(FRFrame) $base.frame#3

    trace variable ChangeVarInfo(FastRotation) w ChangeFastRotation
    # mpc
    #trace variable ChangeVarInfo(BackgroundColor) w changeBackgroundColor

    trace variable ChangeVarInfo(BackgroundColor) w ChangeColor
    trace variable ChangeVarInfo(ColorPoints) w ChangeColor
    trace variable ChangeVarInfo(ColorLines) w ChangeColor
    trace variable ChangeVarInfo(ColorPolyLines) w ChangeColor
    trace variable ChangeVarInfo(ColorSurfaces) w ChangeColor
    trace variable ChangeVarInfo(ColorSurfsIsoparametric) w ChangeColor
    trace variable ChangeVarInfo(ColorVolumes) w ChangeColor
    trace variable ChangeVarInfo(ColorNodes) w ChangeColor
    trace variable ChangeVarInfo(ColorElements) w ChangeColor
    #mpc


    proc LightsmoothedelementsProc {} {
	global ChangeVarInfo
	
	if { $ChangeVarInfo(Lightsmoothedelements) } {
	    $ChangeVarInfo(PG).entry#1 conf -state normal
	} else {
	    $ChangeVarInfo(PG).entry#1 conf -state disabled
	}
    }

    trace variable ChangeVarInfo(Lightsmoothedelements) w "LightsmoothedelementsProc ;#"

    set ChangeVarInfo(ScalePrecision) $base.scale#4
    $base.button#4 conf -command {
	$ChangeVarInfo(ScalePrecision) set \
	    [expr $ChangeVarInfo(CurvePrecision)+0.1]
    }

    $base.button#3 conf -command {
	$ChangeVarInfo(ScalePrecision) set \
	[expr $ChangeVarInfo(CurvePrecision)-0.1]
    }

    set ChangeVarInfo(ScalePrecisionb) $base.scale#4b
    $base.button#4b conf -command {
	$ChangeVarInfo(ScalePrecisionb) set \
	[expr $ChangeVarInfo(SurfacePrecision)+0.1]
    }

    $base.button#3b conf -command {
	$ChangeVarInfo(ScalePrecisionb) set \
	[expr $ChangeVarInfo(SurfacePrecision)-0.1]
    }

    # end additional interface code

}

proc GetColorForGUI { base w col_type} {
    global GidPriv

    if { [ string eq $col_type "Background"] } {
	
	set org_col $GidPriv(Original,Color,Background)
	set retcol [ GIDChooseColor $w.selectcolor -title [_ "Select color"] -color $org_col]
	
	if { ![ string eq $retcol ""] } {
	
	    foreach wid [ concat [ winfo children $base] $base [ winfo children $w]] {
		WarnWinText "[winfo class $wid] $wid conf -bg $retcol"
		set bg_entries [ CCColorActivo $retcol 65]
		switch [ winfo class $wid] {
		    ComboBox {
		        $wid conf -background $retcol
		        $wid conf -entrybg $bg_entries
		    }
		    TCombobox {
		        $wid conf -background $retcol
		        $wid conf -entrybg $bg_entries
		    }
		    Entry {
		        $wid conf -bg $bg_entries
		    }
		    Scale {
		        $wid conf -troughcolor $bg_entries
		        $wid conf -bg $retcol
		    }
		
		    ScrollBar {
		        $wid conf -troughcolor $bg_entries
		        $wid conf -bg $retcol
		    }
		    default {
		        # Frame, Button, Checkbutton, NoteBook, Scale, Label
		        $wid conf -bg $retcol
			# WarnWin [winfo class $wid]
		    }
		}
	    }
	    #[ winfo parent $base] conf -bg $retcol
	} else {
	    ChangeBgWindows $w
	}
    }
}
