# blue.tcl - Copyright (C) 2004 Pat Thoyts <patthoyts@users.sourceforge.net>
#
# blue.tcl,v 1.31 2006/08/04 12:05:43 patthoyts Exp
#
#

proc ttk::LoadImages {imgdir {patterns {*.gif}}} {
    foreach pattern $patterns {
	foreach file [glob -directory $imgdir $pattern] {
	    set img [file tail [file rootname $file]]
	    if {![info exists images($img)]} {
		set images($img) [image create photo -file $file]
	    }
	}
    }
    return [array get images]
}

namespace eval ttk::theme::blue {

    package provide ttk::theme::blue 0.8

    variable I
    array set I [ttk::LoadImages \
                     [file join [file dirname [info script]] blue] *.gif]
    
    variable colors
    array set colors {
	-frame  	"#6699cc"
	-lighter	"#bcd2e8"
	-window	 	"#e6f3ff"
	-selectbg	"#ffff33"
	-selectfg	"#000000"
	-disabledfg	"#666666"
    }

    ttk::style theme create blue -settings {

	ttk::style configure . \
	    -borderwidth 	1 \
	    -background 	$colors(-frame) \
	    -fieldbackground	$colors(-window) \
	    -troughcolor	$colors(-lighter) \
	    -selectbackground	$colors(-selectbg) \
	    -selectforeground	$colors(-selectfg) \
	    ;
	ttk::style map . -foreground [list disabled $colors(-disabledfg)]

	## Buttons.
	#
	ttk::style configure TButton -padding "10 0"
	ttk::style layout TButton {
	    Button.button -children {
		Button.focus -children {
		    Button.padding -children {
			Button.label
		    }
		}
	    }
	}

	ttk::style element create button image $I(button-n) \
	    -border 4 -sticky ew
	ttk::style map button -image [list pressed $I(button-p) active $I(button-h)]
	# ttk::style configure TButton -image $I(button-n) \
	#     -border 4 -sticky ew
	# ttk::style map TButton -image [list pressed $I(button-p) active $I(button-h)]

	ttk::style element create Checkbutton.indicator image $I(check-nu) \
	    -width 24 -sticky w 

	ttk::style map Checkbutton.indicator -image [list \
		{!disabled active selected} $I(check-hc) \
		{!disabled active} $I(check-hu) \
		{!disabled selected} $I(check-nc) ]

	ttk::style element create Radiobutton.indicator image $I(radio-nu) \
	    -width 24 -sticky w \

	ttk::style map Radiobutton.indicator -image [list \
		{!disabled active selected} $I(radio-hc) \
		{!disabled active} $I(radio-hu) \
		selected $I(radio-nc) ]

	ttk::style configure TMenubutton -relief raised -padding {10 2}
	ttk::style configure TRadiobutton -padding 1
	ttk::style configure TCheckbutton -padding 1

	## Toolbar buttons.
	#
	ttk::style configure Toolbutton \
	    -width 0 -relief flat -borderwidth 2 -padding 4 \
	    -background $colors(-frame) -foreground #000000 ;
	ttk::style map Toolbutton -background [list active $colors(-selectbg)] 
	ttk::style map Toolbutton -foreground [list active $colors(-selectfg)] 
	ttk::style map Toolbutton -relief {
	    disabled 	flat
	    selected	sunken  
	    pressed 	sunken  
	    active  	raised
	}

	## Entry widgets.
	#
	ttk::style configure TEntry \
	    -selectborderwidth 1 -padding 2 -insertwidth 2 -font TkTextFont
	ttk::style configure TCombobox \
	    -selectborderwidth 1 -padding 2 -insertwidth 2 -font TkTextFont

	## Notebooks.
	#
        ttk::style element create tab image $I(tab-n) -border {2 2 2 0} 
	ttk::style map tab -image [list selected $I(tab-h) active $I(tab-h)]
        ttk::style configure TNotebook -padding 2 -tabmargins {2 2 2 0}
        ttk::style map TNotebook.Tab -expand [list active {2 0 2 0} selected {2 0 2 0}]

	## Labelframes.
	#
	ttk::style configure TLabelframe -borderwidth 2 -relief groove

	## Scrollbars.
	#
	ttk::style layout Vertical.TScrollbar {
	    Scrollbar.trough -children {
		Scrollbar.uparrow -side top
		Scrollbar.downarrow -side bottom
		Scrollbar.uparrow -side bottom
		Vertical.Scrollbar.thumb -side top -expand true -sticky ns
	    }
	}

	ttk::style layout Horizontal.TScrollbar {
	    Scrollbar.trough -children {
		Scrollbar.leftarrow -side left
		Scrollbar.rightarrow -side right
		Scrollbar.leftarrow -side right
		Horizontal.Scrollbar.thumb -side left -expand true -sticky we
	    }
	}

	ttk::style element create Horizontal.Scrollbar.thumb image $I(sb-thumb) \
	    -border 3
	ttk::style map Horizontal.Scrollbar.thumb -image \
	    [list {pressed !disabled} $I(sb-thumb-p)]

	ttk::style element create Vertical.Scrollbar.thumb image $I(sb-vthumb) \
	    -border 3
	ttk::style map Vertical.Scrollbar.thumb \
	    -image [list {pressed !disabled} $I(sb-vthumb-p)]

	foreach dir {up down left right} {
	    ttk::style element create ${dir}arrow image $I(arrow${dir}) \
	        -border 1 -sticky {}
	    ttk::style map ${dir}arrow -image [list \
		    disabled $I(arrow${dir}) \
		    pressed $I(arrow${dir}-p) \
		    active $I(arrow${dir}-h)] \
	}

	## Scales.
	#
	ttk::style element create Scale.slider \
	    image $I(slider) 
	ttk::style map Scale.slider -image [list {pressed !disabled} $I(slider-p)]

	ttk::style element create Vertical.Scale.slider \
	    image $I(vslider) 
	ttk::style map Vertical.Scale.slider \
	    -image [list {pressed !disabled} $I(vslider-p)]

        ## Progressbar
        #
        ttk::style element create Horizontal.Progressbar.pbar image $I(sb-thumb) \
	    -border 2
	ttk::style map Horizontal.Progressbar.pbar -image \
	    [list {pressed !disabled} $I(sb-thumb-p)] 
        ttk::style element create Vertical.Progressbar.pbar image $I(sb-vthumb) \
	    -border 2
	ttk::style map Vertical.Progressbar.pbar -image \
            [list {pressed !disabled} $I(sb-vthumb-p)] 

        ## Statusbar parts
        #
        ttk::style element create sizegrip image $I(sizegrip)

        ## Paned window parts.
        #
        ttk::style element create hsash image $I(hseparator-n) -border {2 0} 
	ttk::style map hsash -image \
            [list {active !disabled} $I(hseparator-a)]
        ttk::style element create vsash image $I(vseparator-n) -border {0 2} 
	ttk::style map vshah -image \
            [list {active !disabled} $I(vseparator-a)]
    }
}

